#========================================================================================
# package      Makefile
# state        private
#----------------------------------------------------------------------------------------
# description  see http://ep-support.jena.epages.de:8080/wiki/index.php/Makefile.PL
#----------------------------------------------------------------------------------------
# example      require DE_EPAGES::Core::API::Makefile;
#               __DATA__
#
#               !OS_SPECIFIC_DEFINES
#
#
#               install:
#                   $(COPY) SourceDir TargetDir
# example      perl Makefile.PL
# example      perl Makefile.PL PERL=/usr/bin/perl
#========================================================================================
use strict;
use Config;
use Date::Format qw(time2str);
use File::Basename  qw ( dirname );

$DE_EPAGES::Core::API::Makefile::VERSION = '6.11.0';

# define here operating system specific defines:

## no critic qw(RequireInterpolationOfMetachars)
my %GENERIC_VARS = (
    'ARCHNAME'  => "x86_64-$Config{osname}",
    'ARCHNAME32' => "i586-$Config{osname}",
);
## use critic

foreach my $var (qw( EPAGES EPAGES_CARTRIDGES EPAGES_SHARED EPAGES_CONFIG EPAGES_LOG
             EPAGES_STATIC EPAGES_STORES EPAGES_WEBROOT EPAGES_PERL EPAGES_APACHE
             HTTPD_ROOT )) {
    next unless defined $ENV{$var};
    $ENV{$var} =~ s,[/\\]+$,,;
    $GENERIC_VARS{$var} = $ENV{$var};
}

my %OS_SPECIFIC_VARS;

if ($Config{osname} eq 'MSWin32') {

    %OS_SPECIFIC_VARS = (
        %GENERIC_VARS,
        'AWK'     => 'gawk',
        'CC'      => 'cl -nologo',
        'CD'      => 'cd /d',
        'CHMOD'   => 'REM chmod',
        'CHOWN'   => 'REM chown',
        'COPY'    => 'xcopy /R /Q /Y /S /E /I',
        'ECHONL'  => 'echo.',
        'MAKE'    => 'nmake -nologo',
        'NUL'     => 'nul',
        'PERL'    => 'perl',
        'TAR'     => 'tar',
        'GTAR'    => "$ENV{EPAGES}\\bin\\tar.exe",
        'GZIP_EXE' => "$ENV{EPAGES}\\bin\\gzip.exe",
        'WINDOWS' => '1',
        'SEP_ROW' => '"~~~\r\n"', ## no critic qw( RequireInterpolationOfMetachars )
        'HHC'     => 'hhc',
        'CD'      => 'cd /d',
	'CAT'       => '$(PERL) -MExtUtils::Command -e cat',
	'CP'        => '$(PERL) -MExtUtils::Command -e cp',
	'COPY_DIR'  => '$(PERL) -MDE_EPAGES::Core::API::Command -e copy_dir',
	'EMPTY_DIR' => '$(PERL) -MDE_EPAGES::Core::API::Command -e empty_dir',
	'COPY_FILE' => '$(PERL) -MDE_EPAGES::Core::API::Command -e copy_file',
	'DEL'       => '$(PERL) -MExtUtils::Command -e rm_f',
	'EQTIME'    => '$(PERL) -MExtUtils::Command -e eqtime',
	'MOVE'      => '$(PERL) -MExtUtils::Command -e mv',
	'MV'        => '$(PERL) -MExtUtils::Command -e mv',
	'MKDIR'     => '$(PERL) -MExtUtils::Command -e mkpath',
	'MKPATH'    => '$(PERL) -MExtUtils::Command -e mkpath',
	'RMDIR'     => '$(PERL) -MDE_EPAGES::Core::API::Command -e delete_dir',
	'TEST'      => '$(PERL) -MDE_EPAGES::Core::API::Command=test -e test',
	'TEST_F'    => '$(PERL) -MExtUtils::Command -e test_f',
	'TEXT2DOS'  => '$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Core/Scripts/Text2Dos.pl',
	'TEXT2UNIX' => '$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Core/Scripts/Text2Unix.pl',
	'TOUCH'     => '$(PERL) -MExtUtils::Command -e touch',
    );
    
} else {

    my $EtcDir = dirname($ENV{EPAGES}). '/etc';
    $EtcDir = '/etc' unless -d "$EtcDir/init.d";
    my $EpagesInitd = '/etc/init.d/$(EPAGES_PROJECT)';  ## no critic qw( RequireInterpolationOfMetachars )

    %OS_SPECIFIC_VARS = (
        %GENERIC_VARS,
        'AWK'     => 'awk',
        'CD'      => 'cd',
        'COPY'    => 'cp -rf',
        'CHMOD'   => 'chmod',
        'CHOWN'   => 'chown',
        'ECHONL'  => 'echo',
        'MAKEDIR' => '$(CURDIR)',  ## no critic qw( RequireInterpolationOfMetachars )
        'NUL'     => '/dev/null',
        'PERL'    => "$ENV{PERL}",
        'TAR'     => 'tar',
        'GTAR'    => 'tar',
        'GZIP_EXE' => 'gzip',
        'SEP_ROW' => '\'~~~\\n\'',
        'CD'      => 'cd',
        'ETC_DIR' => $EtcDir,
        'ETC_INITD' => $EpagesInitd,
        'SU_EXE'  => system('runuser -c true 2>/dev/null') ? 'su' : 'runuser',
	'CAT'       => 'cat',
	'CP'        => 'cp -rp',
	'COPY_DIR'  => 'cp -rp',
	'EMPTY_DIR' => 'rm -rf',
	'COPY_FILE' => 'cp -p',
	'DEL'       => 'rm -rf',
	'MOVE'      => 'mv',
	'MV'        => 'mv',
	'MKDIR'     => 'mkdir -m 755 -p',
	'MKPATH'    => 'mkdir -m 755 -p',
	'RMDIR'     => 'rm -rf',
	'TEST'      => 'test',
	'TEST_F'    => 'test -f',
	'TEXT2DOS'  => 'unix2dos',
	'TEXT2UNIX' => 'dos2unix',
	'TOUCH'     => 'touch',
    );

    my $shell;
    $shell = `/usr/bin/which ksh 2>/dev/null` or $shell = `/usr/bin/which sh 2>/dev/null` ;
    $OS_SPECIFIC_VARS{SHELL} = $shell if $shell =~ /\S/;
}

sub Main {

    my $MakefileName = 'makefile';
    open my $MAKEFILE, '>', $MakefileName or die "open() failed: $!"; ##no critic qw(RequireBriefOpen)
    my $Date = time2str('%D', localtime);
    my $OsSpecificDefines = "CREATED_ON\t= $Date\nOSNAME\t\t= $Config{osname}\nOSVERS\t\t= $Config{osvers}\n\n";

    # allow overrides at configure time (perl Makefile.pm)
    my %Override = map { /(.+?)=(.*)/ } grep { /.+=/ } @ARGV;
    $OS_SPECIFIC_VARS{$_} = $Override{$_} foreach keys %Override;

    $OsSpecificDefines .= "$_\t= $OS_SPECIFIC_VARS{$_}\n" foreach sort keys %OS_SPECIFIC_VARS;
    if ($Config{osname} eq 'MSWin32') {

        while (<DATA>) {

            next if /^\!END_PATH\s*$/;
            next if /^\!BEGIN_PATH\s*$/;
            next if /^\!BEGIN_VERBATIM\s*$/;
            next if /^\!END_VERBATIM\s*$/;
            # beware! for '!if' we use GNUmake notation
            # use GNUmake notation: !ifeq ("$(OSNAME)", "linux")
            # instead of nmake notation: !if ("$(OSNAME)" == "linux")
            s/^(!if)eq([^,]*),/$1$2==/gx;
            s/^(!if)neq([^,]*),/$1$2!=/gx;
            s/^!OS_SPECIFIC_DEFINES/$OsSpecificDefines/;
            print $MAKEFILE $_;
        }

    } else {

        my  @Data = <DATA>;
        my $HasPathDirective = scalar grep { /^\!BEGIN_PATH\s*$/ } @Data;
        my $ChangeBackSlashes = $HasPathDirective ? 0 : 1;
        foreach (@Data) {
            if (/^\!END_PATH\s*$/) {
                $ChangeBackSlashes = 0;
                next;
            } elsif (/^\!BEGIN_PATH\s*$/) {
                $ChangeBackSlashes = 1;
                next;
            } elsif (/^\!BEGIN_VERBATIM\s*$/) {
                $ChangeBackSlashes = 0;
                next;
            } elsif (/^\!END_VERBATIM\s*$/) {
                $ChangeBackSlashes = 1;
                next;
            }
            s,\r$,,;
            s,\\(.),/$1,g if $ChangeBackSlashes;
            s/^!(include|ifn?def|ifn?eq|else|endif)/$1/;
            s/^!OS_SPECIFIC_DEFINES/$OsSpecificDefines.EXPORT_ALL_VARIABLES:\n/;
            print $MAKEFILE $_;
        }

    }
    close $MAKEFILE or die "close() failed: $!";
    chown((getpwnam 'ep_appl')[2], (getgrnam 'epages')[2], $MakefileName);
    chmod(0644, $MakefileName);
}

Main();

1;
