#========================================================================================
# package      DE_EPAGES::Ebay::API::Ebay
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the main ebay functions.
#========================================================================================
package DE_EPAGES::Ebay::API::Ebay;
use base Exporter;
use strict;

our @EXPORT_OK = qw ( UpdateCategories
                      UpdateAuctions
                      UpdateShopAuctions
                      UploadScheduledAuctions
                      UploadScheduledShopAuctions
                      GetCustomerByEbayUserName
                      ResetAuctionCount
                      CheckTokenExpiryDate
                      UpdateTransactionList
                      UpdateRunningAuctions
                      TransactionToOrder
                      CheckProviderTokenExpiryDate );

use DateTime::Duration;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError HandleError );
use DE_EPAGES::Core::API::File qw ( OpenFile MoveFile WriteFile GetFileContentLines SetAccessRights );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug LogWarnings );
use DE_EPAGES::Core::API::PerlTools qw ( trim unique AddDuration GetCurrentDateTime GetLocalTimeZone );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Ebay::API::Cartridge;
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Mail qw ( SendTokenErrorMail );
use DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use DE_EPAGES::Ebay::API::Table::EbayCustomerMap qw (
    ExistsEbayCustomerMap
    GetInfoEbayCustomerMap
    InsertEbayCustomerMap
);
use DE_EPAGES::Ebay::API::Tools qw (
    ConvertFromEbayDate
    GetOldestStartTime
    SetNewEbayAuctionsFeatureValue
);
use DE_EPAGES::Ebay::DAL::Ebay qw (
    DAL_GetPKeyRunningEbayEnviroments
    DAL_GetPKeyShopsByEbayAuctionCount
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObject
    LoadClassByAlias
    LoadObject
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Table::Country qw ( GetPKeyCountryByCode2 );
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithoutContainer );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_NO_ADDTOBASKET STOCK_NOT_VISIBLE );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

#========================================================================================
# function     UpdateAuctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateAuctions( $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates Auctions from eBay<br>
#               (used by scripts).
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UpdateAuctions {
    my ($printOutput) = @_;

    my $NOW = GetCurrentDBHandle()->currentDateTime;

    my $aEnvironmentIDs = [];
    my $IDsFile = $ENV{EPAGES_IDS_FILE};

    if ($IDsFile and -f $IDsFile) {
        $aEnvironmentIDs = [ trim @{GetFileContentLines($IDsFile)} ];
    }
    $aEnvironmentIDs = [ unique @$aEnvironmentIDs, @{DAL_GetPKeyRunningEbayEnviroments()} ];
    if ($IDsFile) {
        my $Content = join("\n", @$aEnvironmentIDs);
        WriteFile($IDsFile, \$Content, { create_directory=>1 });
    }

    foreach my $ShopEnvID (@$aEnvironmentIDs) {
        my $time1 = time;

        # be sure that the ShopEnv (and its Shop) still exists
        unless (ExistsObject($ShopEnvID)) {
            _dropLine($IDsFile, $ShopEnvID);
            next;
        }

        my $ShopEnv = LoadObject($ShopEnvID);

        eval {
            UpdateShopEnvironmentAuctions( $ShopEnv, $printOutput );
        };
        if (ExistsError()) {
            my $Error = GetError();

            # collect all errors, but let the script continue
            Warning($Error);
        }
        _dropLine($IDsFile, $ShopEnvID);

        my $seconds = time - $time1;
        my $Shop = $ShopEnv->getSite();

        my $output = sprintf ("... Shop: %s (id:%s, ebay-id:%s) took %d seconds\n", $Shop->alias, $Shop->id, $ShopEnvID, $seconds);
        GetLog->debug($output);
        print $output if $printOutput
    }

    # set a global LastUpdate flag
    LoadRootObject()->set({'EbayLastUpdate' => $NOW});
}

#========================================================================================
# function     UpdateShopAuctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateShopAuctions( $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates all Auctions of a Shop (get highest bid etc.).
#----------------------------------------------------------------------------------------
# input        $Shop | shop | Shop object
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UpdateShopAuctions {
    my ($Shop, $printOutput ) = @_;

    foreach my $ShopEnv (@{$Shop->get('EbayShopEnvironments')}) {
        UpdateShopEnvironmentAuctions($ShopEnv, $printOutput);
    }
}

#========================================================================================
# function     UpdateShopEnvironmentAuctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateShopEnvironmentAuctions($ShopEnv, $printOutput);
#----------------------------------------------------------------------------------------
# description  Updates all Auctions of a eBay shop env (get highest bid etc.).
#----------------------------------------------------------------------------------------
# input        $ShopEnv | shop environment | EbayShopEnvironment object
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UpdateShopEnvironmentAuctions {
    my ($ShopEnv, $printOutput ) = @_;

    my $NOW = GetCurrentDBHandle()->currentDateTime;

    # first: search for running auctions
    my $SearchName = 'MBO-EbayAuction';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('EbayAuction'), $SearchName);
    my $Shop = $ShopEnv->getSite();
    my $Folder = $Shop->folder('EbayAuctions');
    my %Options = (
        'ObjectID' => $Folder,
        'PageSize' => 10000,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
            'Running'   => 1,
            'EbayShopEnvironment' => $ShopEnv->id,
        },
    );
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    $Search->statistic;
    my $RunningAuctionCount = $Search->countItems;

    # second: check for incomplete checkouts
    $SearchDefinition = GetSearchDefinition(LoadClassByAlias('EbayTransaction'), 'MBO-EbayOrders');
    $Folder = $Shop->folder('EbayTransactions');
    %Options = (
        'ObjectID' => $Folder,
        'PageSize' => 10000,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'             => $Shop->id,
            'IsEbayComplete'      => 0,
            'EbayShopEnvironment' => $ShopEnv->id,
        },
    );
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    $Search->statistic;
    my $IncompleteCheckoutTransactionCount = $Search->countItems;

    if ( 0 == $RunningAuctionCount && 0 == $IncompleteCheckoutTransactionCount ) {
        # nothing to do means "Update is ok"
        $ShopEnv->set({'LastUpdate' => $NOW});
        return;
    }

    my $output = sprintf "Updating Shop: %s (%d) ShopEnvironment: %s EbayEnvironment: %s (%d running, %d incomplete checkout)\n",
                            $Shop->alias,
                            $Shop->id,
                            $ShopEnv->get('NameOrAlias'),
                            $ShopEnv->get('EbayEnvironment')->alias,
                            $RunningAuctionCount,
                            $IncompleteCheckoutTransactionCount;
    GetLog->debug($output);
    print $output if ($printOutput);

    # want to show only real errors occured while ebay communication as warnings in the UI/template
    # so clean warnings stack (log them and forget)
    LogWarnings(BlockWarnings( sub {
        UpdateRunningAuctions( $ShopEnv, $printOutput ); # if ($RunningAuctionCount > 0);
        UpdateTransactions( $ShopEnv, $printOutput ); # if ($IncompleteCheckoutTransactionCount > 0);
    }));

    $ShopEnv->set({'LastUpdate' => $NOW});

    return;
}

#========================================================================================
# function     GetCustomerByEbayUserName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $customer = GetCustomerByEbayUserName( $ShopEnv, $EbayUserName );
#----------------------------------------------------------------------------------------
# description  Gets customer object from a eBay username.
#----------------------------------------------------------------------------------------
# input        $ShopEnv | shop environment of transaction/user | EbayShopEnvironment object
# input        $EbayUserName | username (login at eBay) | String
# return       $customer | customer to this EbayUserName or undef if no customer was found | Customer object
#========================================================================================
sub GetCustomerByEbayUserName {
    my ( $ShopEnv, $EbayUserName ) = @_;

    return undef unless (ExistsEbayCustomerMap($ShopEnv->id, $EbayUserName));
    my $hCustomer = GetInfoEbayCustomerMap($ShopEnv->id, $EbayUserName);
    return LoadObject($hCustomer->{'CustomerID'});
}

#========================================================================================
# function     UploadScheduledAuctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UploadScheduledAuctions( $printOutput );
#----------------------------------------------------------------------------------------
# description  Sends scheduled Auctions to eBay<br>
#               (used by scripts).
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UploadScheduledAuctions {
    my ($printOutput) = @_;

    # get all auction with status "scheduled"
    my $aAuctions = LoadClassByAlias('EbayAuction')->getObjects('ByStatus', [AUCTION_SCHEDULED]);

    foreach my $auction (@$aAuctions) {

        my $scheduleTime = $auction->get('ScheduleTime');

        # fix inconsistency. EPG-13288
        if (not defined $scheduleTime) {
            $auction->set({'Status' => AUCTION_EDITABLE });
            next;
        }

        # check if scheduleTime has come - compared in floating timezone
        next if $scheduleTime > GetCurrentDBHandle()->currentDateTime;

        # check if product is in stock and can be auctioned
        my $Product = $auction->get('Product');
        if (not defined $Product) {
            $auction->set({'Status' => AUCTION_FAILED});
            next;
        }

        my $ProductLowOnStockAction = $auction->getSite()->get('ProductLowOnStockAction');
        next if ($Product->get('StockAlert') eq 'OUT_STOCK' && ($ProductLowOnStockAction == STOCK_NO_ADDTOBASKET || $ProductLowOnStockAction == STOCK_NOT_VISIBLE));

        my $output = sprintf("Uploading Auction: %s(%s)\n", $auction->get('NameOrAlias'), $auction->id);
        GetLog()->debug($output);
        print $output if $printOutput;

        my $result;
        eval {
            $result = $auction->upload;
        };

        if (ExistsError()) {
            my $Error = GetError();
            $auction->set({'Status' => AUCTION_EDITABLE});

            if ($Error->code eq 'EbayRequestFailed') {
                print ' Request Failed: '.$Error->message."\n" if $printOutput;
                GetLog()->debug('Request Failed: '.$Error->message);
            } elsif ($Error->code eq 'EbayRequestNotSuccessful') {
                foreach my $error (@{$Error->{'Vars'}->{'EbayErrors'}}) {
                    print ' Request Not Successful: '.$error->{'LongMessage'}."\n" if $printOutput;
                    GetLog()->debug('Request Not Successful: '.$error->{'LongMessage'});
                }
            } else {
                Error($Error);
            }
        } else {
            my $itemId = $result->{'ItemID'};

            # set status
            $auction->set({'Status' => AUCTION_RUNNING,
                           'ExternalAuctionId' => $itemId});
        }
    }
}

#========================================================================================
# function     UpdateCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateCategories( $EnvironmentAlias, $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates Categories from eBay<br>
#               (used by scripts).
#----------------------------------------------------------------------------------------
# input        $EnvironmentAlias | identifier of an environment | string
# input        $printOutput | print output to console | boolean
# input        $Force | force update, even when IsUpdating flag is set | boolean
#========================================================================================
sub UpdateCategories {
    my ($EnvironmentAlias, $printOutput, $Force) = @_;
    TestString('EnvironmentAlias', $EnvironmentAlias);

    my $Environment = LoadRootObject()->child('EbayEnvironments')->child($EnvironmentAlias);

    eval {
        # Test if category db is working correctly.
        my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($Environment);
        $EbayCategoryTree->getInfoRootCategories();
    };
    if (ExistsError()) {
        _DropCategoriesDB(EBAY_CATEGORIES_TMP_DB_NAME, EBAY_CATEGORIES_TMP_DB_FILENAME);
        _UpdateCategoryDB(
            $EnvironmentAlias,
            $printOutput,
            $Force,
            EBAY_CATEGORIES_TMP_DB_NAME,
            EBAY_CATEGORIES_TMP_DB_FILENAME
        );
        _ReplaceCategoryDB(EBAY_CATEGORIES_DB_FILENAME, EBAY_CATEGORIES_TMP_DB_FILENAME);
    }
    else {
        _UpdateCategoryDB(
            $EnvironmentAlias,
            $printOutput,
            $Force,
            EBAY_CATEGORIES_DB_NAME,
            EBAY_CATEGORIES_DB_FILENAME)
        ;
    }

    return;
}

sub _UpdateCategoryDB {
    my ($EnvironmentAlias, $printOutput, $Force, $DBName, $DBFilename) = @_;

    my $Environment = LoadRootObject()->child('EbayEnvironments')->child($EnvironmentAlias);
    GetLog()->debug('Starting update of category list ('.$Environment->get('Name').')...');

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'Environment'   => $Environment,
    );

    my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($Environment, $DBName);

    if (!$Force && $EbayCategoryTree->getCategoryListIsUpdating) {
        print "Environment $EnvironmentAlias is in the process of being updated.\n" if $printOutput;
        GetLog()->debug("Environment $EnvironmentAlias is in the process of being updated.");
        return;
    }

    # test if we vave to update the category tree
    my $hCheckResult;
    eval {
        $EbayCategoryTree->setCategoryListIsUpdating(1);
        $hCheckResult = $RemoteEbay->getCategories('1');
    };
    if (ExistsError()) {
        my $Error = GetError();
        $EbayCategoryTree->setCategoryListIsUpdating(0);
        eval { SynchronizeCache(); };
        Warning( GetError() ) if ExistsError();
        Error($Error);
    }

    my $lastVersion = $EbayCategoryTree->getCategoryListVersion();

    if ($hCheckResult->{'CategoryVersion'} <= $lastVersion) {
        print "Version (v.$lastVersion) of environment $EnvironmentAlias is the latest. No update needed.\n" if $printOutput;
        $EbayCategoryTree->setCategoryListUpdateTime(GetCurrentDateTime());
        $EbayCategoryTree->setCategoryListIsUpdating(0);
        GetLog()->debug("Update of category list: No update needed, since your version (v.$lastVersion) of environment $EnvironmentAlias is the latest.");

        # update the category characteristics mappings
        # $EbayCategoryTree->updateCSMappings($printOutput);

        return;
    }

    # update the categories here...
    eval {
        $EbayCategoryTree->setCategoryListCount(0);
        $EbayCategoryTree->update($printOutput);
        #$EbayCategoryTree->deleteAllAttributeSets();
        #$EbayCategoryTree->getAllCSMappings();
        #$EbayCategoryTree->updateCSMappings($printOutput);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $EbayCategoryTree->setCategoryListIsUpdating(0);
        GetLog()->debug('Request Failed: '.$Error->message);
        # collect all errors, but let the script continue
        Error($Error);
    }

    GetLog()->debug('Update of category list successfully completed.');

    return;
}

sub _DropCategoriesDB {
    my ($DBName, $DBFilename) = @_;

    # Replacing the content of the file with nothing to reset it.
    OpenFile( '>', $DBFilename, sub {
            my ($FileHandle) = @_;
            print $FileHandle '';
        }
    );
    SetAccessRights($DBFilename);

    my $Cartridge = DE_EPAGES::Ebay::API::Cartridge->new();
    $Cartridge->installEbayCategories($DBName, $DBFilename, 1);

    return;
}

sub _ReplaceCategoryDB {
    my ($OriginalDBFilename, $ReplacementDBFilename) = @_;

    MoveFile($ReplacementDBFilename, $OriginalDBFilename);

    return;
}

#========================================================================================
# function     UpdateRunningAuctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateRunningAuctions(ShopEnv, $printOutput);
#----------------------------------------------------------------------------------------
# description  Updates all running auctions of an shop environment
#               (get highest bid etc.).
#----------------------------------------------------------------------------------------
# input        $ShopEnv | shop environment | EbayShopEnvironment object
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UpdateRunningAuctions {
    my ( $ShopEnv, $printOutput ) = @_;
    GetLog->debug('UpdateRunningAuctions ...');

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'ShopEnvironment'   => $ShopEnv,
    );

    # get seller items
    my $oldestStartTime = GetOldestStartTime( $ShopEnv );
    my $aItemList = $RemoteEbay->getSellerList($oldestStartTime);

    GetLog->debug('GetSellerList returned '.scalar @$aItemList.' auctions.');

    # if there is not one single auction, then there is nothing to do
    return unless @$aItemList;

    my $ListingStatusMap = EBAY_LISTINGSTATUS();
    foreach my $item ( @$aItemList ) {
        my $ExternalAuctionId = $item->{'ItemID'};

        # is it an epages ebay auction
        my $aAuctions = LoadClassByAlias('EbayAuction')->getObjects('ByEbayShopEnvironmentIDExternalAuctionId',[$ShopEnv->id, $ExternalAuctionId]);

        next unless scalar @$aAuctions;

        my $auction = $aAuctions->[0];
        my $BidCount = $item->{'SellingStatus'}->{
            $auction->get('ListingType') eq 'FixedPriceItem' ? 'QuantitySold' : 'BidCount'
        };
        my $params = {
            'CurrentBid'    => $BidCount ? $item->{'SellingStatus'}->{'CurrentPrice'} : 0,
            'EndTime'       => ConvertFromEbayDate($item->{'ListingDetails'}->{'EndTime'}),
            'BidCount'      => $BidCount,
            'Status'        => $ListingStatusMap->{$item->{'SellingStatus'}->{'ListingStatus'}},
        };

        if ($params->{'Status'} == AUCTION_COMPLETED and $auction->get('Status') != AUCTION_COMPLETED) {
            # stock handling
            my $product = $auction->get('Product');
            my $decreaseType = $auction->getSite()->get('EbayStockDecreaseType') || '';
            if (defined $product && $decreaseType eq 'OnUpload') {
                # stock already decreased. Add non-sold items back to stock
                if($product->get('IsMaster')){
                    foreach my $subAuction ( @{$auction->get('SubAuctions')} ) {
                        my $subProduct = $subAuction->get('Product');
                        my $diff = $subAuction->get('Quantity') - $subAuction->get('QuantitySold') // 0;
                        $subProduct->addBundleToStock($diff) if $diff and defined $subProduct;
                    }
                } else {
                    my $diff = $auction->get('Quantity') - $item->{'SellingStatus'}->{'QuantitySold'};
                    $product->addBundleToStock($diff) if $diff;
                }
            }
        }

        $auction->set($params);
    }
    GetLog->debug('done.');
}

#========================================================================================
# function     UpdateTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateTransactions(ShopEnv, $printOutput);
#----------------------------------------------------------------------------------------
# description  Updates all transactions of running and shortly ended auctions
#               of an shop environment. May create customers and send fulfillment mail.
#----------------------------------------------------------------------------------------
# input        $ShopEnv | shop environment | EbayShopEnvironment object
# input        $printOutput | print output to console | boolean
#========================================================================================
sub UpdateTransactions {
    my ( $ShopEnv, $printOutput ) = @_;
    GetLog->debug('UpdateTransactions ...');

    my $LanguageID = GetPKeyLanguageByCode(lc $ShopEnv->get('EbayEnvironment')->get('LanguageID'));
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'ShopEnvironment'   => $ShopEnv,
    );

    # use the creation date if no LastUpdate is available (prevent getting all transactions for the account)
    my $lastUpdate = $ShopEnv->get('LastUpdate') ? $ShopEnv->get('LastUpdate') : $ShopEnv->get('CreationDate') ;

    # lastUpdate is local time
    eval {
        $lastUpdate->set_time_zone(GetLocalTimeZone());
    };
    if (ExistsError()) {
        my $Error = GetError();
        # EPG-18092: probably invalid date caused by daylight saving time!
        # add one hour and try again.
        Error($Error) unless ($Error->shortMessage =~ /Invalid local time for date/);
        $lastUpdate->set_time_zone('floating');
        $lastUpdate = AddDuration($lastUpdate, 'hours', 1 );
        $lastUpdate->set_time_zone(GetLocalTimeZone());
    }
    # eBay need UTC
    my $lastUpdateUTC = $lastUpdate->clone();
    $lastUpdateUTC->set_time_zone('UTC');
    my $duration = DateTime::Duration->new( hours  => 1 );

    # get transactions
    my $aTransactions = $RemoteEbay->getSellerTransactions($lastUpdateUTC-$duration);
    UpdateTransactionList($ShopEnv, $aTransactions, $printOutput);
}

#========================================================================================
# function     UpdateTransactionList
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateTransactionList( $ShopEnv, $aTransactions, $printOutput );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $ShopEnv        | Ebay ShopEnvironment | object
# input        $aTransactions  | result of getSellerTransactions | array.hash.ref
# input        $printOutput    | true if printed output wanted | boolean
#========================================================================================
sub UpdateTransactionList {
    my ( $ShopEnv, $aTransactions, $printOutput ) = @_;

    # if there is not one single transaction, then there is nothing to do
    return unless @$aTransactions;

    my $EbayStockDecreaseType = $ShopEnv->getSite()->get('EbayStockDecreaseType');

    foreach my $hTransaction ( @$aTransactions ) {
        my $ExternalAuctionId = $hTransaction->{'Item'}->{'ItemID'};

        # is it an epages ebay auction

        my $aEbayAuctions = LoadClassByAlias('EbayAuction')->getObjects('ByEbayShopEnvironmentIDExternalAuctionId', [$ShopEnv->id, $ExternalAuctionId]);
        next unless scalar @$aEbayAuctions;

        my $auction = $aEbayAuctions->[0];

        my $aTransactionObjects = LoadClassByAlias('EbayTransaction')->getObjects('ByEbayAuctionIDExternalTransactionId', [$auction->id, $hTransaction->{'TransactionID'}] );

        # try to get a customer from eBay username
        my $customer = GetCustomerByEbayUserName($ShopEnv, $hTransaction->{'Buyer'}->{'UserID'});

        my $transaction;

        my $IsComplete = 0;
        if ( $hTransaction->{'Status'}->{'CheckoutStatus'} eq 'CheckoutComplete'
          or $hTransaction->{'Status'}->{'BuyerSelectedShipping'} eq 'true'
          or $hTransaction->{'Status'}->{'CompleteStatus'} eq 'Complete') {
             $IsComplete = 1;
        }

        if (scalar @$aTransactionObjects) {
            $transaction = $aTransactionObjects->[0];

            # do nothing if transaction is already completed
            next if $transaction->get('IsEbayComplete');

            # update transaction
            $transaction->set({
                'IsEbayComplete'        => $IsComplete,
                'Price'                 => $hTransaction->{'TransactionPrice'},
                'LastMod'               => ConvertFromEbayDate($hTransaction->{'Status'}->{'LastTimeModified'}),
            });

        } else {
            print ' Creating Transaction for Auction: '.$auction->get('ExternalAuctionId')."\n" if $printOutput;

            # create new transaction
            my $folder = $auction->folder('EbayTransactions');

            $transaction = $folder->insertChild('EbayTransaction', {
                'Alias'                 => $folder->nextAlias('EbayTransaction%s'),
                'Customer'              => $customer,
                'CustomerOrder'         => undef,
                'ExternalTransactionId' => $hTransaction->{'TransactionID'},
                'IsEbayComplete'        => $IsComplete,
                'EbayUserName'          => $hTransaction->{'Buyer'}->{'UserID'},
                'EMail'                 => $hTransaction->{'Buyer'}->{'Email'},
                'Price'                 => $hTransaction->{'TransactionPrice'},
                'Quantity'              => $hTransaction->{'QuantityPurchased'},
                'LastMod'               => ConvertFromEbayDate($hTransaction->{'Status'}->{'LastTimeModified'}),
            });

            my $Product = $auction->get('Product');
            my $SubAuction;

            if(defined $Product){
                if($Product->get('IsMaster')){
                    $SubAuction = _FindSubAuctionForSKU($auction, $hTransaction->{'Variation'}->{'SKU'});
                    if($SubAuction){
                        $transaction->set({'SubProductAuction' => $SubAuction});
                        my $newSubProductQuantitySold = $SubAuction->get('QuantitySold') // 0 + $hTransaction->{'QuantityPurchased'};
                        $SubAuction->set({'QuantitySold' => $newSubProductQuantitySold});
                    }
                }

                my $newProductQuantitySold = $auction->get('QuantitySold') // 0 + $hTransaction->{'QuantityPurchased'};
                $auction->set({'QuantitySold' => $newProductQuantitySold});


                # correct stock if StockReduction is set "OnAuctionEnd"
                if (defined $EbayStockDecreaseType and $EbayStockDecreaseType eq 'OnAuctionEnd' ) {
                        #If this is master product, change to the subproduct
                        $Product = $SubAuction->get('Product') if defined $SubAuction;
                        $Product->substractBundleFromStock($hTransaction->{'QuantityPurchased'});

                        # Set the quantity of 0 if the auction is ended
                        if ((($auction->get('Status') == AUCTION_ENDED) or ($auction->get('Status') == AUCTION_COMPLETED))
                            and ($auction->get('ListingType') eq 'FixedPriceItem')
                            and ($auction->get('IsReviseInventory'))) {

                            my $newQuantity = $auction->get('Quantity') - $hTransaction->{'QuantityPurchased'};
                            $auction->set({'Quantity' => $newQuantity});
                        }
                }
            }
        }

        # get address if the transaction was right now completed
        if ($transaction->get('IsEbayComplete')) {
            GetLog->debug('transaction was completed right now.');
            if (not $customer) {
                GetLog->debug('create customer.');
                $customer = _createCustomerForTransaction($ShopEnv);
                $customer->set({
                    'EbayUserName'        => $hTransaction->{'Buyer'}->{'UserID'},
                    'EbayShopEnvironment' => $ShopEnv,
                });
                InsertEbayCustomerMap({
                   ShopID                => $auction->getSite->id,
                   EbayShopEnvironmentID => $ShopEnv->id,
                   EbayUserName          => $hTransaction->{'Buyer'}->{'UserID'},
                   CustomerID            => $customer->id,
                });

                $transaction->set({'Customer' => $customer});
            }
            _createBillingAddressForCustomer($hTransaction, $customer);
            my $ShippingAddress = _createShippingAddressForCustomer($hTransaction, $customer);

            TransactionToOrder(
                $transaction,
                $customer,
                $hTransaction->{'ShippingServiceSelected'}->{'ShippingService'},
                $hTransaction->{'Status'}->{'PaymentMethodUsed'},
                $hTransaction->{'ShippingServiceSelected'}->{'ShippingServiceCost'},
                $ShippingAddress,
                $hTransaction->{'Variation'}->{'SKU'});

        } elsif (not scalar @$aTransactionObjects) {
            # new uncompleted transaction.
            # send FulfillTransaction Mail ...
            print ' sending fulfill transaction mail for auction '.$ExternalAuctionId.' ... ' if $printOutput;
            eval {
                $transaction->sendFulfillmentMail();
                print "done.\n" if $printOutput;
            };
            if (ExistsError()) {
                print "failed.\n" if $printOutput;
                my $Error = GetError();
                # collect all errors, but let the script continue
                Warning($Error);
            }
        }
    }
    GetLog->debug('done.');
}

#========================================================================================
# function     TransactionToOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       TransactionToOrder( $Transaction, $Customer,
#               $EbayShippingMethodName, $EbayPaymentMethodName, $ShippingCost );
#----------------------------------------------------------------------------------------
# description  Generates a CustomerOrder from an ebay transaction.
#----------------------------------------------------------------------------------------
# input        $Transaction | EbayTransaction | object
# input        $Customer    | Customer | object
# input        $EbayShippingMethodName | Customer selected shipping method | string
# input        $EbayPaymentMethodName  | Customer selected payment method  | string
# input        $ShippingCost           | Shipping cost as defined on ebay  | float
# input        $ShippingAddress | shipping address (optional) | DE_EPAGES::Address::API::Object::Address
# input        $SKU | ePages product number (SKU) received from ebay | string
# hook         API_OrderCreated | hook parameter keys :
#               <ul>
#                 <li>Order        - customerorder          - object
#               </ul>
#========================================================================================
sub TransactionToOrder {
    my ($Transaction, $Customer, $EbayShippingMethodName, $EbayPaymentMethodName, $ShippingCost, $ShippingAddress, $SKU) = @_;
    my $ShopEnv = $Transaction->get('EbayAuction')->get('EbayShopEnvironment');
    my $Env = $ShopEnv->get('EbayEnvironment');

    my $LocalOrder = CreateOrderWithoutContainer($Customer, {
        'Alias' => GetNextOrderNo($ShopEnv->getSite, $Customer),
    });

    my $LanguageID =  GetPKeyLanguageByCode(lc $Env->get('LanguageID'));

    my $Container = $LocalOrder->addLineItemContainer({
                        'Class'      => 'LineItemContainerEbay',
                        'CurrencyID' => $Env->get('CurrencyID'),
                        'LocaleID' => $Env->get('LocaleID'),
    });

    _setShopAddress($LocalOrder->getSite->get('Address'), $LocalOrder);
    _setBillingAddress($Customer, $LocalOrder);
    _setShippingAddress($ShippingAddress, $LocalOrder);
    $LocalOrder->updateTaxArea();
    $Transaction->addToContainer($Container,$LanguageID,$SKU);
    _addShippingAndPayment($Container, $Env, $EbayShippingMethodName, $EbayPaymentMethodName, $ShippingCost);
    _finishImportOrder($LocalOrder);

    return;
}

#========================================================================================
# function     _setShopAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setShopAddress( $ShopAddress, $Order );
#----------------------------------------------------------------------------------------
# description  Clones the shop address into the order.
#----------------------------------------------------------------------------------------
# input        $ShopAddress | shop address | DE_EPAGES::Address::API::Object::Address
# input        $Order | customer order | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub _setShopAddress {
    my ($ShopAddress, $Order) = @_;
    return unless ($ShopAddress);
    $Order->set({'ShopAddress' => $ShopAddress->clone({'Parent' => $Order, 'Alias' => 'ShopAddress' })});
    return;
}

#========================================================================================
# function     _finishImportOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       _finishImportOrder( $Order );
#----------------------------------------------------------------------------------------
# description  Recalculates and calls HookAPI_OrderCreated to finalize the creation
#               of the order.
#----------------------------------------------------------------------------------------
# input        $Order | order | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub _finishImportOrder {
    my ($Order) = @_;
    $Order->recalculate;

    eval {
        TriggerHook(
            'API_OrderCreated',
            {
                'Order'   => $Order,
            }
        );
    };
    if ( ExistsError() ) {
        Warning( GetError() );
    }
    return;
}

#========================================================================================
# function     _addShippingAndPayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       _addShippingAndPayment( $Container, $Env, $EbayShippingMethodName, $EbayPaymentMethodName, $ShippingCost );
#----------------------------------------------------------------------------------------
# description  creates lineitems for shipping and payment
#----------------------------------------------------------------------------------------
# input        $Container | lineitemcontainer | object
# input        $ShopEnv | ebay shop environment | object
# input        $EbayShippingMethodName | shipping method name | string
# input        $EbayPaymentMethodName | payment method name | string
# input        $ShippingCost | shipping cost | float
#========================================================================================
sub _addShippingAndPayment {
    my ($Container, $Env, $EbayShippingMethodName, $EbayPaymentMethodName, $ShippingCost) = @_;
    # customer selected a shipping service.
    if ($EbayShippingMethodName ne 'NotSelected') {

        my $EbayShippingMethod;
        if ($Env->folder('EbayShippingMethods')->existsChild($EbayShippingMethodName)) {
            $EbayShippingMethod = $Env->folder('EbayShippingMethods')->child($EbayShippingMethodName);
        }

        my $TaxClass             = $Container->get('TaxMatrix')->defaultTaxClass;
        my $TaxRate              = $Container->taxRate($TaxClass);

        my $LineItemShipping = $Container->insertLineItem(
            LoadClassByAlias('LineItemShipping'), {
            'Alias'          => $Container->nextAlias('LineItemShipping %s'),
            'ShippingMethod' => undef,
            'LineItemPrice'  => $ShippingCost,
            'TaxClass'       => $TaxClass,
            'TaxRate'        => $TaxRate//0,
        });

        if ($EbayShippingMethod) {
            $LineItemShipping->set({'Name' => $EbayShippingMethod->get('NameOrAlias', $Env->get('LanguageID'))});
        }
    }

    my $EbayPaymentMethod;
    if ($Env->folder('EbayPaymentMethods')->existsChild($EbayPaymentMethodName)) {
        $EbayPaymentMethod = $Env->folder('EbayPaymentMethods')->child($EbayPaymentMethodName);
    }

    my $LineItemPayment = $Container->insertLineItem(
        LoadClassByAlias('LineItemPayment'),
        {
            'LineItemPrice' => 0,
            'PaymentMethod' => undef,
            'Alias'         => $Container->nextAlias('LineItemPayment %s'),
    });

    if ($EbayPaymentMethod) {
        $LineItemPayment->set({'Name' => $EbayPaymentMethod->get('NameOrAlias', $Env->get('LanguageID')) });
    }

    return;
}

#========================================================================================
# function     _createCustomerForTransaction
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createCustomerForTransaction( $ShopEnv );
#----------------------------------------------------------------------------------------
# description  creates a new customer
#----------------------------------------------------------------------------------------
# input        $ShopEnv | ebay shop environment | object
# return       $Customer | customer object | DE_EPAGES::Customer::API::Object::Customer
#========================================================================================
sub _createCustomerForTransaction {
    my ($ShopEnv) = @_;
    my $Shop = $ShopEnv->getSite();

    # create customer
    my $folder = $Shop->folder('Customers');

    return $Shop->get('CustomerClass')->insertObject({
        'Alias'                 => GetNextCustomerNo($Shop),
        'CustomerGroup'         => $Shop->get('NonMemberCustomerGroup'),
        'Shop'                  => $Shop,
        'Parent'                => $folder,
    });
}

#========================================================================================
# function     _setBillingAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setBillingAddress( $Customer, $Order );
#----------------------------------------------------------------------------------------
# description  clones the Customers billing address into the order
#----------------------------------------------------------------------------------------
# input        $Customer | customer | DE_EPAGES::Customer::API::Object::Customer
# input        $Order | order | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $CloneAddress | cloned address | DE_EPAGES::Address::API::Object::Address
#========================================================================================
sub _setBillingAddress {
    my ( $Customer, $Order ) = @_;
    my $BillingAddress = $Customer->get('BillingAddress');

    my $CloneAddress
      = $BillingAddress->clone( { 'Parent' => $Order, 'Alias' => 'BillingAddress' } );
    $Order->set( { 'BillingAddress' => $CloneAddress } );
    return $CloneAddress;
}

#========================================================================================
# function     _setShippingAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setShippingAddress( $Address, $Order );
#----------------------------------------------------------------------------------------
# description  clones the Customers shipping address into the order
#----------------------------------------------------------------------------------------
# input        $Address | address | DE_EPAGES::Address::API::Object::Address
# input        $Order | order | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $CloneAddress | cloned address | DE_EPAGES::Address::API::Object::Address
#========================================================================================
sub _setShippingAddress {
    my ( $Address, $Order ) = @_;

    return unless ($Address);

    my $CloneAddress
      = $Address->clone( { 'Parent' => $Order, 'Alias' => 'ShippingAddress' } );
    $Order->set( { 'ShippingAddress' => $CloneAddress } );
    return $CloneAddress;
}

#========================================================================================
# function     _createBillingAddressForCustomer
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createBillingAddressForCustomer( $hTransaction, $Customer );
#----------------------------------------------------------------------------------------
# description  creates a billing address object out of the data coming from ebay
#----------------------------------------------------------------------------------------
# input        $hTransaction | ebay transaction data | ref.hash
# input        $Customer | customer | DE_EPAGES::Customer::API::Object::Customer
# return       $CloneAddress | cloned address | DE_EPAGES::Address::API::Object::Address
#========================================================================================
sub _createBillingAddressForCustomer {
    my ($hTransaction, $Customer) = @_;

    my $hEbayAddress = $hTransaction->{'Buyer'}->{'RegistrationAddress'};
    if(not defined $hEbayAddress) {
        $hEbayAddress =  $hTransaction->{'Buyer'}->{'BuyerInfo'}->{'ShippingAddress'};
    }
    my $CustomerAddress = $Customer->folder('Addresses')->insertChild('Address',{
        'Alias'         => $Customer->folder('Addresses')->nextAlias('BillingAddress %s'),
        %{ _getAddressInfo($hEbayAddress)},
        'EMail'         => $hTransaction->{'Buyer'}->{'Email'},
    });
    $Customer->set({'BillingAddress' => $CustomerAddress});

    return $CustomerAddress;
}

#========================================================================================
# function     _createShippingAddressForCustomer
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createShippingAddressForCustomer( $hTransaction, $Customer );
#----------------------------------------------------------------------------------------
# description  creates a shipping address object out of the data coming from ebay
#----------------------------------------------------------------------------------------
# input        $hTransaction | ebay transaction data | ref.hash
# input        $Customer | customer | DE_EPAGES::Customer::API::Object::Customer
# return       $CloneAddress | cloned address | DE_EPAGES::Address::API::Object::Address
#========================================================================================
sub _createShippingAddressForCustomer {
    my ($hTransaction, $Customer) = @_;

    my $hEbayAddress = $hTransaction->{'Buyer'}->{'BuyerInfo'}->{'ShippingAddress'};
    my $hInfo = _getAddressInfo($hEbayAddress);

    my $hEbayAddressBilling = $hTransaction->{'Buyer'}->{'RegistrationAddress'};
    if( not defined $hEbayAddressBilling ) {
        $hEbayAddressBilling = $hTransaction->{'Buyer'}->{'BuyerInfo'}->{'ShippingAddress'};
    }
    my $hBillingInfo = _getAddressInfo($hEbayAddressBilling);

    # check if shipping address equals billing address.
    return if (! scalar grep { $hInfo->{$_} ne $hBillingInfo->{$_} } keys %$hInfo );

    my $ShippingAddress = $Customer->folder('Addresses')->insertChild('Address',{
        'Alias'         => $Customer->folder('Addresses')->nextAlias('ShippingAddress %s'),
        %{ $hInfo },
    });

    return $ShippingAddress;
}

#========================================================================================
# function     _getAddressInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getAddressInfo( $hTransaction, $Customer );
#----------------------------------------------------------------------------------------
# description  serializes ebay customer address data into hInfo-hash for use in database
#----------------------------------------------------------------------------------------
# input        $hEbayAddress | ebay address data | ref.hash
# return       $hInfo | hInfo hash | ref.hash
#========================================================================================
sub _getAddressInfo {
    my ($hEbayAddress) = @_;

    my ($firstname, $lastname) = $hEbayAddress->{'Name'} =~ /(.*)\s(\S+)$/;
    my $Country = $hEbayAddress->{'Country'};
    my $countryid;
    eval {
        $countryid = defined $Country ? GetPKeyCountryByCode2( $Country ) : undef;
    };
    HandleError(
        OnError => sub {
            my ($Error) = @_;
            if ($Error->code eq 'NoResults_CountryByCode2') {
                Warning($Error);
                return 'FIXED';
            }
        }
        );

    my %addressHash = (
        'DisplayName'   => $hEbayAddress->{'Name'},
        'FirstName'     => $firstname,
        'LastName'      => $lastname,
        'City'          => $hEbayAddress->{'CityName'},
        'CountryID'     => $countryid,
        # EPG-12337 : ebay usually submits "Invalid Request" instead of phone number.
        'Phone'         => $hEbayAddress->{'Phone'} !~ /Invalid/ ? $hEbayAddress->{'Phone'} : undef,
        'Zipcode'       => $hEbayAddress->{'PostalCode'},
        'State'         => $hEbayAddress->{'StateOrProvince'},
        'Street'        => $hEbayAddress->{'Street1'},
        'Street2'       => $hEbayAddress->{'Street2'},
    );

    my @attributes =  qw ( DisplayName FirstName LastName City Phone Zipcode State Street Street2 );

    foreach my $attribute ( @attributes ){
        my $MaxAttributeLength = LoadClassByAlias('Address')->attribute($attribute)->get('Length');
        $addressHash{$attribute} = substr($addressHash{$attribute},0,$MaxAttributeLength) if $MaxAttributeLength;
    }

    return \%addressHash;
}

#========================================================================================
# function     ResetAuctionCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       ResetAuctionCount();
#----------------------------------------------------------------------------------------
# description  Resets AuctionCount for each Shop which has the feature "EbayAuctions"
#               if its current count is greater than zero.
#========================================================================================
sub ResetAuctionCount {
    my $Feature = LoadFeatureByAlias('EbayAuctions');
    my $aShopIDs = DAL_GetPKeyShopsByEbayAuctionCount($Feature->id);

    foreach my $ShopID (@$aShopIDs) {
        # be sure that the ShopEnv (and its Shop) still exists
        next unless ExistsObject($ShopID);

        my $Shop = LoadObject($ShopID);
        eval {
            SetNewEbayAuctionsFeatureValue($Shop, 0);
        };
        if (ExistsError()) {
            my $Error = GetError();

            # collect all errors, but let the script continue
            Warning($Error);
        }
    }
}

#========================================================================================
# function     CheckProviderTokenExpiryDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckProviderTokenExpiryDate();
#----------------------------------------------------------------------------------------
# description  Checks eBay provider token if it is about to expire.
#               Prints a warning if token expires in less than
#               30 days. Prints an error if token has already expired.
#========================================================================================
sub CheckProviderTokenExpiryDate {
    my $NOW = DateTime->now();
    my $System = LoadRootObject();
    my $ProviderTokenExpiryDate = $System->get('EbayProviderTokenExpiryDate');
    return unless (defined $ProviderTokenExpiryDate);

    # token expiry date is in local timezone.
    $ProviderTokenExpiryDate->set_time_zone(GetLocalTimeZone());
    # convert to UTC
    $ProviderTokenExpiryDate->set_time_zone('UTC');
    my $ExpiresInDays = ($ProviderTokenExpiryDate->epoch() - $NOW->epoch()) / (60*60*24); ## no critic qw(ProhibitMagicNumbers)
    LogDebug( 'ExpiresInDays:', $ExpiresInDays);

    if ($ExpiresInDays <= 30 && $ExpiresInDays > 0) { ## no critic qw(ProhibitMagicNumbers)
        printf 'WARNING: Provider token for store \'%s\' expires in %d day(s).%s',
            $System->get('StoreName'),
            $ExpiresInDays,
            "\n";
    } elsif ($ExpiresInDays <= 0) {
        printf STDERR 'ERROR: Provider token for store \'%s\' has expired since %d day(s).%s',
            $System->get('StoreName'),
            (-1)*$ExpiresInDays,
            "\n";
    }
    return;
}
#========================================================================================
# function     CheckTokenExpiryDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckTokenExpiryDate($NoOutput);
#----------------------------------------------------------------------------------------
# description  Searches for eBay shop environments which ebay tokens are about to expire.
#               Sends off email to the merchant if the ebay token expires in less than
#               ten days.
#----------------------------------------------------------------------------------------
# input        $NoOutput | don't print any output if true | boolean
#========================================================================================
sub CheckTokenExpiryDate {
    my ($NoOutput) = @_;

    my $NOW = GetCurrentDBHandle()->currentDateTime;
    my $Duration = DateTime::Duration->new( days  => 10 );
    my $EndDuration =  DateTime::Duration->new( days  => 30 );
    my $ExpiryDate = $NOW + $Duration;
    my $EndNotificationDate = $NOW - $EndDuration;

    # search for expiring tokens
    my $System = LoadRootObject();
    my $SearchName = 'MBO-EbayTokenExpiryDate';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('EbayShopEnvironment'), $SearchName);
    my %Options = (
        'ObjectID' => $System->id,
        'PageSize' => 10000,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Date' => $ExpiryDate,
            'DateTo' => $EndNotificationDate,
        },
    );

    GetLog()->debug("GetSearchDefinition(LoadClassByAlias('EbayShopEnvironment'), '$SearchName')");
    LogDebug('Options', \%Options);
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);

    # initialize search
    $Search->statistic;

    my ($items) = [];
    for my $idx (1 .. $Search->get('CountOfPages')) {
        push @$items, @{$Search->items($idx)};
    }

    if (scalar @$items) {
        my $MailType = $System->folder('MailTypes')->child('ErrorEbaySettings');
        foreach my $ShopEnv (@$items) {
            # Skip shops closed by provider, but not closed by merchant. see EPG-12483
            my $Shop = $ShopEnv->getSite();
            next if ($Shop->get('ClosedOrClosedTempByProvider'));

            # do not send mail, if some attributes are set
            my $hSettings = $Shop->get(\@NO_MERCHANT_MAIL_ATTRIBUTES);
            my $NoMail;
            foreach my $v (values(%$hSettings)){
                $NoMail = 1 if $v;
            }
            next if $NoMail;

            eval {
                SendTokenErrorMail( $ShopEnv, $MailType, 'TokenWillExpireSoon' );
            };
            if (ExistsError()) {
                my $Error = GetError();

                 # collect all errors, but let the script continue
                Warning($Error);
                my $ErrorMessage = $Error->shortMessage;
                print("Error: $ErrorMessage\n") unless $NoOutput;
            }
        }
    }

    return;
}


sub _FindSubAuctionForSKU{
    my ($Auction, $SKU) = @_;

    foreach my $SubAuction ( @{$Auction->get('SubAuctions')} ) {
        if( $SubAuction->get('Product')->get('Alias') eq $SKU ){
            return $SubAuction;
        }
    }
    return undef;
}

sub _dropLine {
    my ($IDsFile, $ShopEnvID) = @_;
    return unless $IDsFile;
    return unless -f $IDsFile;

    my $Content;
    foreach my $Line (@{GetFileContentLines($IDsFile)}) {
        $Content .= $Line unless $Line =~ /^$ShopEnvID\s*$/
    }
    WriteFile($IDsFile, \$Content, { create_directory=>1 });
}

1;
