#========================================================================================
# package      DE_EPAGES::Ebay::API::Object::EbayCategoryTree
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for EbayCategoryTree.
#========================================================================================
package DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use base qw( DE_EPAGES::Shop::API::Object::ShopObject );

use strict;
use DE_EPAGES::Database::API::Config qw ( DatabaseConnectionData );
use DE_EPAGES::Database::API::Connection qw (Connect);
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( GetLoopFromDoubleArray isValid );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Tools qw ( ConvertFromEbayDate );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Ebay::API::XML::EbayAttributeSetImportHandler qw(ImportAttributeSetAttributes);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new;
#----------------------------------------------------------------------------------------
# description  Connects to the EbayCategory database.
#----------------------------------------------------------------------------------------
# return       $EbayCategoryDB | EbayCategoryDB object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment, $DBName) = @_;

    $DBName = EBAY_CATEGORIES_DB_NAME unless isValid($DBName);

    # connect the database
    my $hConnection = DatabaseConnectionData( $DBName );
    my $dbi = Connect($hConnection->{'datasource'}, '', '', $DBName);

    my $self = bless {
            'dbi' => $dbi,
            'Environment' => $Environment,
            'AttributeSetMap' => {},
        }, $class;

    # insert site into ebaysite table
    $self->insertEbaySite unless $self->existsEbaySite;

    my $EbaySiteAlias = $self->siteAlias;
    my $aaRows = $dbi->execute('SELECT ebaysiteid FROM ebaysite WHERE alias = ?', [$EbaySiteAlias]);
    $self->{'SiteID'} = $aaRows->[0]->[0];

    return $self;
}

sub dbi { return shift->{'dbi'}; }

sub environment { return shift->{'Environment'}; }

sub siteAlias { return shift->environment->alias; }

sub siteID { return shift->{'SiteID'}; }

sub attributeSetMap { return shift->{'AttributeSetMap'}; }


#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->update( $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates category list from eBay.
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
# return       $hResult | category list information, e.g. version, count with following keys:
#               <ul>
#                   <li>CategoryVersion - current eBay category list version - int
#                   <li>CategoryCount   - total count of categories - numeric(20)
#                   <li>UpdateTime      - date of last update of eBay's category list
#                       (on eBay side) (optional)- datetime
#               </ul>| reference to hash
#========================================================================================
sub update {
    my $self = shift;
    my ($printOutput) = @_;


    # delete EbayCategoryTemp for Site
    $self->deleteEbayCategoryTemp;

    # update category lists
    my $hResult = $self->updateMainCategoryList($printOutput);

    # swap category lists from EbayCategoryTemp to EbayCategory table
    $self->swapEbayCategories;

    # set CategoryListVersion,CategoryListCount, CategoryListUpdateTimeEbay for EbaySite
    $self->setCategoryListAttributes($hResult);

    return $hResult;
}

#========================================================================================
# function     getAttributesCS
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->getAttributesCS();
#----------------------------------------------------------------------------------------
# description  get new CategorySet mappings from ebay and update table ebaycategory
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
#========================================================================================
sub getAttributesCS {
    my $self = shift;
    my ($printOutput) = @_;

    my $Environment = $self->environment;

    # IDs to request from global set
    my @ExternalAttributeSetIDs = keys(%{$self->{'AttributeSetMap'}});

    my $count=1;
    my $Sets;

    print "get Values and dependencies for AttributeSetIDs ...\n " if $printOutput;

    foreach my $ExternalAttributeSetID (@ExternalAttributeSetIDs) {
        # ignore  attributesets we have already in the database
        my $AttributeSetVersion = $self->{'AttributeSetMap'}{$ExternalAttributeSetID}{'AttributeSetVersion'};
        my $DBAttributeSetVersion = $self->getAttributeSetVersionByExternalAttributeSetID($ExternalAttributeSetID);

        next if $AttributeSetVersion eq $DBAttributeSetVersion;
        print "$ExternalAttributeSetID " if $printOutput;

        push @$Sets,$ExternalAttributeSetID;
        if ($count < 10){ ## no critic qw( ProhibitMagicNumbers )
            $count++;
        } else {
            my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
                'Environment'   => $Environment,
            );
            my $hGetResult = $RemoteEbay->getAttributesCS($Sets);
            #LogDebug("getAttributesCS RESULT",$hGetResult);
            $self->_insertAttributeSetAttributes($hGetResult->{'AttributeData'});
            $count=1;
            $Sets = undef;
            print "\n " if $printOutput;
        }
    }
    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'Environment'   => $Environment,
    );
    if (defined $Sets and scalar(@$Sets)) {
        my $hGetResult = $RemoteEbay->getAttributesCS($Sets);
        #LogDebug("LASTgetAttributesCS RESULT",$hGetResult);
        $self->_insertAttributeSetAttributes($hGetResult->{'AttributeData'});
    }
    print "OK\n" if $printOutput;
}

sub _insertAttributeSetAttributes {
    my $self = shift;
    my ($sAttributeData) = @_;

    ImportAttributeSetAttributes($self,$sAttributeData);
}

#====================================================== ==================================
# function     updateCSMappings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->updateCSMappings();
#----------------------------------------------------------------------------------------
# description  get new CategorySet mappings from ebay and update table ebaycategory
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
#========================================================================================
sub updateCSMappings {
    my $self = shift;
    my ($printOutput) = @_;


    $self->getAllCSMappings($printOutput,$self->environment->get('AttributeSystemVersion'));

    return;
}

#========================================================================================
# function     getAllCSMappings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->getAllCSMappings($printOutput,$AttributeSystemVersion);
#----------------------------------------------------------------------------------------
# description  Get all CategorySet mappings from ebay and update the category database
#               if $AttributeSystemVersion is not defined
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
# input        $AttributeSystemVersion | AttributeSystemVersion to request | int
#========================================================================================
sub getAllCSMappings {
    my $self = shift;
    my ($printOutput,$AttributeSystemVersion) = @_;

    my $Environment = $self->environment;
    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'Environment'   => $Environment,
    );


    #$AttributeSystemVersion = 3451; # 3 AttributesSets

    print "getting changed attribute sets for AttributeSetVersion: $AttributeSystemVersion ...\n" if $printOutput;


    # get mappings from ebay
    my $hGetResult = $RemoteEbay->getCategory2CS(undef,$AttributeSystemVersion);

    # here we set for each category if we can use the global attributesetids (AttrSetIDCondition)
    # update the fields usesiteconditionattributesetid
    # mapping form tables to AttrSetID
    my $AttrSetCondition = $Environment->get('AttrSetIDCondition');
    foreach my $SideWideCharSet (@{$hGetResult->{'SiteWideCharacteristicSets'}}) {
        my $CharSetID = $SideWideCharSet->{'CharacteristicsSet'}{'AttributeSetID'};
        next unless ($CharSetID == $AttrSetCondition);
        my $aExcludes = $SideWideCharSet->{'ExcludeCategoryID'};
        next unless $aExcludes;
        if (ref $aExcludes ne 'ARRAY') {
            my $sExcludes = $aExcludes;
            $aExcludes = [$sExcludes];
        }

        $self->_updateSideWideCharSetExcludes($aExcludes);
    }

    # here we set the attributes...
    $self->_updateCategoryAttributeSet($printOutput,$hGetResult);

    # set the correct version for further calls
    $self->setAttributeSystemVersion($hGetResult->{'AttributeSystemVersion'});
}

#========================================================================================
# function     _updateCategoryAttributeSet
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->_updateCategoryAttributeSet($ahCategory);
#----------------------------------------------------------------------------------------
# description  update the internal AttributeSetID for each Category
#========================================================================================
sub _updateCategoryAttributeSet {
    my $self = shift;
    my ($printOutput,$hGetResult) = @_;

    my $UnmappedCategoryArray = $hGetResult->{'UnmappedCategoryArray'}[0]{'Category'};
    my $MappedCategoryArray = $hGetResult->{'MappedCategoryArray'}[0]{'Category'};

    if (ref $MappedCategoryArray ne 'ARRAY') {
        my $sMappedCategoryArray = $MappedCategoryArray;
        $MappedCategoryArray = [$sMappedCategoryArray];
    }

    if (ref $UnmappedCategoryArray ne 'ARRAY') {
        my $sUnmappedCategoryArray = $UnmappedCategoryArray;
        $UnmappedCategoryArray = [$sUnmappedCategoryArray];
    }

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;

    # setze globale Merkliste ExternalAttributeSetID => AttributeSetID
    foreach my $hCategory (@{$MappedCategoryArray}) {
        my $ExternalAttributeSetID = $hCategory->{'CharacteristicsSets'}{'AttributeSetID'};
        my $AttributeSetVersion = $hCategory->{'CharacteristicsSets'}{'AttributeSetVersion'};
        my $InternalAttributeSetID = $self->getAttributeSetIDByExtAttrSetIDAndVersion($ExternalAttributeSetID,$AttributeSetVersion);

        $self->{'AttributeSetMap'}{$ExternalAttributeSetID} = {'AttributeSetID' => $InternalAttributeSetID,
            'AttributeSetVersion' => $AttributeSetVersion,
        };
    }

    #  set Attributes for AttributeSetIDs from $self->{'AttributeSetMap'}
    #  test if we have a different AttributeSetVersion for the AttributeSet in the DB (then get the new Attribute)
    $self->getAttributesCS($printOutput);

    $dbi->transaction( sub {

        print "change AttributeSetIDs in Categories ...\n" if $printOutput;

        # delete Category-Attributeset assignment (unmapped)
        foreach my $hCategory (@{$UnmappedCategoryArray}) {
            my $CategoryID = $hCategory->{'CategoryID'};
            my $sql = 'UPDATE ebaycategory SET attributesetid = ? '.
                'WHERE externalcategoryid = ? and ebaysiteid = ?';
            $dbi->do( $sql,[ undef, $CategoryID, $EbaySiteID, ] );
        }

        #set the AttributeSetID for the categories
        foreach my $hCategory (@{$MappedCategoryArray}) {
            my $CategoryID = $hCategory->{'CategoryID'};
            my $ExternalAttributeSetID = $hCategory->{'CharacteristicsSets'}{'AttributeSetID'};
            my $InternalAttributeSetID = $self->{'AttributeSetMap'}{$ExternalAttributeSetID}{'AttributeSetID'};

            #LogDebug('set Attributeset SetID for Cat:',"A: ".$InternalAttributeSetID." C:".$CategoryID);
            my $sql = 'UPDATE ebaycategory '.
                'SET attributesetid = ? '.
                'WHERE externalcategoryid = ? and ebaysiteid = ?';
            $dbi->do( $sql,[ $InternalAttributeSetID, $CategoryID, $EbaySiteID, ] );

        }

    } );

    # remove unused Attributesets
    my $aUnUsedAttributeSetIDs = $self->getUnusedAttributeSetIDs();
    foreach my $InternalAttributeSetID (@$aUnUsedAttributeSetIDs) {
        #LogDebug('delete unused Attributeset SetID:',$InternalAttributeSetID);
        print "remove unused AttributeSet: $InternalAttributeSetID ...\n" if $printOutput;
        $self->deleteAttributeSet($InternalAttributeSetID);
    }

    # delete global hash
    $self->{'AttributeSetMap'} = {};
}

#========================================================================================
# function     _updateSideWideCharSetExcludes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->_updateSideWideCharSetExcludes($AttrSetColumnName,$aExcludes);
#----------------------------------------------------------------------------------------
# description  update exlude list for SideWideCharSets
#========================================================================================
sub _updateSideWideCharSetExcludes {
    my $self = shift;
    my ($aExcludes) = @_;

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;

    $dbi->transaction( sub {

        #reset all to 1 (use the global IDs)
        my $sql = 'UPDATE ebaycategory SET usesiteconditionattributesetid = 1 '.
            'WHERE ebaysiteid = ?';
        $dbi->do( $sql,[ $EbaySiteID ] );

        #set the exclude list
        my $_sql = 'UPDATE ebaycategory SET usesiteconditionattributesetid = 0 '.
            'WHERE externalcategoryid = ? and ebaysiteid = ?';
        foreach my $ExclCategoryID (@{$aExcludes}) {
            $dbi->do( $_sql, [ $ExclCategoryID, $EbaySiteID ] );
        }
    });
}

#========================================================================================
# function     updateMainCategoryList
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->updateMainCategoryList( $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates main category list from eBay.
#----------------------------------------------------------------------------------------
# input        $printOutput | print output to console | boolean
# return       $hResult | category list information, e.g. version, count with following keys:
#               <ul>
#                   <li>CategoryVersion - current eBay category list version - int
#                   <li>CategoryCount   - total count of categories - numeric(20)
#                   <li>UpdateTime      - date of last update of eBay's category list
#                       (on eBay side) (optional)- datetime
#               </ul>| reference to hash
#========================================================================================
sub updateMainCategoryList {
    my $self = shift;
    my ($printOutput) = @_;

    my $Environment = $self->environment;
    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'Environment'   => $Environment,
    );

    my $EbaySiteID = $self->siteID;

    # get main categories
    my $hResult;
    print "getting main categories for Site ID $EbaySiteID  ...\n" if $printOutput;
    my $hGetResult;
    $hGetResult = $RemoteEbay->getCategories(0, 1);
    @{$hResult}{qw (CategoryVersion UpdateTime)} = @{$hGetResult}{qw (CategoryVersion UpdateTime)};
    print " done.\n" if $printOutput;

    print ' inserting main categories ('.scalar @{$hGetResult->{'Category'}}.")...\n" if $printOutput;
    GetLog()->debug('Update of category list: inserting main categories ('.scalar @{$hGetResult->{'Category'}}.')');
    my $dbi = $self->dbi;
    $dbi->transaction( sub {
        foreach my $hCategory (@{$hGetResult->{'Category'}}) {
            $self->insertEbayCategoryTemp( $hCategory );
            print ' -> ' . $hCategory->{'CategoryName'}."\n" if $printOutput;
        }
    } );
    print "  done.\n" if $printOutput;

    foreach my $hCategory (@{$hGetResult->{'Category'}}) {
        #update subs of maincats
        $self->updateCategoryListParts($hCategory->{'CategoryID'}, $printOutput);
    }

    return $hResult;
}

#========================================================================================
# function     updateCategoryListParts
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->updateCategoryListParts( $ExternalCategoryId, $printOutput );
#----------------------------------------------------------------------------------------
# description  Updates sub category lists from eBay.
#----------------------------------------------------------------------------------------
# input        $ExternalCategoryId | identifier of external eBay category | int
# input        $printOutput | print output to console | boolean
# return       $hResult | category list information, e.g. version, count with following keys:
#               <ul>
#                   <li>CategoryVersion - current eBay category list version - int
#                   <li>CategoryCount   - total count of categories - numeric(20)
#                   <li>UpdateTime      - date of last update of eBay's category list
#                       (on eBay side) (optional)- datetime
#               </ul>| reference to hash
#========================================================================================
sub updateCategoryListParts {
    my $self = shift;
    my ($ExternalCategoryId, $printOutput) = @_;

    my $Environment = $self->environment;
    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
        'Environment'   => $Environment,
    );

    #update subs of main categories
    print "getting sub categories of $ExternalCategoryId ...\n" if $printOutput;
    my $hGetResult ;
    $hGetResult = $RemoteEbay->getCategories(0, 0, $ExternalCategoryId);
    print " done.\n" if $printOutput;

    print ' inserting sub categories ('.scalar @{$hGetResult->{'Category'}}.")...\n" if $printOutput;
    GetLog()->debug("Update of category list: inserting sub categories of $ExternalCategoryId("
        .scalar @{$hGetResult->{'Category'}}.')');

    my $count = 0;
    my $dbi = $self->dbi;
    $dbi->transaction( sub {
        foreach my $hCategory (@{$hGetResult->{'Category'}}) {
            next if $hCategory->{'CategoryID'} eq $ExternalCategoryId;
            $self->insertEbayCategoryTemp( $hCategory );
            print "\r  current: $count" if ($count % 100 == 0 and $printOutput); ## no critic qw(ProhibitMagicNumbers)
            $count++;
        }
    } );
    print "\r  current: $count ... done.\n" if $printOutput;

    return;
}

#========================================================================================
# function     insertEbayCategoryTemp
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->insertEbayCategoryTemp( $hCategory );
#----------------------------------------------------------------------------------------
# description  Inserts category data into EbayCategoryTemp.
#----------------------------------------------------------------------------------------
# input        $hCategory | eBay category attributes | ref.hash
#========================================================================================
sub insertEbayCategoryTemp {
    my $self = shift;
    my ( $hCategory ) = @_;

    my $hCategoryInfo = {
        'EbaySiteID'        => $self->siteID,
        'ExternalCategoryId'    => $hCategory->{'CategoryID'},
        'Name'              => $hCategory->{'CategoryName'},
        'CategoryLevel'     => $hCategory->{'CategoryLevel'},
        'IsVirtual'         => ($hCategory->{'Virtual'} eq 'true'),
        'IsExpired'         => ($hCategory->{'Expired'} eq 'true'),
        'IsLeaf'            => ($hCategory->{'LeafCategory'} eq 'true'),
        'AttributeSetID'    => undef,
        'UseSiteConditionAttributeSetID'    => 1,
        'UseSiteReturnPolicyAttributeSetID' => 1,
    };
    my $dbi = $self->dbi;

    # skip if external category id is not distinct
    if (defined $self->{'TempCategoryIDs'}->{$hCategoryInfo->{'ExternalCategoryId'}}) {
        Warning('ExternalCategoryIDNotDistinct', {
                'EbaySite' => $self->siteAlias,
                'Category Name' => $hCategoryInfo->{'Name'},
                'ExternalCategoryID' => $hCategoryInfo->{'ExternalCategoryId'},
            });
        return;
    }

    # find internal category id for external ebay category id (parent id)
    if ($hCategory->{'CategoryParentID'} ne $hCategory->{'CategoryID'}) {
        my $ExternalParentID = $hCategory->{'CategoryParentID'};
        die "external category id not cached $ExternalParentID"
            if not defined $self->{'TempCategoryIDs'}->{$ExternalParentID};
        $hCategoryInfo->{'ParentCategoryID'} = $self->{'TempCategoryIDs'}->{$ExternalParentID};
    }
    my @Keys = qw( ExternalCategoryId ParentCategoryID EbaySiteID Name CategoryLevel IsExpired IsVirtual IsLeaf
        AttributeSetID UseSiteConditionAttributeSetID UseSiteReturnPolicyAttributeSetID );
    $dbi->do( <<END_SQL, [ @{$hCategoryInfo}{@Keys} ] );
INSERT INTO ebaycategorytemp (externalcategoryid, parentcategoryid, ebaysiteid, name, categorylevel,
   isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, usesitereturnpolicyattributesetid )
VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
END_SQL
    my $EbayCategoryID = $dbi->dbh->last_insert_id(undef, undef, undef, undef);
    # caches internal category ids
    $self->{'TempCategoryIDs'}->{$hCategoryInfo->{'ExternalCategoryId'}} = $EbayCategoryID;

    return;
}

#========================================================================================
# function     existsEbaySite
# state        private
#----------------------------------------------------------------------------------------
# syntax       $exists = $EbayCategoryTree->existsEbaySite();
#----------------------------------------------------------------------------------------
# description  Returns true if the EbaySite exists.
#----------------------------------------------------------------------------------------
# return       $exists | 1/0  the EbaySite exists (yes/no) | boolean
#========================================================================================
sub existsEbaySite {
    my $self = shift;

    my $EbaySiteAlias = $self->siteAlias;
    my $dbi = $self->dbi;
    my $aaRows = $dbi->execute('SELECT COUNT(*) FROM ebaysite WHERE alias = ?', [$EbaySiteAlias]);

    return $aaRows->[0]->[0];
}

#========================================================================================
# function     existsEbayCategory
# state        private
#----------------------------------------------------------------------------------------
# syntax       $exists = $EbayCategoryTree->existsEbayCategory($CategoryID);
#----------------------------------------------------------------------------------------
# description  Returns true if the Ebay Category with the given identifier exists.
#----------------------------------------------------------------------------------------
# input        $CategoryID | internal CategoryID (not the eBay Category Id)  | int
# return       $exists | 1/0  the EbaySite exists (yes/no) | boolean
#========================================================================================
sub existsEbayCategory {
    my $self = shift;
    my ($CategoryID) = @_;

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;
    my $aaRows = $dbi->execute(
        'SELECT COUNT(*) FROM ebaycategory WHERE ebaycategoryid = ? AND ebaysiteid = ?',
        [$CategoryID, $EbaySiteID]
    );

    return $aaRows->[0]->[0];
}

#========================================================================================
# function     insertEbaySite
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->insertEbaySite($CategoryID);
#----------------------------------------------------------------------------------------
# description  Creates an entry in the EbaySite table.
#========================================================================================
sub insertEbaySite {
    my $self = shift;

    my $EbaySiteAlias = $self->siteAlias;
    my $dbi = $self->dbi;
    $dbi->do('INSERT INTO ebaysite (categorylistisupdating, alias) VALUES (?, ?)',
        [0, $EbaySiteAlias]
    );

    return;
}

#========================================================================================
# function     getPKeyEbayCategoryByExternalCategoryId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryID = $EbayCategoryTree->getPKeyEbayCategoryByExternalCategoryId($ExternalCategoryId);
#----------------------------------------------------------------------------------------
# description  Returns primary key of EbayCategory by ExternalCategoryId.
#               (generated function GetPKeyEbayCategoryByEbaySiteID).
#----------------------------------------------------------------------------------------
# input        $ExternalCategoryId | eBay's Category Id | numeric(20)
# return       $EbayCategoryID | internal CategoryID (not the eBay Category Id) (primary key) | int
#========================================================================================
sub getPKeyEbayCategoryByExternalCategoryId {
    my $self = shift;
    my ($ExternalCategoryId) = @_;

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;
    my $sql = 'SELECT ebaycategoryid FROM ebaycategory WHERE ebaysiteid = ? AND externalcategoryid = ?';
    my $aaRows = $dbi->execute($sql, [$EbaySiteID, $ExternalCategoryId], ['Integer']);

    return $aaRows->[0]->[0];
}

#========================================================================================
# function     deleteEbayCategoryTemp
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->deleteEbayCategoryTemp();
#----------------------------------------------------------------------------------------
# description  Removes all entries of table ebaycategorytemp for current SiteID.
#========================================================================================
sub deleteEbayCategoryTemp {
    my $self = shift;

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;
    $dbi->do('DELETE FROM ebaycategorytemp WHERE ebaysiteid=?', [$EbaySiteID]);

    return;
}

#========================================================================================
# function     swapEbayCategories
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->swapEbayCategories();
#----------------------------------------------------------------------------------------
# description  Copies all entries of table ebaycategorytemp into table ebaycategory.
#               This way eBay catgeories can be downloaded into table ebaycategorytemp
#               while eBay categories are still available for shops via table
#               ebaycategory.
#========================================================================================
sub swapEbayCategories {
    my $self = shift;

    my $EbaySiteID = $self->siteID;
    my $dbi = $self->dbi;
    $dbi->transaction( sub {

        $dbi->do(<<END_SQL);
CREATE TEMPORARY TABLE _temp (
    ebaycategoryid       integer      NOT NULL
,   externalcategoryid   numeric      NOT NULL
,   parentcategoryid     integer      NULL
,   ebaysiteid           integer      NOT NULL
,   name                 text         NOT NULL
,   categorylevel        integer      NOT NULL
,   isexpired            integer      NOT NULL
,   isvirtual            integer      NOT NULL
,   isleaf               integer      NOT NULL
,   attributesetid       integer      NULL
,   usesiteconditionattributesetid    integer NOT NULL
,   usesitereturnpolicyattributesetid integer NOT NULL
)
END_SQL

        # Copy all data (with the attributesetid from ebaycategory) to a temp table
        $dbi->do(<<END_SQL, [$EbaySiteID]);
INSERT INTO _temp (ebaycategoryid,externalcategoryid, parentcategoryid,ebaysiteid, name, categorylevel,
       isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, usesitereturnpolicyattributesetid)
SELECT ect.ebaycategoryid, ect.externalcategoryid, ect.parentcategoryid, ect.ebaysiteid, ect.name, ect.categorylevel,
       ect.isexpired, ect.isvirtual, ect.isleaf, ec.attributesetid, ect.usesiteconditionattributesetid, ect.usesitereturnpolicyattributesetid
FROM ebaycategorytemp ect
LEFT JOIN ebaycategory ec ON ec.ebaysiteid = ect.ebaysiteid AND ec.externalcategoryid = ect.externalcategoryid
WHERE ect.ebaysiteid = ?
END_SQL

        $dbi->do('DELETE FROM ebaycategory WHERE ebaysiteid=?', [$EbaySiteID]);


        #copy data to ebaycategory (the old attributesetids are still available)
        $dbi->do(<<END_SQL, [$EbaySiteID]);
INSERT INTO ebaycategory (ebaycategoryid, externalcategoryid, parentcategoryid, ebaysiteid, name, categorylevel,
       isexpired, isvirtual, isleaf, attributesetid ,usesiteconditionattributesetid, usesitereturnpolicyattributesetid)
SELECT ebaycategoryid, externalcategoryid, parentcategoryid, ebaysiteid, name, categorylevel,
       isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, usesitereturnpolicyattributesetid
FROM _temp WHERE ebaysiteid = ?
END_SQL

        $dbi->do('DROP TABLE _temp', []);

        $dbi->do('DELETE FROM ebaycategorytemp WHERE ebaysiteid=?', [$EbaySiteID]);

    } );

    return;
}

#========================================================================================
# function     totalCount
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->totalCount();
#----------------------------------------------------------------------------------------
# description  Return the total count of eBay categories for given eBay environment.
#========================================================================================
sub totalCount {
    my $self = shift;

    my $dbi = $self->dbi;
    my $EbaySiteID = $self->siteID;
    my $sql = 'SELECT COUNT(*) FROM ebaycategory WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute($sql, [$EbaySiteID], ['Integer']);

    return $aaRows->[0]->[0];
}

#========================================================================================
# function     setCategoryListAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->setCategoryListAttributes($hResult);
#----------------------------------------------------------------------------------------
# description  Updates table ebaysite and sets category list attributes such as
#               categorylistversion, categorylistcount, categorylistupdatetimeebay,
#               categorylistupdatetime, categorylistisupdating.
#----------------------------------------------------------------------------------------
# input       $hResult | with following keys:
#               <ul>
#                   <li>CategoryVersion - current eBay category list version - int
#                   <li>CategoryCount   - total count of categories - numeric(20)
#                   <li>UpdateTime      - date of last update of eBay's category list
#                       (on eBay side) (optional)- datetime
#               </ul>| reference to hash
#========================================================================================
sub setCategoryListAttributes {
    my $self = shift;
    my ($hResult) = @_;

    my $dbi = $self->dbi;
    my $EbaySiteAlias = $self->siteAlias;
    $hResult->{'CategoryCount'} = $self->totalCount;
    my $EbayUpdateTime = exists $hResult->{'UpdateTime'} ? ConvertFromEbayDate( $hResult->{'UpdateTime'} ) : undef;
    my $ServerUpdateTime = $dbi->currentDateTime;
    my $sql = 'UPDATE ebaysite SET categorylistversion = ?, categorylistcount = ?, '.
        'categorylistupdatetimeebay = ?, categorylistupdatetime = ?, categorylistisupdating = ? '.
        'WHERE alias = ?';
    # date format: 2007-05-16 11:14:18
    $dbi->do(   $sql,
        [   $hResult->{'CategoryVersion'},
            $hResult->{'CategoryCount'},
            $EbayUpdateTime->ymd.' '.$EbayUpdateTime->hms,
            $ServerUpdateTime->ymd.' '.$ServerUpdateTime->hms,
            0,
            $EbaySiteAlias ]
    );

    return;
}

#-----------------------------------------------------------------------------------------------------------

#========================================================================================
# function     getInfoRootCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCategories = $EbayCategoryTree->getInfoRootCategories();
#----------------------------------------------------------------------------------------
# description  Returns all eBay root categories.
#----------------------------------------------------------------------------------------
# return       $ahCategories | with following keys:
#               <ul>
#                   <li>CategoryLevel      - level of category in tree   - int
#                   <li>EbayCategoryID     - internal category id - int
#                   <li>ExternalCategoryId - external eBay category id   - numeric(20)
#                   <li>IsExpired          - true if category is expired on eBay's side - boolean
#                   <li>IsLeaf             - true if category has no sub categories - boolean
#                   <li>IsVirtual          - true if category is virtual - boolean
#                   <li>Name               - category name - string
#                   <li>ParentCategoryID   - identifier of the parent category - int
#                   <li>AttributeSetID     - attributeset id of category - int
#                   <li>UseSiteConditionAttributeSetID    - true if global sets are valid for Condition - boolean
#                   <li>UseSiteReturnPolicyAttributeSetID - true if global sets are valid for ReturnPolicy - boolean
#               </ul>| ref.array.hash
#========================================================================================
sub getInfoRootCategories {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT ebaycategoryid, externalcategoryid, parentcategoryid, name, categorylevel, '.
        'isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, usesitereturnpolicyattributesetid '.
        'FROM ebaycategory '.
        'WHERE categorylevel = ? AND ebaysiteid = ?';
    my $SiteID = $self->siteID;
    my $dbResult = $dbi->execute(
        $sql,
        [1, $SiteID],
        ['Integer', 'Integer', 'Integer', 'String', 'Integer', 'Boolean', 'Boolean', 'Boolean', 'Integer', 'Boolean', 'Boolean']
    );

    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (EbayCategoryID ExternalCategoryId ParentCategoryID Name CategoryLevel IsExpired IsVirtual IsLeaf
            AttributeSetID UseSiteConditionAttributeSetID UseSiteReturnPolicyAttributeSetID )]
    );
}



#========================================================================================
# function     getInfoEbayCategoriesByParentCategoryID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahCategories = $EbayCategoryTree->getInfoEbayCategoriesByParentCategoryID($ParentCategoryID);
#----------------------------------------------------------------------------------------
# description  Returns objects of EbayCategory by alternate key ParentCategoryID.
#----------------------------------------------------------------------------------------
# input        $ParentCategoryID | internal Parent Category (not a eBay Category Id) | int
# return       $ahCategories | with following keys:
#               <ul>
#                   <li>EbayCategoryID     - internal category id - int
#                   <li>ExternalCategoryId - external category Id - numeric(20)
#                   <li>ParentCategoryID   - internal parent category id (optional)- int
#                   <li>EbaySiteID    - Ebay Site identifier - int
#                   <li>Name          - name of category     - string
#                   <li>CategoryLevel - level from root      - int
#                   <li>IsExpired          - true if category is expired on eBay's side - boolean
#                   <li>IsLeaf             - true if category has no sub categories - boolean
#                   <li>IsVirtual          - true if category is virtual - boolean
#                   <li>AttributeSetID     - attributeset id of category - int
#                   <li>UseSiteConditionAttributeSetID    - true if global sets are valid for Condition - boolean
#                   <li>UseSiteReturnPolicyAttributeSetID - true if global sets are valid for ReturnPolicy - boolean
#               </ul> | ref.array.hash
#========================================================================================
sub getInfoEbayCategoriesByParentCategoryID {
    my $self = shift;
    my ($ParentCategoryID) = @_;

    my $dbi = $self->dbi;
    my $sql = 'SELECT ebaycategoryid, externalcategoryid, parentcategoryid, name, categorylevel, '.
        'isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, '.
        'usesitereturnpolicyattributesetid FROM ebaycategory WHERE parentcategoryid = ?';
    my $dbResult = $dbi->execute(
        $sql,
        [$ParentCategoryID],
        ['Integer', 'Float', 'Integer', 'String', 'Integer', 'Boolean',
            'Boolean', 'Boolean', 'Integer', 'Boolean', 'Boolean'
        ]
    );

    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (EbayCategoryID ExternalCategoryId ParentCategoryID Name CategoryLevel IsExpired IsVirtual IsLeaf
            AttributeSetID UseSiteConditionAttributeSetID UseSiteReturnPolicyAttributeSetID )]
    );
}

#========================================================================================
# function     getInfoEbayCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hEbayCategory = $EbayCategoryTree->getInfoEbayCategory($EbayCategoryID);
#----------------------------------------------------------------------------------------
# description  Returns the EbayCategory by identifier.
#----------------------------------------------------------------------------------------
# input        $EbayCategoryID | internal CategoryID (not the eBay Category Id) (primary key) | int
# return       $hEbayCategory | with following keys:
#               <ul>
#                   <li>EbayCategoryID     - internal category id - int
#                   <li>ExternalCategoryId - external category Id - numeric(20)
#                   <li>ParentCategoryID   - internal parent category id (optional)- int
#                   <li>EbaySiteID    - Ebay Site identifier - int
#                   <li>Name          - name of category     - nvarchar(255)
#                   <li>CategoryLevel - level from root      - int
#                   <li>IsExpired          - true if category is expired on eBay's side - boolean
#                   <li>IsLeaf             - true if category has no sub categories - boolean
#                   <li>IsVirtual          - true if category is virtual - boolean
#                   <li>AttributeSetID     - attributeset id of category - int
#                   <li>UseSiteConditionAttributeSetID    - true if global sets are valid for Condition - boolean
#                   <li>UseSiteReturnPolicyAttributeSetID - true if global sets are valid for ReturnPolicy - boolean
#               </ul>| reference to hash
#========================================================================================
sub getInfoEbayCategory {
    my $self = shift;
    my ($EbayCategoryID) = @_;

    my $dbi = $self->dbi;
    my $aaRows = $dbi->execute(
        'SELECT ebaycategoryid, externalcategoryid, parentcategoryid, name, '.
            'categorylevel, isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, '.
            'usesitereturnpolicyattributesetid '.
            'FROM ebaycategory '.
            'WHERE ebaycategoryid=?',
        [$EbayCategoryID],
        ['Integer', 'Float', 'Integer', 'String', 'Integer', 'Boolean', 'Boolean', 'Boolean',
            'Integer', 'Boolean', 'Boolean']
    );

    return undef unless scalar @$aaRows;
    my $aRow = $aaRows->[0];
    my %Category;
    @Category{qw (EbayCategoryID ExternalCategoryId ParentCategoryID Name CategoryLevel IsExpired IsVirtual IsLeaf
        AttributeSetID UseSiteConditionAttributeSetID UseSiteReturnPolicyAttributeSetID )} = @$aRow;

    return \%Category;
}



#========================================================================================
# function     getInfoEbayCategoryByExternalCategoryId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hEbayCategory = $EbayCategoryTree->getInfoEbayCategoryByExternalCategoryId($ExternalCategoryId);
#----------------------------------------------------------------------------------------
# description  Returns the eBay category by external category id. If no eBay category
#               exists for the given external category id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $ExternalCategoryId | eBay's Category Id | numeric(20)
# return       $hEbayCategory | with following keys:
#               <ul>
#                   <li>EbayCategoryID     - internal category id - int
#                   <li>ExternalCategoryId - external category Id - numeric(20)
#                   <li>ParentCategoryID   - internal parent category id (optional)- int
#                   <li>EbaySiteID    - Ebay Site identifier - int
#                   <li>Name          - name of category     - nvarchar(255)
#                   <li>CategoryLevel - level from root      - int
#                   <li>IsExpired          - true if category is expired on eBay's side - boolean
#                   <li>IsLeaf             - true if category has no sub categories - boolean
#                   <li>IsVirtual          - true if category is virtual - boolean
#                   <li>AttributeSetID     - attributeset id of category - int
#                   <li>UseSiteConditionAttributeSetID    - true if global sets are valid for Condition - boolean
#                   <li>UseSiteReturnPolicyAttributeSetID - true if global sets are valid for ReturnPolicy - boolean
#               </ul> | reference to hash
#========================================================================================
sub getInfoEbayCategoryByExternalCategoryId {
    my $self = shift;
    my ($ExternalCategoryId) = @_;

    return undef unless defined $ExternalCategoryId;
    my $dbi = $self->dbi;
    my $sql = 'SELECT ebaycategoryid, externalcategoryid, parentcategoryid, name, categorylevel, '.
        'isexpired, isvirtual, isleaf, attributesetid, usesiteconditionattributesetid, '.
        'usesitereturnpolicyattributesetid '.
        'FROM ebaycategory WHERE ebaysiteid = ? AND externalcategoryid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID, $ExternalCategoryId],
        ['Integer', 'Float', 'Integer', 'String', 'Integer', 'Boolean', 'Boolean', 'Boolean',
            'Integer', 'Boolean', 'Boolean'
        ]
    );
    return undef unless scalar @$aaRows;
    my $aRow = $aaRows->[0];
    my %Category;
    @Category{qw (EbayCategoryID ExternalCategoryId ParentCategoryID Name CategoryLevel
        IsExpired IsVirtual IsLeaf AttributeSetID UseSiteConditionAttributeSetID UseSiteReturnPolicyAttributeSetID )} = @$aRow;

    return \%Category;
}

#========================================================================================
# function     getInfoAttributeSetByAttributeSetId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttributeSet = $EbayCategoryTree->getInfoAttributeSetByAttributeSetId($AttributeSetID);
#----------------------------------------------------------------------------------------
# description  Returns the attributeset by attributeset id. If no attributeset
#               exists for the given attributeset id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $AttributeSetID | Attributeset Id | int
# return       $hAttributeset | with following keys:
#               <ul>
#                   <li>AttributeSetID     - internal attribute set id - int
#                   <li>ExternalAttributeSetID     - external attributeset id - int
#                   <li>EbaySiteID    - Ebay Site identifier - int
#                   <li>AttributeSetVersion   - AttributeSet version - int
#                   <li>Name          - name of attributeset     - nvarchar(255)
#               </ul> | reference to hash
#========================================================================================
sub getInfoAttributeSetByAttributeSetId {
    my $self = shift;
    my ($AttributeSetID) = @_;

    return undef unless defined $AttributeSetID;
    my $dbi = $self->dbi;
    my $sql = 'SELECT attributesetid, externalattributesetid, ebaysiteid, attributesetversion, name '.
        'FROM attributeset WHERE ebaysiteid = ? AND attributesetid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID, $AttributeSetID],
        ['Integer', 'Integer', 'Integer', 'Integer', 'String']
    );
    return undef unless scalar @$aaRows;
    my $aRow = $aaRows->[0];
    my %AttributeSet;
    @AttributeSet{qw (AttributeSetID ExternalAttributeSetID EbaySiteID AttributeSetVersion Name)} = @$aRow;

    return \%AttributeSet;
}

#========================================================================================
# function     getInfoAttributesByAttributeSetId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahAttributes = $EbayCategoryTree->getInfoAttributesByAttributeSetId($AttributeSetID);
#----------------------------------------------------------------------------------------
# description  Returns the attributes by attributeset id. If no attribute
#               exists for the given attributeset id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $AttributeSetID | Attributeset Id | int
# return       $hAttributes | with following keys:
#               <ul>
#                   <li>AttributeID         - internal attribute id - int
#                   <li>AttributeSetID      - internal attributeset id - int
#                   <li>ExternalAttributeID - external attribute id - int
#                   <li>ParentID            - external parent attribute id - int
#                   <li>Type                - attribute type - int
#                   <li>EditType            - edit type - int
#                   <li>Label               - label - string
#                   <li>LabelVisible        - is label visible - boolean
#                   <li>Message             - attribute message - string
#                   <li>IsRequired          - is this attribute mandatory - boolean
#                   <li>CanDisplayOther     - can this attribute trigger displaying an other field - boolean
#                   <li>WidgetType          - widget type - string
#                   <li>InputType           - type of input - string
#                   <li>InputColumns        - columns of input - int
#                   <li>InputSize           - size of input - int
#                   <li>InputDisplaySize    - display size of input - int
#                   <li>InputMaxLength      - max length of input- int
#                   <li>DefaultUnitOfMeasureFormatString  - default measurement unit - int
#               </ul> | reference to hash
#========================================================================================
sub getInfoAttributesByAttributeSetId {
    my $self = shift;
    my ($AttributeSetID) = @_;

    return undef unless defined $AttributeSetID;
    my $dbi = $self->dbi;
    my $sql = 'SELECT attributeid, attributesetid, externalattributeid, parentid, type, edittype, label, labelvisible, '.
        'message, isrequired, candisplayother, widgettype, inputtype, inputcolumns, inputsize, inputdisplaysize, '.
        'inputmaxlength, defaultunitofmeasureformatstring '.
        'FROM attribute WHERE attributesetid = ? order by attributeid';
    my $dbResult = $dbi->execute(
        $sql,
        [$AttributeSetID],
        ['Integer', 'Integer', 'Integer', 'Integer', 'Integer', 'Integer', 'String', 'Boolean', 'String', 'Boolean',
            'Boolean', 'String', 'String', 'Integer', 'Integer', 'Integer', 'Integer', 'Integer']
    );

    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (AttributeID AttributeSetID ExternalAttributeID ParentID Type EditType Label LabelVisible
            Message IsRequired CanDisplayOther WidgetType InputType InputColumns InputSize
            InputDisplaySize InputMaxLength DefaultUnitOfMeasureFormatString )]
    );

}

#========================================================================================
# function     getInfoAttributeValuesByAttributeId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahAttributeValues = $EbayCategoryTree->getInfoAttributeValuesByAttributeId($AttributeID);
#----------------------------------------------------------------------------------------
# description  Returns the attributevalues by attribute id. If no value
#               exists for the given attribute id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $AttributeID | Attribute Id | int
# return       $hAttributes | with following keys:
#               <ul>
#                   <li>ValueID             - internal value id - int
#                   <li>AttributeID         - internal attribute id - int
#                   <li>ExternalValueID     - external value id - int
#                   <li>Name                - the Value itself - string
#               </ul> | reference to hash
#========================================================================================
sub getInfoAttributeValuesByAttributeId {
    my $self = shift;
    my ($AttributeID) = @_;

    return undef unless defined $AttributeID;
    my $dbi = $self->dbi;
    my $sql = 'SELECT valueid, attributeid, externalvalueid, name '.
        'FROM attributevalue WHERE attributeid = ? order by externalvalueid ';
    my $dbResult = $dbi->execute(
        $sql,
        [$AttributeID],
        ['Integer', 'Integer', 'Integer', 'String']
    );
    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (ValueID AttributeID ExternalValueID Name)]
    );

}

#========================================================================================
# function     getInfoAttributeValueDependenciesByAttributeId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahAttributeValues = $EbayCategoryTree->getInfoAttributeValueDependenciesByAttributeId($AttributeID);
#----------------------------------------------------------------------------------------
# description  Returns the attributevaluedependencies by attribute id. If no value
#               exists for the given attribute id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $AttributeID | Attribute Id | int
# return       $hAttributesValueDependencies | with following keys:
#               <ul>
#                   <li>ExternalChildAttributeID    - external child attribute id - int
#                   <li>ParentValueID               - internal parent value id - int
#                   <li>ExternalParentValueID       - external parent value id - int
#                   <li>ChildValueID                - internal child value id - int
#                   <li>ExternalChildValueID        - internal child value id - int
#                   <li>Name                        - the Value itself - string
#               </ul> | reference to hash
#========================================================================================
sub getInfoAttributeValueDependenciesByAttributeId {
    my $self = shift;
    my ($AttributeID) = @_;

    return undef unless defined $AttributeID;
    my $dbi = $self->dbi;
    my $dbResult = $dbi->execute( <<END_SQL,[$AttributeID],['Integer', 'Integer', 'Integer', 'Integer', 'Integer', 'String']);
        SELECT aa.externalattributeid, avdp.parentvalueid, av.externalvalueid, avdp.childvalueid, avv.externalvalueid, avdp.name FROM attributevaluedependency avdp
        INNER JOIN attributevalue av ON av.valueid = avdp.parentvalueid
        INNER JOIN attribute a ON a.attributeid = av.attributeid
        INNER JOIN attributevalue avv ON avv.valueid = avdp.childvalueid
        INNER JOIN attribute aa ON aa.attributeid = avv.attributeid
        WHERE a.attributeid=?
        ORDER by avdp.parentvalueid,avv.externalvalueid
END_SQL

    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (ExternalChildAttributeID ParentValueID ExternalParentValueID ChildValueID ExternalChildValueID Name)]
    );
}

#========================================================================================
# function     getInfoAttributeValidationRulesByAttributeId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahAttributeValues = $EbayCategoryTree->getInfoAttributeValidationRulesByAttributeId($AttributeID);
#----------------------------------------------------------------------------------------
# description  Returns the attributevalidationrules by attribute id. If no value
#               exists for the given attribute id then "undef" is returned.
#----------------------------------------------------------------------------------------
# input        $AttributeID | Attribute Id | int
# return       $hAttributeValidationRules | with following keys:
#               <ul>
#                   <li>AttributeID                 - internal attribute id - int
#                   <li>ExternalValidationRuleName  - validation rule name  - string
#                   <li>Length                      - length of value - int
#                   <li>Min                         - min value - int
#                   <li>Max                         - max value - int
#                   <li>Mask                        - mask for value (like AAA-XXXX) - string
#                   <li>PeerAttributeID             - coresponding attribute id (date range check) - int
#                   <li>Precision                   - value display format - int
#                   <li>Regex                       - expression to check the value - string
#                   <li>Required                    - is value required - boolean
#                   <li>Separator                   - specifies a separator - int
#               </ul> | reference to hash
#========================================================================================
sub getInfoAttributeValidationRulesByAttributeId {
    my $self = shift;
    my ($AttributeID) = @_;

    return undef unless defined $AttributeID;
    my $dbi = $self->dbi;
    my $dbResult = $dbi->execute( <<END_SQL,[$AttributeID],['Integer', 'Integer', 'Integer', 'Integer', 'Integer', 'Integer', 'String']);
        SELECT avr.attributeid, avr.externalvalidationrulename, avr.length, avr.min, avr.max, avr.mask, avr.peerattributeid, avr.precision, avr.regex, avr.required, avr.separator
        FROM attributevalidationrules avr
        WHERE avr.attributeid=?
END_SQL

    return GetLoopFromDoubleArray(
        $dbResult,
        [qw (AttributeID ExternalValidationRuleName Length Min Max Mask PeerAttributeID Precision Regex Required Separator)]
    );

}

#========================================================================================
# function     deleteEbayCategory
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->deleteEbayCategory($EbayCategoryID);
#----------------------------------------------------------------------------------------
# description  Deletes a(n) EbayCategory.
#----------------------------------------------------------------------------------------
# input        $EbayCategoryID | internal CategoryID (not the eBay Category Id) (primary key) | int
#========================================================================================
sub deleteEbayCategory {
    my $self = shift;
    my ($EbayCategoryID) = @_;

    my $dbi = $self->dbi;
    $dbi->do('DELETE FROM ebaycategory WHERE ebaycategoryid=?', [$EbayCategoryID]);

    return;
}

#========================================================================================
# function     deleteAllAttributeSets
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->deleteAllAttributeSets();
#----------------------------------------------------------------------------------------
# description  Deletes all AttrbiuteSets
#----------------------------------------------------------------------------------------
# input        -
#========================================================================================
sub deleteAllAttributeSets {
    my $self = shift;

    my $dbi = $self->dbi;
    $dbi->transaction( sub {

        # delete all Attributesets for a site
        my $aAllAttributeSetIDs = $self->getAllAttributeSetIDs();
        foreach my $InternalAttributeSetID (@$aAllAttributeSetIDs) {
            #LogDebug('delete ',$InternalAttributeSetID);
            $self->deleteAttributeSet($InternalAttributeSetID);
        }

        # set categories.attributeset to NULL
        my $sql = 'UPDATE ebaycategory '.
            'SET attributesetid = ? '.
            'WHERE ebaysiteid = ?';
        $dbi->do( $sql,[ undef ,$self->siteID] );

        # delete Version
        $self->setAttributeSystemVersion(undef);
    });

    return;
}

#========================================================================================
# function     deleteAttributeSet
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->deleteAttributeSet($InternalAttributeSetID);
#----------------------------------------------------------------------------------------
# description  Deletes an AttributeSet.
#----------------------------------------------------------------------------------------
# input        $AttributeSetID | internal AttributeSetID (not the eBay AttributeSet Id) (primary key) | int
#========================================================================================
sub deleteAttributeSet {
    my $self = shift;
    my ($InternalAttributeSetID) = @_;
    my $dbi = $self->dbi;

    #LogDebug('deleteAttributeSet: $InternalAttributeSetID',$InternalAttributeSetID);

    $dbi->transaction( sub {
        # delete attributevalidationrules
        $dbi->do(<<END_SQL, [$InternalAttributeSetID]);
            DELETE FROM attributevalidationrules
            WHERE attributeid IN(
                SELECT DISTINCT avr.attributeid
                FROM attributevalidationrules avr
                INNER JOIN attribute att ON att.attributeid = avr.attributeid
                WHERE att.attributesetid=?
            )
END_SQL

        # delete attributevaluedependency
        $dbi->do(<<END_SQL, [$InternalAttributeSetID]);
            DELETE FROM attributevaluedependency
            WHERE parentvalueid in(
                SELECT DISTINCT avdp.parentvalueid FROM attributevaluedependency avdp
                INNER JOIN attributevalue atv ON avdp.parentvalueid = atv.valueid
                INNER JOIN attribute att ON att.attributeid = atv.attributeid
                WHERE att.attributesetid=?
            )
END_SQL

        # delete attributevalue
        $dbi->do(<<END_SQL, [$InternalAttributeSetID]);
            SELECT * FROM attributevalue
            WHERE valueid in(
                SELECT av.valueid FROM attributevalue av
                INNER JOIN attribute att ON att.attributeid = av.attributeid
                WHERE att.attributesetid=?
            )
END_SQL

        # delete attribute
        $dbi->do(   'DELETE FROM attribute WHERE attributesetid=?',[$InternalAttributeSetID]);

        # delete attributeset
        $dbi->do(   'DELETE FROM attributeset WHERE attributesetid=?',[$InternalAttributeSetID]);

    });

    return;
}

#========================================================================================
# function     getCategoryPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahPath = $EbayCategoryTree->getCategoryPath( $CategoryID, $MaxDepth );
#----------------------------------------------------------------------------------------
# description  Builds a category path from the given category to the root category.
#----------------------------------------------------------------------------------------
# input        $CategoryID | identifier of category | int
# input        $MaxDepth | restricts the length of the path (optional) | int
# return       $ahPath | with following keys:
#               <ul>
#                   <li>EbayCategoryID     - internal category id - int
#                   <li>ExternalCategoryId - external category Id - numeric(20)
#                   <li>ParentCategoryID   - internal parent category id (optional)- int
#                   <li>Name          - name of category     - nvarchar(255)
#                   <li>CategoryLevel - level from root      - int
#                   <li>IsExpired          - true if category is expired on eBay's side - boolean
#                   <li>IsLeaf             - true if category has no sub categories - boolean
#                   <li>IsVirtual          - true if category is virtual - boolean
#                   <li>AttributeSetID     - attributeset id of category - int
#                   <li>UseSiteConditionAttributeSetID    - true if global sets are valid for Condition - boolean
#                   <li>UseSiteReturnPolicyAttributeSetID - true if global sets are valid for ReturnPolicy - boolean
#               </ul> | reference to hash
#========================================================================================
sub getCategoryPath {
    my $self = shift;
    my ($CategoryID, $MaxDepth) = @_;

    my $ExistsEbayCategory = $self->existsEbayCategory($CategoryID);
    if (not $CategoryID or not $ExistsEbayCategory or (defined $MaxDepth and $MaxDepth == 0)) {
        return [];
    }

    my $category = $self->getInfoEbayCategory( $CategoryID );

    my $aPath = [];
    if (defined $MaxDepth) {
        $aPath = $self->getCategoryPath( $category->{'ParentCategoryID'}, $MaxDepth - 1 );
    } else {
        $aPath = $self->getCategoryPath( $category->{'ParentCategoryID'} );
    }
    push @$aPath, $category;

    return $aPath;
}

#========================================================================================
# function     getCategoryListVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CategoryListVersion = $EbayCategoryTree->getCategoryListVersion;
#----------------------------------------------------------------------------------------
# description  Returns version of an environments category list.
#----------------------------------------------------------------------------------------
# return       $CategoryListVersion | category list version | int
#========================================================================================
sub getCategoryListVersion {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT categorylistversion FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;
}




#========================================================================================
# function     getAttributeSystemVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $AttributeSystemVersion = $EbayCategoryTree->getAttributeSystemVersion;
#----------------------------------------------------------------------------------------
# description  Returns version of the category CS mappings for the site.
#----------------------------------------------------------------------------------------
# return       $AttributeSystemVersion | Attribute mappings version | int
#========================================================================================
sub getAttributeSystemVersion {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT attributesystemversion FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;
}

#========================================================================================
# function     getAttributeSetVersionByExternalAttributeSetID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $AttributeSetVersion = $EbayCategoryTree->getAttributeSetVersionByExternalAttributeSetID($ID);
#----------------------------------------------------------------------------------------
# description  Returns version of the AttributeSet.
#----------------------------------------------------------------------------------------
# input        $ExternalAttributeSetID | ebay attributeset id | Integer
# return       $AttributeSetVersion | AttributeSet version | int
#========================================================================================
sub getAttributeSetVersionByExternalAttributeSetID {
    my $self = shift;
    my ($ExternalAttributeSetID) = @_;

    my $dbi = $self->dbi;
    my $sql = 'SELECT attributesetversion FROM attributeset WHERE ebaysiteid = ? '.
        'AND externalattributesetid = ? ';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID, $ExternalAttributeSetID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;
}


#========================================================================================
# function     setAttributeSystemVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->setAttributeSystemVersion(1234);
#----------------------------------------------------------------------------------------
# description  Sets Attribute AttributeSystemVersion.
#----------------------------------------------------------------------------------------
# input        $AttributeSystemVersion | attribute system version | Integer
#========================================================================================
sub setAttributeSystemVersion {
    my $self = shift;
    my ($AttributeSystemVersion) = @_;

    my $dbi = $self->dbi;
    my $sql = 'UPDATE ebaysite SET attributesystemversion = ? WHERE ebaysiteid = ?';
    $dbi->do(
        $sql,
        [$AttributeSystemVersion, $self->siteID]
    );

    return;
}

#========================================================================================
# function     getAllAttributeSetIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aAllAttributeSetIDs = $EbayCategoryTree->getAllAttributeSetIDs();
#----------------------------------------------------------------------------------------
# description  Returns all AttributeSet IDs from table attributeset .
#----------------------------------------------------------------------------------------
# return       $aAllAttributeSetIDs | all AttributeSet IDs | ref.array
#========================================================================================
sub getAllAttributeSetIDs {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'select attributesetid FROM attributeset WHERE ebaysiteid = ? and attributesetid is not NULL';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    return [] unless scalar @$aaRows;

    my $aRet;
    foreach my $aRow (@$aaRows) {
        push @$aRet,$aRow->[0] if defined $aRow;
    }
    return $aRet;

}

#========================================================================================
# function     getUnusedAttributeSetIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aAllAttributeSetIDs = $EbayCategoryTree->getUnusedAttributeSetIDs();
#----------------------------------------------------------------------------------------
# description  Returns all unused AttributeSet IDs for a Site .
#----------------------------------------------------------------------------------------
# return       $aAllUnusedAttributeSetIDs | all unused AttributeSet IDs | ref.array
#========================================================================================
sub getUnusedAttributeSetIDs {
    my $self = shift;

    my $dbi = $self->dbi;
    my $aaRows = $dbi->execute( <<END_SQL,[$self->siteID]);
        SELECT attributesetid FROM attributeset
        WHERE attributesetid NOT IN(SELECT attributesetid FROM ebaycategory)
        AND ebaysiteid = ?
END_SQL

    return [] unless scalar @$aaRows;

    my $aRet;
    foreach my $aRow (@$aaRows) {
        push @$aRet,$aRow->[0] if defined $aRow;
    }
    return $aRet;

}

#========================================================================================
# function     getAttributeSetIDByExternalCategoryID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $AttributeSetID = $EbayCategoryTree->getAttributeSetIDByExternalCategoryID($ID);
#----------------------------------------------------------------------------------------
# description  Returns all used AttributeSet IDs .
#----------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------
# input        $ExternalCategoryID | ebay category id | int
# return       $AttributeSetID | internal AttributeSet ID | int
#========================================================================================
sub getAttributeSetIDByExternalCategoryID {
    my $self = shift;
    my ($CategoryID) = @_;


    my $dbi = $self->dbi;
    my $sql = 'select attributesetid FROM ebaycategory WHERE ebaysiteid = ? and externalcategoryid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID,$CategoryID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;

}


#========================================================================================
# function     getAttributeSetIDByExtAttrSetIDAndVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $AttributeSetID = $EbayCategoryTree->getAttributeSetIDByExtAttrSetIDAndVersion($ExtSetID,$AttSetVersion);
#----------------------------------------------------------------------------------------
# description  Returns AttributeSetID by EbaySite, AttributeSetVersion and ExtAttrSetID .
#----------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------
# input        $ExternalAttributeSetID | ebay attribute set id | int
# input        $AttributeSetVersion | ebay attribute set version id | int
# return       $AttributeSetID | internal AttributeSet ID | int
#========================================================================================
sub getAttributeSetIDByExtAttrSetIDAndVersion {
    my $self = shift;
    my ($ExternalAttributeSetID,$AttributeSetVersion) = @_;


    my $dbi = $self->dbi;
    my $sql =   'select attributesetid FROM attributeset '.
        'WHERE ebaysiteid = ? '.
        'AND externalattributesetid = ? '.
        'AND attributesetversion = ? ';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID,$ExternalAttributeSetID,$AttributeSetVersion]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;

}

#========================================================================================
# function     getCategoryListUpdateTimeEbay
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CategoryListUpdateTimeEbay = $EbayCategoryTree->getCategoryListUpdateTimeEbay;
#----------------------------------------------------------------------------------------
# description  Returns time when list was last updated by eBay.
#----------------------------------------------------------------------------------------
# return       $CategoryListUpdateTimeEbay | time of last ebay update | DateTime
#========================================================================================
sub getCategoryListUpdateTimeEbay {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT categorylistupdatetimeebay FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    my $Value = $aaRows->[0]->[0];
    return unless defined $Value;

    my $UpdateTimeEbay = ConvertFromEbayDate($Value);

    return $UpdateTimeEbay;
}

#========================================================================================
# function     getCategoryListUpdateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CategoryListUpdateTime = $EbayCategoryTree->getCategoryListUpdateTime;
#----------------------------------------------------------------------------------------
# description  Returns time when list was last updated by server.
#----------------------------------------------------------------------------------------
# return       $getCategoryListUpdateTime | time of last server update | DateTime
#========================================================================================
sub getCategoryListUpdateTime {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT categorylistupdatetime FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );
    my $Value = $aaRows->[0]->[0];
    return unless defined $Value;

    my $UpdateTimeServer = ConvertFromEbayDate($Value);

    return $UpdateTimeServer;
}

#========================================================================================
# function     getCategoryListIsUpdating
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CategoryListIsUpdating = $EbayCategoryTree->getCategoryListIsUpdating;
#----------------------------------------------------------------------------------------
# description  Returns true if category list is currently updating.
#----------------------------------------------------------------------------------------
# return       $getCategoryListIsUpdating | true if update is in process | Boolean
#========================================================================================
sub getCategoryListIsUpdating {
    my $self = shift;

    my $dbi = $self->dbi;
    my $sql = 'SELECT categorylistisupdating FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;
}

#========================================================================================
# function     setCategoryListIsUpdating
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->setCategoryListIsUpdating(1);
#----------------------------------------------------------------------------------------
# description  Sets Attribute CategoryListIsUpdating true if parameter "1" is passed on
#               to the function.
#----------------------------------------------------------------------------------------
# input        $CategoryListIsUpdating | true if update is in process | Boolean
#========================================================================================
sub setCategoryListIsUpdating {
    my $self = shift;
    my ($CategoryListIsUpdating) = @_;

    my $dbi = $self->dbi;
    my $sql = 'UPDATE ebaysite SET categorylistisupdating = ? WHERE ebaysiteid = ?';
    $dbi->do(
        $sql,
        [$CategoryListIsUpdating ,$self->siteID]
    );

    return;
}

#========================================================================================
# function     setCategoryListUpdateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->setCategoryListUpdateTime(DateTime->now());
#----------------------------------------------------------------------------------------
# description  Sets CategoryListUpdateTime.
#----------------------------------------------------------------------------------------
# input        $CategoryListUpdateTime | time of update | DateTime
#========================================================================================
sub setCategoryListUpdateTime {
    my $self = shift;
    my ($CategoryListUpdateTime) = @_;

    my $dbi = $self->dbi;
    my $sql = 'UPDATE ebaysite SET categorylistupdatetime = ? WHERE ebaysiteid = ?';
    $dbi->do(
        $sql,
        [$CategoryListUpdateTime->ymd.' '.$CategoryListUpdateTime->hms, $self->siteID]
    );

    return;
}

#========================================================================================
# function     setCategoryListCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayCategoryTree->setCategoryListCount(0);
#----------------------------------------------------------------------------------------
# description  Sets Attribute CategoryListCount.
#----------------------------------------------------------------------------------------
# input        $CategoryListCount | current list count | Integer
#========================================================================================
sub setCategoryListCount {
    my $self = shift;
    my ($CategoryListCount) = @_;

    my $dbi = $self->dbi;
    my $sql = 'UPDATE ebaysite SET categorylistcount = ? WHERE ebaysiteid = ?';
    $dbi->do(
        $sql,
        [$CategoryListCount, $self->siteID]
    );

    return;
}

#========================================================================================
# function     getCategoryListCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CategoryListCount = $EbayCategoryTree->getCategoryListCount;
#----------------------------------------------------------------------------------------
# description  Returns total count of categories for an environment.
#----------------------------------------------------------------------------------------
# return       $CategoryListCount | category list count | int
#========================================================================================
sub getCategoryListCount {
    my $self = shift;

    my $dbi = $self->dbi;

    my $sql = 'SELECT categorylistcount FROM ebaysite WHERE ebaysiteid = ?';
    my $aaRows = $dbi->execute(
        $sql,
        [$self->siteID]
    );

    return undef unless scalar @$aaRows;
    my $Value = $aaRows->[0]->[0];

    return $Value;
}

1;
