#========================================================================================
# §package      DE_EPAGES::Ebay::API::Object::RemoteRESTEbay
# §state        public
#----------------------------------------------------------------------------------------
# §description  object for REST requests for ebay OAuth calls and ebay catalog API
#========================================================================================
package DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use base DE_EPAGES::Ebay::API::Object::RemoteEbay;

use strict;
use HTTP::Request;
use HTTP::Status qw(HTTP_OK);
use JSON::XS qw ( decode_json encode_json );
use LWP::UserAgent;
use Encode qw ( encode );

use DE_EPAGES::Core::API::Error          qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log            qw( GetLog GetCommunicationLog );
use DE_EPAGES::Core::API::PerlTools      qw( Compare GetCurrentDateTime AddDuration );
use DE_EPAGES::Core::API::TestParameter  qw( TestString );
use DE_EPAGES::Core::API::Url            qw ( BuildParameterString );
use DE_EPAGES::Core::API::Warning        qw( Warning );
use DE_EPAGES::Ebay::API::Constants; #EBAY_OAUTH_SCOPE EBAY_REST_ROUTES REMOTE_CATALOG_REST_SANDBOX_API_URL REMOTE_CATALOG_REST_API_URL EBAY_OAUTH_SCOPE 
use DE_EPAGES::Object::API::Factory             qw( LoadRootObject );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::WebService::API::JSONRestClient;
use DE_EPAGES::WebService::API::Codec::FormUrlencoded;
use URI::Escape qw (uri_unescape);


#ERRORDEFINITION
sub ErrorDefinition { return {}
}
#ERRORDEFINITION END

#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;

    return $class->SUPER::new(%options);
}

#========================================================================================
# §function     getOAuthTokensByAuthCode
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getOAuthTokensByAuthCode($AuthCode);
#----------------------------------------------------------------------------------------
# §description  get the OAuth2 token and the refresh token by the auth code
#               https://api.sandbox.ebay.com/identity/v1/oauth2/token is called
#               see also https://developer.ebay.com/api-docs/static/oauth-auth-code-grant-request.html
#----------------------------------------------------------------------------------------
# §input        $AuthCode | ebay auth code for fetching token | string
# §return       $result | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>expires_in - access token exiration time - string
#                   <li>refresh_token - refresh token for refreshing access token - string
#                   <li>refresh_token_expires_in - refresh token exiration time - string
#                   <li>token_type - token type - string
#               </ul> | ref.hash
#========================================================================================
sub getOAuthTokensByAuthCode {
    my $self = shift;
    my ($AuthCode) = @_;

    TestString("AuthCode",$AuthCode);

    # get the Runames (Live and SandBox) from Store
    my $hEbayAttributes = LoadRootObject()->get( [qw (EbayRuName EbaySandBoxRuName) ] );

    my $Environment     = $self->environment;
    my $EbayRuName      = $Environment->get('IsSandBox') ? $hEbayAttributes->{'EbaySandBoxRuName'} : $hEbayAttributes->{'EbayRuName'};
    my $EbayEndpointURL = $Environment->get('OAuthTokenRequestEndpoint');

    my $aAuthorization  = $self->_getBasicAuthorization;
    my $hBody  =  { 'grant_type'  => 'authorization_code',
                    'code'        => $AuthCode,
                    'redirect_uri'=> $EbayRuName
                  };

    my $Response = $self->sendTokenRequest($EbayEndpointURL, $aAuthorization, $hBody);
    return $Response->code == HTTP_OK ? decode_json($Response->content) : undef;    

}

#========================================================================================
# §function     oAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->oAuthToken();
#----------------------------------------------------------------------------------------
# §description  returns the current OAuth2 token for the environment
#               the token is automatically refreshed and saved if the
#               token is expired
#               this token is used to call the ebay trading and catalog api
#----------------------------------------------------------------------------------------
# §return       $result | OAuth2 token for the environment | string
#========================================================================================
sub oAuthToken {
    my $self = shift;

    return undef unless $self->existsRefreshToken();

    #test if we have a valid OAuth token
    # oherwise renew the token and return
    my $Prefix = $self->variableNamePrefix;
    my $TokenExpirationTime = $self->useProvider 
        ? $self->store->get($Prefix.'ProviderOAuthTokenExpiryDate')
        : $self->shopEnvironment->get('EbayOAuthTokenExpiryDate');

    # renew token if needed (    # 0 = eq, -1: $a < ,1:$a>)
    if ( Compare('DateTime',GetCurrentDateTime(),$TokenExpirationTime) >= 0 ){
        $self->renewOAuthToken();
    }

    return $self->_getOAuthToken();
}

#========================================================================================
# §function     refreshToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->refreshToken();
#----------------------------------------------------------------------------------------
# §description  returns the token for refreshing the OAuth token
#----------------------------------------------------------------------------------------
# §return       $result | refresh token | string
#========================================================================================
sub refreshToken {
    my $self = shift;

    my $Prefix = $self->variableNamePrefix;
    return $self->useProvider 
        ? $self->store->get($Prefix.'ProviderToken')
        : $self->shopEnvironment->get('EbayToken');
}

#========================================================================================
# §function     renewOAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->renewOAuthToken();
#----------------------------------------------------------------------------------------
# §description  renews the OAuth token and save the token on the environment
#========================================================================================
sub renewOAuthToken {
    my $self = shift;

    my $EbayEndpointURL = $self->environment->get('OAuthTokenRequestEndpoint');
    my $aAuthorization  = $self->_getBasicAuthorization;
    my $hBody  =  { 'grant_type'    => 'refresh_token',
                    'refresh_token' => $self->refreshToken(),
                    'scope'         => uri_unescape(EBAY_OAUTH_SCOPE)
                  };

    my $Response = $self->sendTokenRequest($EbayEndpointURL, $aAuthorization, $hBody);
    my $hOAuthToken = $Response->code == HTTP_OK ? decode_json($Response->content) : undef;    

    # save the renewed token
    $self->saveOAuthToken($hOAuthToken) if $Response->code == HTTP_OK;

    return;

}

#========================================================================================
# §function     _getBasicAuthorization
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       my $aAuthorization = $self->_getBasicAuthorization();
#----------------------------------------------------------------------------------------
# §description  returns the authoritzation values
#----------------------------------------------------------------------------------------
# §return       $aAuthorization | authorization values | ref array
#========================================================================================
sub  _getBasicAuthorization {
    my $self = shift;

    my $hEbayAttributes = LoadRootObject()->get( [qw (EbayCertId EbaySandBoxCertId EbayAppId EbaySandBoxAppId) ] );

    my $Environment = $self->environment;
    my $IsSandbox   = $Environment->get('IsSandBox');
    my $EbayAppId   = $IsSandbox ? $hEbayAttributes->{'EbaySandBoxAppId'}  : $hEbayAttributes->{'EbayAppId'}; 
    my $EbayCertId  = $IsSandbox ? $hEbayAttributes->{'EbaySandBoxCertId'} : $hEbayAttributes->{'EbayCertId'}; 
    
    my $aAuthorization = [$EbayAppId, $EbayCertId];

    return $aAuthorization;

}

#========================================================================================
# §function     saveOAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->saveOAuthToken($hTokenInfo);
#----------------------------------------------------------------------------------------
# §description  saves the OAuth token (and the refresh token) on the environment
# §input        $hTokenInfo | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>expires_in - access token exiration time - string
#                   <li>refresh_token - (optional) refresh token for refreshing access token - string
#                   <li>refresh_token_expires_in - (optional) refresh token exiration time - string
#                   <li>token_type - token type - string
#               </ul> | ref.hash
#========================================================================================
sub saveOAuthToken {
    my $self = shift;
    my ($hOAuthTokens) = @_;

    # save the correct expiration time for the tokens (minus 10 seconds)
    my $Now = GetCurrentDateTime();
    my $EbayRefreshToken =  $hOAuthTokens->{'refresh_token'} // undef;
    my $EbayRefreshTokenExpiryDate = defined $EbayRefreshToken? AddDuration($Now->clone, 'seconds', $hOAuthTokens->{'refresh_token_expires_in'} - 10 ) : undef;
    
    my $EbayAccessToken =  $hOAuthTokens->{'access_token'};
    my $EbayAccessTokenExpiryDate = AddDuration($Now->clone, 'seconds', $hOAuthTokens->{'expires_in'} - 10 );
    
    my $hValues = {};

    # save all tokens for the provider or the shop environment
    if ($self->useProvider) {
        GetLog->debug('Save new OAuth token for Provider');
        $hValues->{$self->variableNamePrefix.'ProviderOAuthToken'}           = $EbayAccessToken;
        $hValues->{$self->variableNamePrefix.'ProviderOAuthTokenExpiryDate'} = $EbayAccessTokenExpiryDate;


        # save renew token if needed
        if(defined $EbayRefreshToken){
            $hValues->{$self->variableNamePrefix.'ProviderToken'}           = $EbayRefreshToken;
            $hValues->{$self->variableNamePrefix.'ProviderTokenExpiryDate'} = $EbayRefreshTokenExpiryDate;
            # we don't use user data anymore
            $hValues->{$self->variableNamePrefix.'ProviderLogin'}           = undef;
            $hValues->{$self->variableNamePrefix.'ProviderPassword'}        = undef;
        }
        $self->store->set( $hValues );

        eval { SynchronizeCache(); };
        Warning( GetError() ) if ExistsError();
    } else {
        # save token of shop environment
        GetLog->debug('Save new OAuth token for Shop Environment');
        $hValues->{'EbayOAuthToken'}           = $EbayAccessToken;
        $hValues->{'EbayOAuthTokenExpiryDate'} = $EbayAccessTokenExpiryDate;
        
        # save renew token if needed
        if(defined $EbayRefreshToken){
            $hValues->{'EbayToken'}           = $EbayRefreshToken;
            $hValues->{'EbayTokenExpiryDate'} = $EbayRefreshTokenExpiryDate;
            # we don't use username ans sessionid anymore
            $hValues->{'EbayUserName'}        = undef;
            $hValues->{'EbaySessionID'}       = undef;
        }
        $self->shopEnvironment->set( $hValues );
    }

    return ;
}

#========================================================================================
# §function     sendTokenRequest
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->sendTokenRequest($EbayEndpointURL, $Header, $Body);
#----------------------------------------------------------------------------------------
# §description  send request to the given endpoint with the given parameters
#----------------------------------------------------------------------------------------
# §input        $EbayEndpointURL | api endpoint | string
# §input        $aAuthorization | Basic authorization parameters | ref.array
# §input        $Body | body parameters | ref.hash
# §return       $Response | HTTP::Response object | HTTP::Response
#========================================================================================
sub sendTokenRequest {
    my $self = shift;
    my ($EbayEndpointURL, $aAuthorization, $Body) = @_;

    my $Client = DE_EPAGES::WebService::API::JSONRestClient->new($EbayEndpointURL,{
                'BasicAuthentication' => $aAuthorization,
                'Codec'               => DE_EPAGES::WebService::API::Codec::FormUrlencoded->new(),
                'UserAgent'           => LWP::UserAgent->new(timeout => 10)
                });

    my $Request = $Client->buildRequest('POST', undef, $Body);
    $Request->header('Accept' => 'application/json');

    my $LogCategory = $self->_getLogCategory();
    my $Response;
    $self->logCommunication( $LogCategory, $Request);
    $Response = $Client->send($Request);
    $self->logCommunication( $LogCategory, $Response);

    # test if we have a 200 as rc otherwise throw errors
    $self->_testResponse($Response);
 
    return $Response;

}

sub _testResponse {
    my $self = shift;
    my ($Response) = @_;

    if( $Response->is_success ) {
        Error('HttpError', {
            Code => $Response->code,
            StatusLine    => $Response->status_line,
            ContentLength => $Response->content_length,
            Response      => $Response,
        }) unless $Response->content_length;
    }
    else {
        Error('HttpError', {
            Code       => $Response->code,
            StatusLine => $Response->status_line,
            Response   => $Response,
            Content    => $Response->content,
        });
    }
}

#========================================================================================
# §function     logCommunication
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $RemoteRESTEbay->logCommunication( $Category, $HTTPObject );
# §example      $RemoteRESTEbay->logCommunication( 'Ebay::RemoteEbay', $Response );
#----------------------------------------------------------------------------------------
# §description  Logs HTTP communications.
#----------------------------------------------------------------------------------------
# §input        $Category | log category, undef for no logging | string
# §input        $Message | Message to be logged | HTTP::Request or HTTP::Response object
#========================================================================================
sub logCommunication {
    shift;
    my ($Category, $Message) = @_;

    my $Logger = GetCommunicationLog($Category);

    if (defined $Category && $Logger->is_debug) {
            if( $Message->isa('HTTP::Request') ) {
                $Logger->debug( "Request headers:\n" . $Message->headers_as_string );
                $Logger->debug( $Message->method .' '. $Message->uri ."\n". $Message->content);
            }
            else {
                $Logger->debug( "Response headers:\n" . $Message->headers_as_string );
                $Logger->debug( $Message->status_line . "\n" . $Message->content);
            }
    }
    return;
}

#========================================================================================
# §function     existsRefreshToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->existsRefreshToken();
#----------------------------------------------------------------------------------------
# §description  test if the refresh token exists
#----------------------------------------------------------------------------------------
# §return       $Result | 1 if exists, 0 if not exists | Boolean
#========================================================================================
sub existsRefreshToken {
    my $self = shift;
    my ($hParams) = @_;

    return 1 if $self->refreshToken();
    return 0;
}

#========================================================================================
# §function     existsOAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->existsOAuthToken();
#----------------------------------------------------------------------------------------
# §description  test if the OAuth token exists
#----------------------------------------------------------------------------------------
# §return       $Result | 1 if exists, 0 if not exists | Boolean
#========================================================================================
sub existsOAuthToken {
    my $self = shift;
    my ($hParams) = @_;

    my $OAuthToken = $self->_getOAuthToken();

    return 1 if $OAuthToken;
    return 0;
}

#========================================================================================
# §function     _getOAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $OAuthToken = $self->_getOAuthToken();
#----------------------------------------------------------------------------------------
# §description  returns the OAuthToken from the environment
#----------------------------------------------------------------------------------------
# §return       $OAuthToken | the token | string
#========================================================================================
sub _getOAuthToken {
    my $self = shift;

    return $self->useProvider
       ? $self->store->get($self->variableNamePrefix.'ProviderOAuthToken')
       : $self->shopEnvironment->get('EbayOAuthToken');
}

#========================================================================================
# §function     searchProductByGtin
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $hProductData = $self->searchProductByGtin($Gtin);
#----------------------------------------------------------------------------------------
# §description  returns the result of the product search by gtin
#----------------------------------------------------------------------------------------
# §input        $Gtin | GTIN number (can be an EAN, UPC, ISBN) | string
# §return       $hProductData | found products | ref. hash
#========================================================================================
sub searchProductByGtin {
    my $self = shift;
    my ($Gtin) = @_;

    my $Route = EBAY_REST_ROUTES->{'CATALOG_PRODUCT_SEARCH'};

    my $Env         = $self->environment;
    my $IsSandbox   = $Env->get('IsSandBox');
    my $CatalogEndpointURL = $IsSandbox ? REMOTE_CATALOG_REST_SANDBOX_API_URL : REMOTE_CATALOG_REST_API_URL;

    my $hParams = {};
    # add a locale like de-DE
    my $AcceptLanguage = $Env->get('LocaleID');
    $AcceptLanguage =~ s/_/-/g;

    # add a Marketplace id like EBAY_US or EBAY_DE  to the http header
    #(use the 1st two letters of the alias from the EbayEnvironments)
    my $hMarketPlaceID = {'X-EBAY-C-MARKETPLACE-ID' => 'EBAY_' . substr($AcceptLanguage,-2)};
    $hParams->{'Header'} = {
            'Accept-Language'         => $AcceptLanguage,
            %$hMarketPlaceID
    };
    $hParams->{'Query'} = {'gtin' => $Gtin};

    return $self->sendRESTRequest('GET', $CatalogEndpointURL, $Route, $hParams);

}

#========================================================================================
# §function     sendRESTRequest
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->sendRESTRequest($Method, $APIUrl, $Route, $hParams);
# §syntax       $Shop->sendRESTRequest('POST', 'https://api.sandbox.ebay.com/commerce/catalog/v1_beta',
#                                      /product_summary/search, $hParams);
#----------------------------------------------------------------------------------------
# §description  sends a request to the ebay API server with the given parameter
#               Throws API_CALL_ERROR if the request was not successfull.
#----------------------------------------------------------------------------------------
# §input        $Method | request method  | string
# §input        $APIUrl | API url  | string
# §input        $Route | route to object | string
# §input        $hParams | with following keys:
#               <ul>
#                   <li>Header - http header parameters (optional) - ref.hash
#                   <li>Query - query parameters for call (optional) - ref.hash
#                   <li>Content - payload for call (optional) - ref.hash
#                   <li>IgnoreCodes - ignore this response codes (do not produce errors) - ref.array
#               </ul> | ref.hash
# §return       $Response | JSON decoded response or undef | ref.hash
#========================================================================================
sub sendRESTRequest {
    my $self = shift;
    my ($Method, $APIUrl, $Route, $hParams) = @_;

    $Method //= 'GET';
    TestString( 'Method', $Method );
    TestString( 'APIUrl', $APIUrl );
    TestString( 'Route', $Route );

    # encode content
    my $EncodedContent;
    $EncodedContent = encode('UTF-8',encode_json($hParams->{'Content'})) if defined $hParams->{'Content'};

    # build the correct request URL by adding route and query params
    my $RequestURL = $self->_buildRequestURL($APIUrl, $Route, $hParams->{'Query'});
    my $Request = HTTP::Request->new($Method => $RequestURL);

    my $hHTTPHeader = { 'Authorization' => 'Bearer ' . $self->oAuthToken(),
                        'Content-Type'  => 'application/json',
                        'Accept'        => 'application/json',
                        'Accept-Charset'=> 'utf-8',
                     };

    # add additional header values
    foreach my $Key (keys(%{  $hParams->{'Header'} })) {
        $hHTTPHeader->{$Key} = $hParams->{'Header'}->{$Key};
    }
    $Request->header(%$hHTTPHeader);

    # adds the content and send request
    $Request->content($EncodedContent) if defined $EncodedContent;

    my $LogCategory = $self->_getLogCategory();
    $self->logCommunication( $LogCategory, $Request);
    my $Response = LWP::UserAgent->new->request($Request);
    $self->logCommunication( $LogCategory, $Response);

    # error handling
    unless ($Response->is_success) {
        my $ErrorObject = {
            'Method'  => $Method,
            'ApiURL'  => $RequestURL,
            'Content' => $EncodedContent,
            'Status'  => $Response->code,
            'Message' => $Response->content
        };

        # test if we have response codes to ignore
        if ($Response->code ~~ @{ $hParams->{'IgnoreCodes'} }) {
            Warning('API_CALL_ERROR', $ErrorObject);
            return;
        }

        Error('API_CALL_ERROR', $ErrorObject);
    }

    # return decoded content if we have a GET request only
    my $JSONResponse = $Response->decoded_content;
    return $JSONResponse ? decode_json($JSONResponse) : undef;

}

#========================================================================================
# §function     _getLogCategory
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->_getLogCategory();
#----------------------------------------------------------------------------------------
# §description  returns the log category if logging is enabled
#----------------------------------------------------------------------------------------
# §return       $LogCategory |  log category for enabled logging | string
#========================================================================================
sub _getLogCategory {
    my $self = shift;

    return undef unless LoadRootObject()->get('EbayEnableLogging');
    my $ShopID = $self->useProvider ? 'Provider' : $self->shopEnvironment->getSite()->id;

    return  "Ebay::RemoteEbay::$ShopID";
}

#========================================================================================
# §function     _buildRequestURL
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->_buildRequestURL($APIUrl, $Route, $hQueryParams);
# §syntax       $Shop->_buildRequestURL($APIUrl, '/product_summary/search', {'gtin'=> '08154711'});
#----------------------------------------------------------------------------------------
# §description  inserts the $Route into the given $APIUrl and adds adds $hQueryParams
#               afterwards to the URL
#----------------------------------------------------------------------------------------
# §input        $APIUrl | API url | string
# §input        $Route | route to add to the url | string
# §input        $hQueryParams | locale id for call (optional) | string
# §return       $APIUrl |  api URL including the route and the query params | string
#========================================================================================
sub _buildRequestURL{
    my $self = shift;
    my ($APIUrl, $Route, $hQueryParams) = @_;

    my $ParamString = defined $hQueryParams ? BuildParameterString($hQueryParams) . '&' : '';
    $APIUrl .= "$Route?$ParamString";

    return $APIUrl;
}

1;
