use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Ebay::API::Cartridge;
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeEbayNoOAuthTokenNotifications
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeEbayNoOAuthTokenNotifications();
#----------------------------------------------------------------------------------------
# description      Adds the ebay notification for missing OAuth authentication to all Unity shops
#========================================================================================
sub InitializeEbayNoOAuthTokenNotifications {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::Ebay::API::Cartridge->new;
    foreach my $ShopID (@$aShopIDs) {
        eval {
            my $Shop = LoadObject($ShopID);
            # import notification
            ImportObjects($Cartridge->xmlDirectory().'/ShopNotificationEbayNoOAuthToken.xml',
                $Shop->pathString);
            # check all ebay environments for missing OAuthTokens
            my $EbayNoOAuthTokenNotification = $Shop->folder('ShopNotifications')->child('EbayNoOAuthToken');
            my $Reactivate = 0;

            foreach my $EbayShopEnvironment (@{$Shop->get('EbayShopEnvironments')}){
                # inserts the notification if we have an token but not a oauth token
                if($EbayShopEnvironment->get('EbayToken') and (not $EbayShopEnvironment->get('EbayOAuthToken')) ){
                    $EbayNoOAuthTokenNotification->addObject($EbayShopEnvironment);
                    $Reactivate = 1;
                }
            }
            $EbayNoOAuthTokenNotification->activate() if $Reactivate;
        };
        warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".")
            if ExistsError();
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeEbayNoOAuthTokenNotifications(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds the ebay notification for missing OAuth tokens for all environments on all Unity shops.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
