#========================================================================================
# package      DE_EPAGES::Ebay::UI::EbayAuction
# state        private
# base         DE_EPAGES::Presentation::UI::Object
# description  UI functions for handling ebay auctions
#========================================================================================
package DE_EPAGES::Ebay::UI::EbayAuction;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Ebay qw ( UpdateShopAuctions );
use DE_EPAGES::Ebay::API::EbayAuction qw ( InsertNewAuctionFromProduct );
use DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Product::API::Shop qw ( GetProduct );
use DE_EPAGES::Ebay::API::Tools qw( GetAuctionsSellerProfiles );

#ERRORDEFINITION
sub ErrorDefinition { return {
    'EbaySiteNeedsCurrencyThatIsNotEnabledOnShop' => 'Currencies do not match. The choosen eBay Site needs #EbayCurrency as currency, in the shop are only these currencies avialable: #ShopCurrencies.', ##no critic qw(ProhibitLongLines)
    }
}
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $auction = $Servlet->object;

    my $ShopEnv = $auction->get('EbayShopEnvironment');
    my $Env = $ShopEnv->get('EbayEnvironment');

    # get info of category
    my $ExternalCategoryId = $auction->get('EbayCategoryId');
    my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($Env);
    my $categoryId = $EbayCategoryTree->getPKeyEbayCategoryByExternalCategoryId($ExternalCategoryId);
    my $hCategory = $EbayCategoryTree->getInfoEbayCategory($categoryId);
    if (defined $hCategory) {
        $hCategory->{'Path'} = $EbayCategoryTree->getCategoryPath($hCategory->{'EbayCategoryID'} );
        $Servlet->params('EbayCategory', $hCategory);
    }

    eval {
        # load EbayStore categories
        my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
            'ShopEnvironment' => $ShopEnv,
        );

        my $StoreOwner = $RemoteEbay->getUser(['StoreOwner'])->{'StoreOwner'};
        if ($StoreOwner eq 'true') {
            my $aStoreCategories = $RemoteEbay->getStore()->{'CustomCategories'};
            $Servlet->vars('EbayStoreCategories', $aStoreCategories);
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        if($Error->code eq 'EbayNoToken') {
            $Servlet->form->mapFormError({
                'Form'    => 'EbayNoToken',
                'Name'    => 'EbayNoToken',
                'Message' => $Error->vars->{'SOAPMessage'} || $Error->shortMessage
            }, $Error);
        } elsif ($Error->code eq 'EbayRequestNotSuccessful') {
            my $aEbayErrors = $Error->vars->{'EbayErrors'};

            # check if the sole error is 'User has no shop'
            if (scalar @$aEbayErrors == 1 and grep {$_->{'ErrorCode'} == EBAY_ERROR_SELLER_NO_STORE} @$aEbayErrors) {
                GetLog->debug('Shop has no EbayStore.');
            } else {
                # all other warnings will be shown to the merchant
                $Servlet->vars('EbayErrors', $aEbayErrors);
            }
        } else {
            Error($Error);
        }
    }
}

sub NewAuction {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $IsOneClickAuction = $Servlet->form->value('OneClickAuction');

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles([$Shop->get('EbayDefaultAuction')]);

    eval {
        my $auction = InsertNewAuctionFromProduct(
            $Shop,
            $Product,
            undef,
            $Shop->get('EbayDefaultAuction')->getSellerProfile($hhBusinessPolicySellerProfiles)
        );

        if ($IsOneClickAuction) {
            #verify auction, executes error if one
            _verifyAuction($Servlet, $auction);
            $Servlet->form->executeFormError;

            #upload to eBay
            _uploadAuction($Servlet, $auction);
            $Servlet->form->executeFormError;
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ($Error->code eq 'CanNotAuctionDownloadProduct');
        $Servlet->form->mapFormError({ 'Name' => 'NewAuction' }, $Error);
    }
    $Servlet->form->executeFormError;
}

sub CloneAuctions {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $MasterObject->getSite();
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayAuction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles(\@ListObjects);

    foreach my $Auction (@ListObjects) {
        # clone the auction
        $Auction->clone({
            'Alias'             => $Auction->parent->nextAlias('EbayAuction%s'),
            'Status'            => AUCTION_EDITABLE,
            'ExternalAuctionId' => undef,
            'ScheduleType'      => 0,
            'ScheduleTime'      => undef,
            'CurrentBid'        => 0,
            'BidCount'          => 0,
            'StartTime'         => GetCurrentDBHandle()->currentDateTime,
            'EndTime'           => undef,
        },$Auction->getSellerProfile($hhBusinessPolicySellerProfiles));
    }

    return;
}

sub CreateScheduleRepeat {
    my $self = shift;
    my ($Servlet) = @_;

    # nothing to do
    return if ! $Servlet->params('ListObjectID');

    my $MasterObject = $Servlet->object;
    my $Shop;
    # MasterObject can be Shop or Product
    if ($MasterObject->instanceOf('Product' )) {
        $Shop = $MasterObject->getSite();
    } elsif ($MasterObject->instanceOf('Shop' )) {
        $Shop = $MasterObject;
    } else {
        Error('ServletObjectNotProductOrShop');
    }

    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayAuction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    my $showWarnings = 0;
    foreach my $auction (@ListObjects) {
        my $status = $auction->get('Status');
        next if ($status == AUCTION_SCHEDULED);
        $Form->addFormError( {
                'Name'      => 'CanNotRepeatUnscheduledAuctions',
                'Message'   => 'You can only repeat auctions that are scheduled.',
                'Reason'    => 'CanNotRepeatUnscheduledAuctions',
                'Auction'   => $auction->tleHash,
        } );
        $showWarnings = 1;
    }
    if ($showWarnings) {
        $Form->executeFormError;
        return;
    }

    $Servlet->vars('SCHEDULEREPEAT',1);
    $Servlet->vars('ChangeObjects', [ map { $_->tleHash } @ListObjects]);

    # show Schedule Repeat Form
    return if (not $Servlet->params('ScheduleRepeatTimeType'));

    # validate input data and get unformatted values
    my $ScheduleForm = $Servlet->form->form( $Shop, 'ScheduleRepeatForm' );

    my $count        = $ScheduleForm->{'ScheduleRepeatCount'};
    my $type         = $ScheduleForm->{'ScheduleRepeatTimeType'};
    my $value        = $ScheduleForm->{'ScheduleRepeatTimeValue'};

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles(\@ListObjects);

    # now do clone
    foreach my $Auction (@ListObjects) {
        my $duration = DateTime::Duration->new( $type => $value );
        my $startTime = $Auction->get('ScheduleTime');

        for (my $i=0; $i<$count; $i++) {
            $startTime = $startTime + $duration;

            # clone the auction
            $Auction->clone({
                'Alias'             => $Auction->parent->nextAlias('EbayAuction%s'),
                'ExternalAuctionId' => undef,
                'ScheduleType'      => 1,
                'ScheduleTime'      => $startTime,
                'CurrentBid'        => 0,
                'BidCount'          => 0,
                'StartTime'         => $startTime,
                'EndTime'           => undef,
            },$Auction->getSellerProfile($hhBusinessPolicySellerProfiles));
        }
    }

    $Servlet->vars('SCHEDULEREPEAT',0);

    return;
}

sub _verifyAuction {
    my ($Servlet, $auction) = @_;

    if ($auction->get('Status') > AUCTION_VERIFIED) {
        $Servlet->form->addFormError({
            'Name'      => 'CanNotVerifyRunningAuctions',
            'Message'   => 'You can only verify auctions that are not yet uploaded.',
            'Reason'    => 'CanNotVerifyRunningAuctions',
            'Auction'   => $auction->tleHash,
        });
        return 0;
    }
    # check if product exists in shop
    if (not defined $auction->get('Product')) {
        $Servlet->form->addFormError({
            'Name'      => 'NoProductInAuction',
            'Message'   => 'You can only verify auctions which have a product assigned.',
            'Reason'    => 'NoProductInAuction',
            'Auction'   => $auction->tleHash,
        });
        return 0;
    }

    my $result;
    eval { $result = $auction->verify; };
    if (ExistsError()) {
        my $Error = GetError();
        $auction->set({'Status' => AUCTION_EDITABLE});
        if ($Error->code ~~ [qw(EbayRequestFailed EbayTokenError EbayNoToken)]) {
            $Servlet->form->mapFormError({
                'Form'      => 'VerifyAuction',
                'Name'      => 'Token',
                'Message'   => $Error->vars->{'SOAPMessage'},
                'Auction'   => $auction->tleHash,
            }, $Error);
        } elsif ($Error->code eq 'EbayRequestNotSuccessful') {
            foreach my $error (@{$Error->{'Vars'}->{'EbayErrors'}}) {
                $Servlet->form->addFormError({
                    'Form'    => 'VerifyAuction',
                    'Name'      => 'EbayRequestNotSuccessful',
                    'Message'   => $error->{'LongMessage'},
                    'Reason'    => 'EbayRequestNotSuccessful',
                    'Code'      => $error->{'ErrorCode'},
                    'Auction'   => $auction->tleHash,
                });
            }
        } elsif ($Error->code eq 'EbayFeatureLimitExceeded') {
            $Servlet->form->addFormError({
                'Form'      => 'VerifyAuction',
                'Name'      => 'EbayFeatureLimitExceeded',
                'Reason'    => 'EbayFeatureLimitExceeded',
            });
        } else {
            Error($Error);

        }

        return 0;
    } else {

        # test if we have warnings and add they to the form
        if (exists $result->{'Warnings'}) {
            foreach my $warning (@{$result->{'Warnings'}}) {
                $Servlet->form->addFormError({
                    'Name'      => 'EbayRequestSuccessfulWithWarnings',
                    'Message'   => $warning->{'LongMessage'},
                    'Reason'    => 'EbayRequestSuccessfulWithWarnings',
                    'Code'      => $warning->{'ErrorCode'},
                    'MoreInfo'  => $warning->{'Message'},
                    'Auction'   => $auction->tleHash,
                });
            }
        }

        # set status
        $auction->set({
            'Status' => $auction->get('ScheduleType') == 0 ? AUCTION_VERIFIED : AUCTION_SCHEDULED
        });
    }
    return 1;
}

sub _uploadAuction {
    my ($Servlet, $auction) = @_;

    my $status = $auction->get('Status');

    # check if auction is verified or scheduled
    unless ($status & (AUCTION_VERIFIED | AUCTION_SCHEDULED)) {
        return if ($status != AUCTION_EDITABLE);
        $Servlet->form->addFormError({
            'Name'      => 'CanNotUploadUnverifiedAuctions',
            'Message'   => 'You can only upload auctions that are verified.',
            'Reason'    => 'CanNotUploadUnverifiedAuctions',
            'Auction'   => $auction->tleHash,
        });
        return;
    }
    # check if product exists in shop
    if (not defined $auction->get('Product')) {
        $Servlet->form->addFormError({
            'Name'      => 'NoProductInAuction',
            'Message'   => 'You can only upload auctions which have a product assigned.',
            'Reason'    => 'NoProductInAuction',
            'Auction'   => $auction->tleHash,
        });
        return;
    }

    my $result;
    my $itemId;
    eval {
        $result = $auction->upload;
    };
    if (ExistsError()) {
        my $Error = GetError();
        $auction->set({'Status' => AUCTION_EDITABLE});

        if ($Error->code eq 'EbayRequestFailed') {
            $Servlet->form->addFormError({
                'Name'      => 'EbayRequestFailed',
                'Message'   => $Error->{'Vars'}->{'SOAPMessage'},
                'Reason'    => 'EbayRequestFailed',
                'Auction'   => $auction->tleHash,
            });
        } elsif ($Error->code eq 'EbayRequestNotSuccessful') {
            foreach my $error (@{$Error->{'Vars'}->{'EbayErrors'}}) {
                $Servlet->form->addFormError({
                    'Name'      => 'EbayRequestNotSuccessful',
                    'Message'   => $error->{'LongMessage'},
                    'Reason'    => 'EbayRequestNotSuccessful',
                    'Code'      => $error->{'ErrorCode'},
                    'Auction'   => $auction->tleHash,
                });
            }
        } elsif ($Error->code eq 'EbayFeatureLimitExceeded') {
            $Servlet->form->addFormError({
                'Name'      => 'EbayFeatureLimitExceeded',
                'Reason'    => 'EbayFeatureLimitExceeded',
            });

        } elsif ($Error->code eq 'EbayNoToken') {
            $Servlet->form->mapFormError({
                'Form'    => 'EbayNoToken',
                'Name'    => 'EbayNoToken',
                'Message' => $Error->vars->{'SOAPMessage'} || $Error->shortMessage
            }, $Error);
        } else {
            Error($Error);
        }
    } else {

       # test if we have warnings and add they to the form
        if (exists $result->{'Warnings'}) {
            foreach my $warning (@{$result->{'Warnings'}}) {
                $Servlet->form->addFormError({
                    'Name'      => 'EbayRequestSuccessfulWithWarnings',
                    'Message'   => $warning->{'LongMessage'},
                    'Reason'    => 'EbayRequestSuccessfulWithWarnings',
                    'Code'      => $warning->{'ErrorCode'},
                    'MoreInfo'  => $warning->{'Message'},
                    'Auction'   => $auction->tleHash,
                });
            }
        }

        $itemId = $result->{'ItemID'};
        $Servlet->params('ItemID', $itemId);
    }
    return $itemId;
}

sub VerifyAuctions {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $MasterObject->getSite();
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayAuction' ) } @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    _verifyAuction($Servlet, $_) foreach @ListObjects;
    $Servlet->form->executeFormError;

    return;
}

sub UploadAuctions {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $MasterObject->getSite();
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayAuction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    my $ItemIdsString;
    foreach my $ListObject (@ListObjects) {
        my $itemId = _uploadAuction($Servlet, $ListObject);
        if (defined $itemId) {
            $ItemIdsString = defined $ItemIdsString ? "$ItemIdsString, $itemId" : $itemId;
        }
    }
    $Servlet->params('ItemID', $ItemIdsString);
    $Servlet->form->executeFormError;

    return;
}

sub UpdateAuctions {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;

    my $Shop;
    # MasterObject can be Shop or Product
    if ($MasterObject->instanceOf('Product' )) {
        $Shop = $MasterObject->getSite();
    } elsif ($MasterObject->instanceOf('Shop' )) {
        $Shop = $MasterObject;
    } else {
        Error('ServletObjectNotProductOrShop');
    }

    eval {
        UpdateShopAuctions( $Shop );
    };
    if (ExistsError()) {
        my $Error = GetError();

        $Servlet->form->mapFormError({
            'Form'    => 'UpdateShopAuctions',
            'Name'    => 'UpdateShopAuctions',
            'Message' => $Error->vars->{'SOAPMessage'} || $Error->shortMessage
        }, $Error);

        LogError($Error) unless $Error->code ~~ ['EbayNoToken', 'EbayTokenError'];
    }
    $Servlet->form->executeFormError;
    return;
}

sub AuctionProductsFromTray {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $Objects = $Tray->objectsByClass(LoadClassByAlias('Product'));
    my $Form = $Servlet->form;

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles([$Shop->get('EbayDefaultAuction')]);

    my $auction;
    foreach my $Product (@$Objects) {
        eval {
            $auction = InsertNewAuctionFromProduct(
                $Shop,
                $Product,
                undef,
                $Shop->get('EbayDefaultAuction')->getSellerProfile($hhBusinessPolicySellerProfiles)
            );
        };
        if (ExistsError()) {
            my $Error = GetError();

            Error($Error) unless $Error->code ~~ [qw(
                CanNotAuctionDownloadProduct CanNotAuctionMasterProduct NoShopEnvironment
            )];
            $Form->mapFormError({'Name' => 'AuctionProductsFromTray'}, $Error);
        }
    }
    $Form->executeFormError;
    my $IsOneClickAuction = $Form->value('OneClickAuction');

    if ($IsOneClickAuction and defined $auction) {
        #verify auction, executes error if one
        _verifyAuction($Servlet, $auction);
        $Servlet->form->executeFormError;

        #upload to eBay
        _uploadAuction($Servlet, $auction);
        $Servlet->form->executeFormError;
    }
    return;
}

sub AssignUserTemplate {
    my $self = shift;
    my ($Servlet) = @_;

    my $auction = $Servlet->object;
    my $Shop = $auction->getSite();

    my $EbayUserTemplateID = $Servlet->form->value('EbayUserTemplate');
    return unless $EbayUserTemplateID;

    my $UserTemplate = $Shop->loadObject( $EbayUserTemplateID, 'EbayUserTemplate' );

    # get lang of ebay site
    my $langid = GetPKeyLanguageByCode(lc $auction->get('EbayShopEnvironment')->get('EbayEnvironment')->get('LanguageID'));
    # use default shop lang if ebay site's lang isn't active in the shop
    $langid = undef unless $langid ~~ [ map {$_->{'LanguageID'}} @{$auction->getSite()->get('Languages')} ];

    my $processedContent = $UserTemplate->process( $auction->get('Product'), $langid );

    $auction->set({
        'EbayUserTemplate'  => $UserTemplate,
        'Description'       => $processedContent,
    });

    return 1;
}

sub AuctionProduct {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    my $Alias = $Form->value('NewProduct');
    return unless defined $Alias and $Alias =~ /\S/;

    my $TargetProduct = GetProduct($Shop, $Alias);

    if (not defined $TargetProduct) {
        $Servlet->form->addFormError({
            'Reason'=> 'ERROR_INVALIDPRODUCT',
            'Form'  => 'NoForm',
            'Name'  => 'NewProduct',
        });
    }

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles([$Shop->get('EbayDefaultAuction')]);

    my $auction;

    eval {
        $auction = InsertNewAuctionFromProduct(
            $Shop,
            $TargetProduct,
            undef,
            $Shop->get('EbayDefaultAuction')->getSellerProfile($hhBusinessPolicySellerProfiles)
        ) if defined $TargetProduct;
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code ~~ [qw(
            CanNotAuctionMasterProduct
            NoShopEnvironment
            CanNotAuctionDownloadProduct
        )];
        $Form->mapFormError({'Name' => 'AuctionProduct'}, $Error);
    }
    $Form->executeFormError;
    my $IsOneClickAuction = $Form->value('OneClickAuction');
    if ($IsOneClickAuction  and defined $auction) {
        #verify auction, executes error if one
        _verifyAuction($Servlet, $auction);
        $Servlet->form->executeFormError;

        #upload to eBay
        _uploadAuction($Servlet, $auction);
        $Servlet->form->executeFormError;
    }
    return;
}

sub ExportAuctions{
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $AuctionFolder = $Shop->folder('EbayAuctions');
    my $hValues = $Servlet->form->form($AuctionFolder, 'ListedObjects');

    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayAuction' ) } @{$hValues->{'ListObjectIDs'}};
    return return unless scalar @ListObjects;

    $Servlet->vars('SelectedObjects', [map {$_->tleHash} @ListObjects]);
    $Servlet->vars('CSVExportImportType', 'EbayAuction');
    $Servlet->vars('EXPORT_SELECTED_OBJECTS', 1);
    $Servlet->vars('ViewActionAbort', $Servlet->params('ViewAction'));
    $Servlet->vars('ObjectIDAbort', $Shop->id);
    $Servlet->vars('ViewAction', 'MBO-ViewCSVExportImportEbayAuction');
    $Servlet->vars('ObjectID', $Shop->id);
    return;
}




1;
