#========================================================================================
# §package      DE_EPAGES::EbayProvider::API::Object::RemoteStore
# §state        public
#----------------------------------------------------------------------------------------
# §description  stub object for web service requests to store db
#========================================================================================
package DE_EPAGES::EbayProvider::API::Object::RemoteStore;

use strict;
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient;

# attributes: Store
sub new {
    my $class = shift;
    my %options = @_;
    TestObject('Store', $options{'Store'});

    return bless \%options, $class;
}

#========================================================================================
# §function     store
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $store = $self->store;
#----------------------------------------------------------------------------------------
# §description  Returns the store database object
#----------------------------------------------------------------------------------------
# §return       $Store | Store object | object
#========================================================================================
sub store {
    return shift->{'Store'};
}

#========================================================================================
# §function     factoryService
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->factoryService( $Store );
#----------------------------------------------------------------------------------------
# §description  instanciates StoreWebserviceClient
#----------------------------------------------------------------------------------------
# §input        $Store | Store (optional) | object
# §return       $WebServiceClient | webservice client | object
#========================================================================================
sub factoryService {
    my $self = shift;
    my ($Store) = @_;
    $Store ||= $self->store;

    my $soap = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/FactoryService/2004/08'
    );

    return $soap;
}

#========================================================================================
# §function     objectService
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->objectService( $Store );
#----------------------------------------------------------------------------------------
# §description  Instanciates StoreWebServiceClient for ObjectService
#----------------------------------------------------------------------------------------
# §input        $Store | Store (optional) | object
# §return       $WebServiceClient | webservice client | object
#========================================================================================
sub objectService {
    my $self = shift;
    my ($Store) = @_;
    $Store ||= $self->store;

    my $soap = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/ObjectService/2004/08'
    );

    return $soap;
}

#========================================================================================
# §function     ebayConfigService
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->ebayConfigService( $Store );
#----------------------------------------------------------------------------------------
# §description  Instanticates webservice client for EbayConfigService
#----------------------------------------------------------------------------------------
# §input        $Store | store (optional) | object
# §return       $WebServiceClient | webservice client | object
#========================================================================================
sub ebayConfigService {
    my $self = shift;
    my ($Store) = @_;
    $Store ||= $self->store;

    my $service = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/EbayConfigService/2004/10'
    );

    return $service;
}

#========================================================================================
# §function     get
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->get( ['Name'], Language->id );
#----------------------------------------------------------------------------------------
# §description  gets attributes of store
#----------------------------------------------------------------------------------------
# §input        $aNames | list of keys | ref.array.string
# §input        $LanguageID | Language ID (otional) | integer
# §return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub get {
    my $self = shift;
    my ( $aNames, $LanguageID ) = @_;

    my $hValues = {};

    my $Store = $self->{'Store'};
    if (defined $Store) {

        my $RootObjectID = $self->factoryService->rootObject()->result;

        $hValues = $self->objectService->get( $RootObjectID, $aNames, $LanguageID )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $hValues;
}

#========================================================================================
# §function     getEbayEnvironment
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getEbayEnvironment();
#----------------------------------------------------------------------------------------
# §description  Gets info for all ebay environments on the store
#----------------------------------------------------------------------------------------
# §return       $aValues | tleHash of each EbayEnvironment | ref.array.hash
#========================================================================================
sub getEbayEnvironment {
    my $self = shift;

    my $aValues = [];

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $aValues = $self->ebayConfigService->getEbayEnvironment()->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $aValues;
}

#========================================================================================
# §function     set
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->set( {'Key' => 'Value'}, Language->id );
#----------------------------------------------------------------------------------------
# §description  sets attributes of store
#----------------------------------------------------------------------------------------
# §input        $hValues | key-value pairs | ref.hash
# §input        $LanguageID | Language ID (otional) | integer
# §return       $hValues | plain attributes | ref.hash.string
#========================================================================================
sub set {
    my $self = shift;
    my ( $hValues, $LanguageID ) = @_;

    my $Store = $self->{'Store'};
    if (defined $Store) {

        my $RootObjectID = $self->factoryService->rootObject()->result;

        $self->objectService->set( $RootObjectID, $hValues, $LanguageID );
    } else {
        warn sprintf 'No Store defined.';
    }

    return;
}

#========================================================================================
# §function     testConnection
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       if ($self->testConnection( 'US' )) {}
#----------------------------------------------------------------------------------------
# §description  Tests connection to eBay's API gateway
#----------------------------------------------------------------------------------------
# §input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# §return       $result | successful? | boolean
#========================================================================================
sub testConnection {
    my $self = shift;
    my ($EnvironmentAlias) = @_;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->testConnection( $EnvironmentAlias )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

#========================================================================================
# §function     checkEbayToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       if ($self->checkEbayToken( 'US' )) {}
#----------------------------------------------------------------------------------------
# §description  Checks Authentication of an EbayEnvironment
#----------------------------------------------------------------------------------------
# §input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# §return       $result | successful? | boolean
#========================================================================================
sub checkEbayToken {
    my $self = shift;
    my ($EnvironmentAlias) = @_;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->checkEbayToken( $EnvironmentAlias )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

#========================================================================================
# §function     getOAuthTokensByAuthCode
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getOAuthTokensByAuthCode( $AuthCode, $EnvironmentAlias );
#----------------------------------------------------------------------------------------
# §description  fetch a token from ebay
#----------------------------------------------------------------------------------------
# §input        $AuthCode    | authentication code for fetching tokens | string
# §input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# §return       $result | result set (with token) | hash
#========================================================================================
sub getOAuthTokensByAuthCode {
    my $self = shift;
    my ($AuthCode, $EnvironmentAlias) = @_;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->getOAuthTokensByAuthCode( $AuthCode, $EnvironmentAlias )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

#========================================================================================
# §function     saveOAuthToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->saveOAuthToken( $hTokenInfo, $EnvironmentAlias );
#----------------------------------------------------------------------------------------
# §description  save tokens on the provider attributes on the store
#----------------------------------------------------------------------------------------
# §input        $hTokenInfo    | tokens with info | ref hash
# §input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
#========================================================================================
sub saveOAuthToken {
    my $self = shift;
    my ($hTokenInfo, $EnvironmentAlias) = @_;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $self->ebayConfigService->saveOAuthToken( $hTokenInfo, $EnvironmentAlias )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return;
}

#========================================================================================
# §function     getCategories
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getCategories( $env->id );
#----------------------------------------------------------------------------------------
# §description  Starts asynchronous category download for given environment
#----------------------------------------------------------------------------------------
# §input        $EnvironmentID | ObjectID of an EbayEnvironment | string
# §return       $result | request successful submitted? | boolean
#========================================================================================
sub getCategories {
    my $self = shift;
    my ($EnvironmentID) = @_;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->getCategories( $EnvironmentID )->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

#========================================================================================
# §function     getCallTypes
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getCallTypes();
#----------------------------------------------------------------------------------------
# §description  Get all Call Types
#----------------------------------------------------------------------------------------
# §return       $result | array of CallTypeInfos | ref.array.hash
#========================================================================================
sub getCallTypes {
    my $self = shift;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->getCallTypes()->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

#========================================================================================
# §function     getCallStatistics
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getCallStatistics( $hArgs );
#----------------------------------------------------------------------------------------
# §description  Get the call statistics for the input args
#----------------------------------------------------------------------------------------
# §input        $hArgs | with following keys:
#                       <ul>
#                           <li>DateFrom        - start date - datetime string
#                           <li>DateTo          - end date   - datetime string
#                           <li>Interval        - interval of statistics periods - integer
#                           <li>IntervalType    - type of interval (e.g. 'days', 'weeks') - string
#                           <li>EbayCallType    - id of call type - integer
#                           <li>EbayEnvironment - id of ebayenvironment - integer
#                       </ul> | ref.hash
# §return       $result | array of results:
#                       <ul>
#                           <li>StartDate - start date of statistics period - datetime string
#                           <li>Count - result for this period - integer
#                       </ul> | ref.array.hash
#========================================================================================
sub getCallStatistics {
    my $self = shift;
    my ($hArgs) = @_;

    my $result;

    my $Store = $self->{'Store'};
    if (defined $Store) {
        $result = $self->ebayConfigService->getCallStatistics($hArgs)->result;
    } else {
        warn sprintf 'No Store defined.';
    }

    return $result;
}

1;
