#==============================================================================
# package      DE_EPAGES::Moneybookers::UI::BasketOrder
# state        private
#==============================================================================
package DE_EPAGES::Moneybookers::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone isValid );
use DE_EPAGES::Core::API::Url qw (UrlEncodeValue);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log  qw ( LogPayment );
use DE_EPAGES::Moneybookers::API::Tools;
use DE_EPAGES::Moneybookers::API::Constants qw (@SPLIT_GATEWAY_METHODS);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );



#========================================================================================
# function     ViewPaymentMoneybookers
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentMoneybookers($Servlet)
#----------------------------------------------------------------------------------------
# description  Is invoked at checkout payment step (both classic and multi-checkout).
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet |  object
#========================================================================================
sub ViewPaymentMoneybookers {
    my $self    = shift;
    my ($Servlet) = @_;

    my $BasketOrder  = $Servlet->object(); # basket or order
    my $LineItemContainer = $BasketOrder->container();

    return unless defined($LineItemContainer);

    my $Shop = $BasketOrder->get('Shop');
    $self->_logIframeRequestEnvironment($Servlet, $Shop, $BasketOrder, $LineItemContainer);

    return;
}

#========================================================================================
# function     _logIframeRequestEnvironment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_logIframeRequestEnvironment( $Servlet, $Shop, $BasketOrder, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Arrange servlet variables for the iframe request and log iframe data
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# input        $BasketOrder | basket or customer order object | object
# input        $LineItemContainer | line item container object | object
#========================================================================================
sub _logIframeRequestEnvironment {
    my $self = shift;
    my ($Servlet, $Shop, $BasketOrder, $LineItemContainer) = @_;

    my $hPaymentDriverIframeRequestData =
            $self->_setupPaymentDriverVariables($Servlet, $Shop, $BasketOrder, $LineItemContainer);
    my $hOrderIframeRequestData = $self->_setupOrderVariables($Servlet, $LineItemContainer);
    my $hUrlIframeRequestData = $self->_setupUrlVariables($Servlet, $Shop, $BasketOrder, $LineItemContainer);

    my $hIframeRequestData = {
        %{$hPaymentDriverIframeRequestData},
        %{$hOrderIframeRequestData},
        %{$hUrlIframeRequestData}
    };
    my $MethodPathName = (caller(0))[3];
    $self->_logRequest($Servlet, $Shop, $MethodPathName, $hIframeRequestData);
}

#========================================================================================
# function     _setupPaymentDriverVariables
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setupPaymentDriverVariables( $Servlet, $Shop, $BasketOrder, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  collect iframe request data that drives moneybookers behavior as well as
#               account and transaction identifier. 
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object |object
# input        $Shop | shop object |object
# input        $BasketOrder | basket or customer order object | object
# input        $LineItemContainer | line item container object | object
# return       $hIframeData | iframe request data for payment driver | refhash
#========================================================================================
sub _setupPaymentDriverVariables {
    my $self = shift;
    my ($Servlet, $Shop, $BasketOrder, $LineItemContainer) = @_;

    my $CreationDate = $self->_getFormattedCurrentDateTime();

    my $CleanShopName = $self->_generateCleanShopName($Shop);
    my $TransactionId = $self->_generateTransactionId($BasketOrder, $CleanShopName, $CreationDate);

    my $PaymentMethod = $LineItemContainer->get('Payment')->get('PaymentMethod');
    my $hPaymentMethod = $PaymentMethod->get([ qw( Email SendStatusEmail SplitGatewayMethod ) ]);
    my $MoneybookersMethod = $self->_MoneybookersMapSGMiFrame($hPaymentMethod->{'SplitGatewayMethod'});
    my $System = $self->_loadRootObject();

    $Servlet->vars('MoneybookersProviderID', $System->get('Moneybookers_ProviderID'));
    $Servlet->vars('PaymentMethodEmail', $hPaymentMethod->{'Email'});
    $Servlet->vars('SendStatusEmail', $hPaymentMethod->{'SendStatusEmail'});

    if ( $MoneybookersMethod eq 'PAY_KAR') {
        $Servlet->vars('PaymentMethodCode', 'PAY');
        $Servlet->vars('WpfRedirect', '1');
        $Servlet->vars('TransactionId', 'PAYOLUTION_INVOICE-'.$TransactionId);
    } elsif ($MoneybookersMethod eq 'PAY_RAT') {
        $Servlet->vars('PaymentMethodCode', 'PAY');
        $Servlet->vars('WpfRedirect', '1');
        $Servlet->vars('TransactionId', 'PAYOLUTION_INS-'.$TransactionId);
    } else {
        $Servlet->vars('PaymentMethodCode', $MoneybookersMethod);
        $Servlet->vars('WpfRedirect', '0');
        $Servlet->vars('TransactionId',$TransactionId);
    }

    my $hIframeData = {};
    $hIframeData->{'platform'} = $Servlet->vars('MoneybookersProviderID');
    $hIframeData->{'pay_to_email'} = $Servlet->vars('PaymentMethodEmail');
    $hIframeData->{'payment_methods'} = $Servlet->vars('PaymentMethodCode');
    $hIframeData->{'wpf_redirect'} = $Servlet->vars('WpfRedirect');
    $hIframeData->{'transaction_id'} = $Servlet->vars('TransactionId');

    if ($Servlet->vars('SendStatusEmail')) {
        $hIframeData->{'status_url2'} = $Servlet->vars('PaymentMethodEmail');
    }

    return $hIframeData;
}

#========================================================================================
# function     _generateCleanShopName
# state        private
#----------------------------------------------------------------------------------------
# syntax       _generateCleanShopName( $Shop );
#----------------------------------------------------------------------------------------
# description  generate and return a clean shop name by replacing special chars like hyphen
#               (EPG-20280)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $cleanShopName | shop name | string
#========================================================================================
sub _generateCleanShopName {
    my $self = shift;
    my ($Shop) = @_;

    my $CleanShopName = $Shop->get('NameOrAlias');
    $CleanShopName = join '', map{ ord($_)<128 ? $_ : sprintf('&#%d;', ord($_)) } split //, $CleanShopName; ## no critic (ProhibitMagicNumbers)
    $CleanShopName = UrlEncodeValue($CleanShopName);
    $CleanShopName = substr($CleanShopName, 0, 20); ## no critic (ProhibitMagicNumbers)

    return $CleanShopName;
}

#========================================================================================
# function     _generateTransactionId
# state        private
#----------------------------------------------------------------------------------------
# syntax       _generateTransactionId( $BasketOrder, $CleanShopName, $CreationDate );
#----------------------------------------------------------------------------------------
# description  generate and return transaction id based on clean shop name and OrderAlias or LiCID
#----------------------------------------------------------------------------------------
# input        $BasketOrder | basket or order object | object
# input        $CleanShopName | shop name string | string
# input        $CreationDate | formatted date | string
# return       $TransactionId | appropriate transaction id name| string
#========================================================================================
sub _generateTransactionId {
    my $self = shift;
    my ($BasketOrder, $CleanShopName, $CreationDate) = @_;

    if ( $BasketOrder->instanceOf('CustomerOrder') ){
        my $BasketOrderAlias = $BasketOrder->alias();
        return "$CleanShopName $BasketOrderAlias $CreationDate";
    } else {
        return "$CleanShopName $CreationDate";
    }
}

#========================================================================================
# function     _setupOrderVariables
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setupOrderVariables( $Servlet, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  collect iframe request data regarding the order.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object |object
# input        $LineItemContainer | line item container object | object
# return       $hIframeData | extended iframe request data to be logged | refhash
#========================================================================================
sub _setupOrderVariables {
    my $self = shift;
    my ($Servlet, $LineItemContainer) = @_;

    my $hLineItemContainer = $LineItemContainer->get([ qw( Payment ID GrandTotal CurrencyID ) ]);
    my $LineItemPayment = $hLineItemContainer->{'Payment'};
    my $hLineItemPayment = $LineItemPayment->get([ qw( IsHirePurchaseAllowed HirePurchaseNoOfMonths HirePurchaseInstallmentAmount ) ]);

    my $hIframeData = {};
    if ( $hLineItemPayment->{'IsHirePurchaseAllowed'} && isValid($hLineItemPayment->{'HirePurchaseNoOfMonths'}) ) {
        $Servlet->vars('IsHirePurchase', 1);
        $Servlet->vars('HirePurchaseInstallmentAmount', $hLineItemPayment->{'HirePurchaseInstallmentAmount'});
        $Servlet->vars('HirePurchaseInstallmentPeriodAmount', 1);
        $Servlet->vars('HirePurchaseInstallmentPeriodUnit', 'month');

        $hIframeData->{'rec_amount'} = $Servlet->vars('HirePurchaseInstallmentAmount');
        $hIframeData->{'rec_period'} = $Servlet->vars('HirePurchaseInstallmentPeriodAmount');
        $hIframeData->{'rec_cycle'} = $Servlet->vars('HirePurchaseInstallmentPeriodUnit');
    } else {
        $Servlet->vars('IsHirePurchase', 0);
        $Servlet->vars('LineItemContainerID', $hLineItemContainer->{'ID'});
        $Servlet->vars('GrandTotal', $hLineItemContainer->{'GrandTotal'});
        $Servlet->vars('CurrencyID', $hLineItemContainer->{'CurrencyID'});

        $hIframeData->{'lineitemcontainer_id'} = $Servlet->vars('LineItemContainerID');
        $hIframeData->{'amount'} = $Servlet->vars('GrandTotal');
        $hIframeData->{'currency'} = $Servlet->vars('CurrencyID');
    }

    return $hIframeData;
}

#========================================================================================
# function     _setupUrlVariables
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setupUrlVariables( $Servlet, $Shop, $BasketOrder, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  collect iframe request data regarding return url statuses.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object |object
# input        $Shop | shop object |object
# input        $BasketOrder | basket or customer order object | object
# input        $LineItemContainer | line item container object | object
# return       $hIframeData | extended iframe request data to be logged | refhash
#========================================================================================
sub _setupUrlVariables {
    my $self = shift;
    my ($Servlet, $Shop, $BasketOrder, $LineItemContainer) = @_;

    $Servlet->vars('ReturnURL', $self->_buildShopUrl(
            $Shop,
            { 'ChangeAction'  => 'PaymentCallbackMoneybookers', 'LICID' => $LineItemContainer->id() },
            { 'UseSSL' => 1 }
        )
    );
    $Servlet->vars('CancelURL', $self->_buildShopUrl(
            $BasketOrder,
            { 'ViewAction' => 'ViewPaymentMoneybookersCancel' },
            { 'UseSSL' => 1 }
        )
    );
    $Servlet->vars('StatusURL', $self->_buildShopUrl(
            $Shop,
            { 'ChangeAction' => 'PaymentStatusMoneybookers' },
            { 'UseSSL' => 1 }
        )
    );

    my $hIframeData = {};
    $hIframeData->{'return_url'} = $Servlet->vars('ReturnURL');
    $hIframeData->{'cancel_url'} = $Servlet->vars('CancelURL');
    $hIframeData->{'status_url'} = $Servlet->vars('StatusURL');

    return $hIframeData;

}

#========================================================================================
# function     ViewPaymentMoneybookersCancel
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentMoneybookersCancel($Servlet)
#----------------------------------------------------------------------------------------
# description  Is invoked once a customer canceled the payment with Skill or the
#               payment failed and the customer is redirected to the shop.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet |  object
#========================================================================================
sub ViewPaymentMoneybookersCancel {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrOrder = $Servlet->object();
    my $Shop = $BasketOrOrder->get('Shop');

    if($Shop->get('IsUnityRegistered')) {

        my $PaymentLineItem = $BasketOrOrder->container()->get('Payment');
        $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'});

        $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));
    }

    return;
}

#========================================================================================
# function     ViewMultiPaymentMoneybookers
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewMultiPaymentMoneybookers($Servlet)
#----------------------------------------------------------------------------------------
# description  Is invoked at multi-checkout payment step.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet |  object
#========================================================================================
sub ViewMultiPaymentMoneybookers {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    $self->ViewPaymentMoneybookers($Servlet);

    return;
}

#========================================================================================
# function     _MoneybookersMapSGMiFrame
# state        private
#----------------------------------------------------------------------------------------
# syntax       _MoneybookersMapSGMiFrame(SplitGatewayMethod)
# example      _MoneybookersMapSGMiFrame(SplitGatewayMethod)
#----------------------------------------------------------------------------------------
# description  Function maps an internal used SplitGatewayMethod to another known by
#               Moneybookers. Required for "NGP" which should be available as
#               "Online Bank Transfer" for international and as "Onlineberweisung" for
#               German use
#----------------------------------------------------------------------------------------
# input        $SplitGatewayMethod | moneybookers payment method code | string
# return       $Value | an external used splitgateway method | string
#========================================================================================
sub _MoneybookersMapSGMiFrame {
    my $self = shift;
    my ($SplitGatewayMethod) = @_;

    my @retVal =  grep { ! $_->{'isGroup'} && $_->{'ID'} eq $SplitGatewayMethod } @SPLIT_GATEWAY_METHODS;

    return $retVal[0]->{'REAL_SGM'};
}

#========================================================================================
# function     _logRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _logRequest( $Servlet, $Shop, $MethodPath );
#----------------------------------------------------------------------------------------
# description  Log information sent to Skrill.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# input        $MethodPath | method path name | string
# input        $hIframeRequestData | iframe request data to be logged | refhash
#========================================================================================
sub _logRequest {
    my $self = shift;
    my ( $Servlet, $Shop, $MethodPath, $hIframeRequestData ) = @_;

    my $Tools = DE_EPAGES::Moneybookers::API::Tools->new();
    $Tools->LogShopContext(
            $Shop,
            $MethodPath,
            sub { LogPayment('Moneybookers', 'Payment Request', $hIframeRequestData); }
    );
}

#=========================================================================================
# Wrappers
#=========================================================================================
sub _getFormattedCurrentDateTime {
    my $self = shift;

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    my $DateTime = GetCurrentDBHandle()->currentDateTime();
    $DateTime->set_time_zone( GetLocalTimeZone() );

    return $Formatter->format_datetime($DateTime, 'datetime', 'short');
}

sub _loadRootObject {
    my $self = shift;

    return LoadRootObject();
}

sub _buildShopUrl {
    my $self = shift;
    my ($Object, $hParams, $hOptions, $Servlet) = @_;

    BuildShopUrl($Object, $hParams, $hOptions, $Servlet);
}

1;

