#========================================================================================
# package      DE_EPAGES::Object::API::Attributes::Object_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for Object, implements the attributes
#               <ul>
#                   <li>ChildObjects - array of all children</li>
#                   <li>ParentObjects - path from System to this object as array</li>
#                   <li>ObjectPath - path from System to this object as string</li>
#                   <li>AttributeValues - array of all attributes and values, including
#                     attribute types.
#                     Each array element is a hash with these keys:
#                       <ul>
#                         <li>Attribute - attribute definition hash with these keys
#                           <ul>
#                             <li>Name - attribute name</li>
#                             <li>Type - attribute type name</li>
#                             <li>Length - attribute length</li>
#                             <li>...</li>
#                           </ul>
#                         <li>Value - attribute value</li>
#                       </ul>
#                   </li>
#========================================================================================
package DE_EPAGES::Object::API::Attributes::Object_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use feature qw(switch);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::File qw ( GetFileNames GetFileInfo ExistsDirectory GetFileContentLines );
use DE_EPAGES::Object::API::ChildObjectsHash;
use DE_EPAGES::Object::API::Factory qw (LoadRootObject JoinObjectPath GenerateSplittedGUIDPath);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

$ENV{EPAGES_STATIC} = $ENV{EPAGES_SHARED} . '/Static' unless defined $ENV{EPAGES_STATIC}; ##no critic (RequireLocalizedPunctuationVars)

use constant PORT_HTTP => 80;
use constant PORT_HTTPS => 443;

sub ErrorDefinition { return {
    'ObjectCycle' => 'object #ObjectID included in parent cycle',
}}

sub getAttributes { ##no critic (ProhibitExcessComplexity)
    my $self = shift;
    my ($Object, $aNames, $LanguageID) = @_;
    my $hInfo = {};
    my $System = LoadRootObject();
    foreach my $Name ( @$aNames ) {
        given( $Name ) {
            when( 'ID' ) {
                $hInfo->{$Name} = $Object->id;
            }
            when( 'Class' ) {
                $hInfo->{$Name} = $Object->class;
            }
            when( 'Child' ) {
                my %Children;
                tie %Children, 'DE_EPAGES::Object::API::ChildObjectsHash', $Object;
                $hInfo->{$Name} = \%Children;
            }
            when( 'ChildObjects' ) {
                $hInfo->{$Name} = $Object->children;
            }
            when( 'Parent' ) {
                $hInfo->{$Name} = $Object->parent;
            }
            when( 'ParentObjects' ) {
                $hInfo->{$Name} = $Object->parentPath;
            }
            when( 'PathFromSite' ) {
                $hInfo->{$Name} = _PathFromSite( $Object );
            }
            when( 'FileAlias' ) {
                $hInfo->{$Name} = $Object->get('FilePathPart')
                                ? $Object->get('FilePathPart')
                                : GenerateSplittedGUIDPath( $Object->get('GUID') );
            }
            when( 'Path' ) {
                $hInfo->{$Name} = $Object->pathString;
            }
            when( 'RelativePath' ) {
                $hInfo->{$Name} = _RelativePath($Object);
            }
            when( 'FilePath' ) {
                $hInfo->{$Name} = $Object->filePath;
            }
            when( 'NameOrAlias' ) {
                $hInfo->{$Name} = _NameOrAlias( $Object, $LanguageID );
            }
            when( 'HistoryName' ) {
                $hInfo->{$Name} = _NameOrAlias( $Object, $LanguageID );
            }
            when( 'AttributeValues' ) {
                $hInfo->{$Name} = _AttributeValues( $Object, $LanguageID );
            }
            when( 'UserDefinedAttributeValues' ) {
                $hInfo->{$Name} = _UserDefinedAttributeValues( $Object, $LanguageID );
            }
            when( 'StaticPath' ) {
                $hInfo->{$Name} = $Object->id == $System->id
                    ? $ENV{'EPAGES_STATIC'}.'/'.$System->get('StoreName')
                    : $ENV{'EPAGES_STATIC'}.'/'.$System->get('StoreName').'/'.$Object->filePath;
            }
            when( 'PublicPath' ) {
                $hInfo->{$Name} = $Object->id == $System->id
                    ? $ENV{'EPAGES_WEBROOT'}.'/'.$System->get('StoreName')
                    : $ENV{'EPAGES_WEBROOT'}.'/'.$System->get('StoreName').'/'.$Object->filePath;
            }
            when( 'PublicFiles' ) {
                my $pubPath = $Object->get('PublicPath');
                my @Files;
                if( ExistsDirectory($pubPath) ) {
                    @Files = sort { lc($a->{'FileName'}) cmp lc($b->{'FileName'}) }
                        map { GetFileInfo($_) }
                        @{GetFileNames($pubPath)};
                    foreach my $PublicFile (@Files) {
                        $PublicFile->{'URLFileName'} = $PublicFile->{'FileName'} if $PublicFile->{'Extension'} eq '.url';
                        $PublicFile->{'FileName'} = GetFileContentLines($PublicFile->{'Name'},{'binary'=>1})->[0] if $PublicFile->{'Extension'} eq '.url';
                    }
                }
                $hInfo->{$Name} = \@Files;
            }
            when( 'PrivatePath' ) {
                $hInfo->{$Name} = $Object->id == $System->id
                    ? $ENV{'EPAGES_STORES'}.'/'.$System->get('StoreName')
                    : $ENV{'EPAGES_STORES'}.'/'.$System->get('StoreName').'/'.$Object->filePath;
            }
            when( 'PrivateFiles' ) {
                my $pubPath = $Object->get('PrivatePath');
                my @Files;
                if( ExistsDirectory($pubPath) ) {
                    @Files = sort { lc($a->{'FileName'}) cmp lc($b->{'FileName'}) }
                        map { GetFileInfo($_) }
                        @{GetFileNames($pubPath)};
                }
                $hInfo->{$Name} = \@Files;
            }
            when( 'WebPath' ) {
                my $WebRoot = $System->get('WebRoot');
                $hInfo->{$Name} = $Object->id == $System->id
                    ? $WebRoot .'/'. $System->get('StoreName')
                    : $WebRoot .'/'. $System->get('StoreName') .'/'. $Object->filePath;
            }
            when( 'WebUrl' ) {
                my $hParams = {
                    'SSL'    => 0,
                    'Section' => 'Storefront'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when( 'WebUrlMobile' ) {
                my $hParams = {
                    'SSL'    => 0,
                    'Section' => 'Mobile'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when( 'WebUrlMobileSSL' ) {
                my $hParams = {
                    'SSL'    => 1,
                    'Section' => 'Mobile'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when( 'WebUrlSSL' ) {
                my $hParams = {
                    'SSL'    => 1,
                    'Section' => 'Storefront'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when( 'WebUrlAdmin' ) {
                my $hParams = {
                    'SSL'    => 0,
                    'Section' => 'Admin'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when( 'WebUrlAdminSSL' ) {
                my $hParams = {
                    'SSL'    => 1,
                    'Section' => 'Admin'
                };
                $hInfo->{$Name} = $Object->getWebUrl($hParams);
            }
            when ('WebUrlUnity') {
                $hInfo->{$Name} = _GetUnityURL($Object, $System, 'UnityUrlScheme', 0);
            }
            when ('WebUrlUnitySSL') {
                $hInfo->{$Name} = _GetUnityURL($Object, $System, 'UnityUrlScheme', 1);
            }
            when ('WebUrlUnityApi') {
                $hInfo->{$Name} = _GetUnityURL($Object, $System, 'UnityApiUrlScheme', 0);
            }
            when ('WebUrlUnityApiSSL') {
                $hInfo->{$Name} = _GetUnityURL($Object, $System, 'UnityApiUrlScheme', 1);
            }
        }
    }
    return $hInfo;
}

sub _GetUnityURL {
    my ($Object, $System, $Scheme, $IsSecureRequested) = @_;

    my $Site = $Object->getSite;
    my $UnityURL = $Site->get('DomainName');
    my $HasSSLCertificate = 0;
    # use own domain only if it is set, we have SSL and not for API Scheme
    my $UseOwnDomain = (defined $UnityURL and $UnityURL =~ m/\S/
        and $Site->get('HasSSLCertificate') and $Scheme !~ m/Api/) ? 1 : 0;

    if ($UseOwnDomain) {
        $HasSSLCertificate = 1;
    }
    else {
        my $hInfoSystem = $System->get(['UnityApiRoute', $Scheme, 'HasSSLCertificateUnity', 'HasSSLCertificateUnityAPI']);
        $UnityURL = $hInfoSystem->{$Scheme};
        # construct API URL
        if ($Scheme eq 'UnityApiUrlScheme') {
            # add API route unless it is already present
            $UnityURL = $UnityURL . $hInfoSystem->{'UnityApiRoute'} unless $UnityURL =~ m/$hInfoSystem->{'UnityApiRoute'}$/;
        }

        return unless $UnityURL;

        $HasSSLCertificate = $hInfoSystem->{'HasSSLCertificateUnity'}
            && ($Scheme ne 'UnityApiUrlScheme' || $hInfoSystem->{'HasSSLCertificateUnityAPI'});
    }

    my $Port;

    if ($HasSSLCertificate and $IsSecureRequested) {
        my $SystemUnitySSLWebServerPort = $System->get('UnitySSLWebServerPort');
        $Port = $SystemUnitySSLWebServerPort == PORT_HTTPS ? '' : $SystemUnitySSLWebServerPort;
    }
    else {
        my $SystemUnityWebServerPort = $System->get('UnityWebServerPort');
        $Port = $SystemUnityWebServerPort == PORT_HTTP ? '' : $SystemUnityWebServerPort;
    }

    if ($Port =~ m/\S/) {
        $UnityURL =~ s/(\/|$)/:$Port$1/;
    }

    my $hParams = {
      'UnityURL' => $UnityURL
    };
    # trigger hook OBJ_UnityURL to build specific unity URLs
    # for objects like products, categories, etc.
    TriggerHook('OBJ_UnityURL', { 'Object' => $Object, 'Params' => $hParams });
    $UnityURL = $hParams->{'UnityURL'};

    unless ($UseOwnDomain) {
        my $Alias = lc $Site->get('WebServerScriptNamePart');
        $Alias =~ s/_/-/g;

        # use query param routing always for API urls
        if ($Scheme !~ m/Api/ and $System->get('UnityUseSubdomainRouting')) {
            # subdomain routing
            # same domain scheme used in DE_EPAGES::Shop::API::Object::Shop->registerUnity
            $UnityURL = "$Alias.$UnityURL";
        }
        else {
            # query param routing
            $UnityURL = "$UnityURL?shop=$Alias";
        }
    }

    my $Protocol = ($HasSSLCertificate and $IsSecureRequested) ? 'https://' : 'http://';
    return $Protocol . $UnityURL;
}

# returns all parent objects from the site
sub _PathFromSite {
    my ($Object) = @_;

    my $SiteID = $Object->getSite()->id;
    my $ThisObject = $Object;
    my @Path;
    while ( defined $ThisObject && $SiteID != $ThisObject->id ) {
        Error('ObjectCycle', { 'ObjectID'=>$Object->id }) if scalar @Path > 100;
        unshift @Path, $ThisObject;
        $ThisObject = $ThisObject->parent;
    }
    return \@Path;
}

# returns a loop of all attributes of an object
sub _AttributeValues {
    my ($Object, $LanguageID) = @_;

    my $aAttributes = $Object->class->attributes;
    my @Loop;
    foreach my $Attribute ( sort { $a->alias cmp $b->alias } @$aAttributes ) {
        my $Value = $Attribute->get('IsArray') || $Attribute->get('Type') eq 'Hash'
            ? undef
            : $Object->get($Attribute->alias, $LanguageID);
        push @Loop, {
            Attribute => $Attribute,
            Object    => $Object,
            Value     => $Value,
        };
    }
    return \@Loop;
}

# returns a loop of all attributes of an object
sub _UserDefinedAttributeValues {
    my ($Object, $LanguageID) = @_;

    return [
        map { {
            Attribute => $_,
            Value     => $Object->get($_->alias, $LanguageID),
        } }
        @{ $Object->class->get('UserDefinedAttributes') }
    ]
}

# returns the attribute value of 'Name' or the Alias if no name exists.
sub _NameOrAlias {
    my ($Object, $LanguageID) = @_;

    if( $Object->class->existsAttribute('Name') ) {
        my $Name = $Object->get('Name', $LanguageID);
        return $Name if defined $Name && $Name ne '';
    }

    return $Object->alias;
}

# returns the object path string from the next Site object
sub _RelativePath {
    my ($Object) = @_;

    return '.' if $Object->instanceOf('Site');

    my $aParents = _PathFromSite($Object);
    return JoinObjectPath( [ map { $_->alias } @$aParents ] );
}

1;
