#========================================================================================
# package      DE_EPAGES::Order::API::OrderDocument
# state        public
# description  object interface for Order Documents.
#========================================================================================
package DE_EPAGES::Order::API::OrderDocument;
use base Exporter;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::File qw (
    CorrectPathSlashes
    ExistsFile
    WriteFile
);
use DE_EPAGES::Core::API::Log qw ( SetLogLevel );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Template qw ( ProcessFileTemplate );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Shop::API::Shop qw ( GetNextNo );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

our @EXPORT_OK = qw (
    CreateOrderDocument
    CreateShippingLabel
    CreateCustomerInformationPDF
    GetCustomerInformationPDFFileName
    GetCustomerInformationPDFLocation
    GetCustomerInformationPDFAttachment
    GetCustomerInformationPDFWebFileName
);

#========================================================================================
# function     CreateOrderDocument
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Document = CreateOrderDocument( $Order, $DocumentTypeName, $DocumentTypePlural);
# syntax       $Document = CreateOrderDocument( $Order, $DocumentTypeName, $DocumentTypePlural, $hOptions );
# example      my $Invoice = CreateOrderDocument($Order, 'Invoice', 'Invoices');
# example      my $Invoice = CreateOrderDocument(
#                   $Order,
#                   'Invoice,
#                   'Invoices',
#                   {
#                       'DeliveryDate' => $DeliveryDate,
#                       'IssueDate' => $IssueDate
#                   }
#               );
#----------------------------------------------------------------------------------------
# description  Creates a new packing slip object for the given Order object.
#----------------------------------------------------------------------------------------
# input        $Order | order object | object
# input        $DocumentTypeName | class package name of the document type, i.e.
#               'Invoice' | string
# input        $DocumentTypePlural | "plural" class package name of the document type,
#               i.e. 'Invoices' | string
# input        $hOptions | optional attributes of new object
#               <ul>
#                 <li>Alias        - object alias - string
#                 <li>DeliveryDate - datetime object for delivery date - object
#                 <li>IssueDate    - datetime object for issue date - object
#                 <li>Issued       - finalize the document (optional) - boolean
# return       $PackingSlip | packing slip object | object
#========================================================================================
sub CreateOrderDocument {
    my ($Order, $DocumentTypeName, $DocumentTypePlural, $hOptions) = @_;

    my $Shop = $Order->getSite();
    my $Document;
    Transaction( sub {
        # Create the alias within the transaction to guarantee sequential invoice numbers without gaps.
        # This may hurt performance for parallel oder document creation.
        my $hDocument = {
            'Alias'         => $hOptions->{'Alias'} // GetNextNo($Shop, $DocumentTypeName, $Order),
            'Order'         => $Order,
            'DeliveryDate'  => $hOptions->{'DeliveryDate'} // $Shop->currentDate(),
            'IssueDate'     => $hOptions->{'IssueDate'} // $Shop->currentDate(),
        };
        $hDocument->{'Issued'} = $hOptions->{'Issued'} if defined $hOptions->{'Issued'};

        $Document = $Order->folder($DocumentTypePlural)->insertChild($DocumentTypeName, $hDocument );

        my $Recursive = 1;
        my $CloneFiles = 0;
        # clone the LineItemContainer to the PackingSlip
        $Order->get('LineItemContainer')->clone({
            'Parent'    => $Document,
            'Alias'     => 'LineItemContainer',
            'Inherit'   => 1,
        }, $Recursive, $CloneFiles);
    });

    # Hook for changes after a order document was created. (JIRA: EPG-21122, TalismaID: 413-387)
    TriggerHook('OBJ_AfterCreateOrderDocument',{'Document' => $Document, 'DocumentTypeName' => $DocumentTypeName});

    return $Document;
}

#========================================================================================
# function     CreateShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label = CreateShippingLabel( $Order);
# example      my $Label = CreateShippingLabel($Order);
#----------------------------------------------------------------------------------------
# description  Creates a new shipping label object for the given Order object ( if the
#               order has assigned a ShippingFulfillmentMethod )
#----------------------------------------------------------------------------------------
# input        $Order | order object | object
# return       $ShippingLabel | shipping label object | object
#========================================================================================
sub CreateShippingLabel {
    my ($Order) = @_;

    my $ShippingLabel ;

    # First of all we check if the 'ShippingFulfillmentMethod' exists in the class Order, because this
    # attribute is specific for the ExternalShipping framework which could be not installed ( something
    # unlikey but not impossible ).
    # This, furthermore, avoids any dependency from Order to ExternalShipping.
    if ( $Order->class->existsAttribute('ShippingFulfillmentMethod') ) {

        # Then we check if the order has assigned any ShippingFulfillmentMethod

        my $FulfillmentMethod = $Order->get('ShippingFulfillmentMethod');

        if ( defined $FulfillmentMethod ) {

            TestObject(
                'FulfillmentMethod',
                $FulfillmentMethod,
                'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod'
            );

            Transaction( sub {

                #At this point, is safe to create the ShippingLabel ...
                $ShippingLabel = $FulfillmentMethod->newShippingLabel($Order);

                # Hook for changes after a order document was created. (JIRA: EPG-21122, TalismaID: 413-387)
                TriggerHook(
                    'OBJ_AfterCreateOrderDocument',
                    {
                        'Document' => $ShippingLabel,
                        'DocumentTypeName' => 'ShippingLabel'
                    }
                );
            }) ;
        }
    }

    return $ShippingLabel;
}

#========================================================================================
# function     CreateCustomerInformationPDF
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateCustomerInformationPDF($CustomerInformation, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Creates a new customer information PDF in the given language.
#----------------------------------------------------------------------------------------
# input        $CustomerInformation | CustomerInformation content object | object
# input        $LanguageID | language | integer
# return       $Success | true if a PDFs was written | boolean
#========================================================================================
sub CreateCustomerInformationPDF {
    my ($CustomerInformation, $LanguageID) = @_;

    my $Shop = $CustomerInformation->getSite;

    my $CustomerInformationContent = $CustomerInformation->get('Text', $LanguageID) // $CustomerInformation->get('Description', $LanguageID);

    if ($CustomerInformationContent and $CustomerInformationContent =~ /\S/) {
        my $LanguageCode = GetCodeByLanguageID($LanguageID);

        my $hVars = {
            'Encoding'    => 'utf-8',
            'Language'    => $LanguageCode,
            'Content'     => $CustomerInformationContent
        };

        my $FileName = $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Order/Templates/CustomerInformationPDF.html';
        my $HTMLContent;

        SetLogLevel('WARN', sub {
            $HTMLContent = ProcessFileTemplate($FileName, $LanguageCode, $hVars);
        },'DE_EPAGES::TLE::API::Template');

        my $PDFFileName = GetCustomerInformationPDFFileName($Shop, $LanguageID, $LanguageCode);

        eval {
            my $PDFContent = DE_EPAGES::EPagesJ::API::PDF::PdfClient->create($Shop)->transform($HTMLContent);
            WriteFile($PDFFileName, \$PDFContent , { 'binary' => 1, 'syswrite' => 1, 'create_directory' => 1 });
        };

        return ExistsError() ? 0 : 1;
    }

    return 0;
}

#========================================================================================
# function     GetCustomerInformationPDFAttachment
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCustomerInformationPDFAttachment($Shop, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the PDF full filename to use as mail attachment.
#               Creates a new customer information PDF if it wasn't created yet.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LanguageID | language | integer
# return       $hAttachment | hash with the PDF filename | ref.hash
#========================================================================================
sub GetCustomerInformationPDFAttachment {
    my ($Shop, $LanguageID) = @_;

    my $Categories = $Shop->child('Categories');
    my $CustomerInformation = $Categories->existsChild('CustomerInformation') ? $Categories->child('CustomerInformation') : undef;

    if ($CustomerInformation) {
        my $PDFFileName = GetCustomerInformationPDFFileName($Shop, $LanguageID);
        my $Success = 1;

        unless (ExistsFile($PDFFileName)) {
            $Success = CreateCustomerInformationPDF($CustomerInformation, $LanguageID);
        }

        return $Success ? { 'filename' => $PDFFileName } : undef;
    }

    return;
}

#========================================================================================
# function     GetCustomerInformationPDFFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCustomerInformationPDFFileName($Shop, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the PDF full filename.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LanguageID | language | integer
# return       $FileName | full PDF filename | string
#========================================================================================
sub GetCustomerInformationPDFFileName {
    my ($Shop, $LanguageID) = @_;

    my $FilePath = GetCustomerInformationPDFLocation($Shop).'/'._CustomerInformationPDFRelativeFileName($LanguageID);
    return CorrectPathSlashes($FilePath),
}

#========================================================================================
# function     GetCustomerInformationPDFLocation
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCustomerInformationPDFLocation($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the folder which contains the generated PDFs. Each resides in a
#               language folder, which is added during creation.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $Location | folder for the generated PDFs | string
#========================================================================================
sub GetCustomerInformationPDFLocation {
    my ($Shop) = @_;

    my $Directory = $Shop->get('PublicPath').'/'._CustomerInformationPDFRelativeDir();
    return CorrectPathSlashes($Directory);
}

#========================================================================================
# function     GetCustomerInformationPDFWebFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCustomerInformationPDFWebFileName($Shop, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the PDF web filename (including shop URL).
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LanguageID | order language | integer
# return       $FileName | full PDF filename | string
#========================================================================================
sub GetCustomerInformationPDFWebFileName {
    my ($Shop, $LanguageID) = @_;

    return $Shop->get('WebPath')
        .'/'._CustomerInformationPDFRelativeDir()
        .'/'._CustomerInformationPDFRelativeFileName($LanguageID);
}

#========================================================================================
# function     _CustomerInformationPDFRelativeFileName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $FilePath = _CustomerInformationPDFRelativeFileName($Shop, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the file name of "Rights of withdrawl.pdf"
#               relative to <function GetCustomerInformationPDFLocation>.
#----------------------------------------------------------------------------------------
# input        $LanguageID | order language | integer
# return       $FilePath | relative file path | string
#========================================================================================
sub _CustomerInformationPDFRelativeFileName {
    my ($LanguageID) = @_;

    my $LanguageCode = GetCodeByLanguageID($LanguageID);
    my $BaseName = Unicode2Filename(Translate('CustomerInformation', $LanguageID, 'DE_EPAGES::Dictionary'));
    return "$LanguageCode/$BaseName.pdf";
}

#========================================================================================
# function     _CustomerInformationPDFRelativeDir
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RelativeDir = _CustomerInformationPDFRelativeDir();
#----------------------------------------------------------------------------------------
# description  Returns the directory containing the file "Rights of withdrawl.pdf"
#               relative to the public directory of the shop.
#----------------------------------------------------------------------------------------
# return       $RelativeDir | relative file path | string
#========================================================================================
sub _CustomerInformationPDFRelativeDir {

    return 'PDF/CustomerInformation';
}

1;
