#========================================================================================
# package      DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# description  Controller for the PaymentMethod PayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus;
use base DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::PayPalPlus::API::Logger;
use DE_EPAGES::PayPalPlus::API::ContextLogger;
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service;
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::PayPalPlus::API::Webservice::OnBoarding::Service;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);

use strict;

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction to display Merchant onboarding information and URL
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $MethodName    = ( caller(0) )[3];
    my $Context       = $self->_createLogContext();
    my $PaymentMethod = $Servlet->object();

    my $System = $self->_LoadRootObject();
    my $HasSSLCertificate = $System->get('HasSSLCertificate');
    if(not $HasSSLCertificate) {
        $Servlet->vars( 'NoValidSSLCertificateError', 1 );
    }

    $Context->logContext($PaymentMethod, $MethodName, sub {
        $self->_fetchOnBoardingUrlOnServlet($Servlet);

        $self->_gatherMerchantStatusFromService($Servlet);

        $self->_gatherExperienceProfileIdFromDatabaseOrService($Servlet) if ($self->_isMerchantConnected($Servlet));
    });

    return;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to save user inputs into the database
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $MethodName = ( caller(0) )[3];
    my $Context = $self->_createLogContext();
    my $PaymentMethod = $Servlet->object();
    $Context->logContext($PaymentMethod, $MethodName, sub {
        $self->Save($Servlet);
    });

    return;
}

#========================================================================================
# function     DisablePayPalPlusConnectivity
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisablePayPalPlusConnectivity( $Servlet );
#----------------------------------------------------------------------------------------
# description  It disables connection to PayPalPlus if some conditions are met.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub DisablePayPalPlusConnectivity {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();

    my $IsDefaultPayment = $PaymentMethod->get('IsDefault');
    if ($IsDefaultPayment) {
        $Servlet->vars('IsDefaultPaymentMethodAndCanNotBeDisconnected', 1);
        return;
    }

    $PaymentMethod->set({
            'ExperienceProfileId' => '',
            'IsActivated' => 0,
            'HasMerchantStatusConflicts' => 1,
            'MerchantClientId' => '',
            'MerchantId' => '',
            'OnboardingTransactionId' => '',
    });
    return;
}

#----------------------------------------------------------------------------------------
# private
#----------------------------------------------------------------------------------------
sub _getReturnToPartnerUrl {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethodPayPalPlus = $Servlet->object();
    my $hOptions = { 'Type' => 'admin', 'UseSSL' => 1, 'AddToken' => 1 };
    my $hParams = { 'ViewAction' => 'MBO-ViewSettings' };

    my $URL = $self->_BuildShopUrl( $PaymentMethodPayPalPlus, $hParams, $hOptions, $Servlet );

    return $URL;
}

#----------------------------------------------------------------------------------------
sub _gatherExperienceProfileIdFromDatabaseOrService {
    my $self          = shift;
    my ($Servlet)     = @_;
    my $PaymentMethod = $Servlet->object();

    #Stop doing webservice call if the experienceProfile already exists
    return if isValid($PaymentMethod->get('ExperienceProfileId'));

    my $ExperienceProfileIdService = $self->_createExperienceProfileIdService($PaymentMethod);
    $Servlet->vars( 'ExperienceProfileIdError', 0 );

    my $Logger = $self->_createLogger();
    if ( $ExperienceProfileIdService->doRequest() ) {
        if ( not $ExperienceProfileIdService->persistResponse() ) {
            $Servlet->vars( 'ExperienceProfileIdError', 1 );
            $Logger->logOnboarding({ 'Status' => 'Unexpected error when persisting profile service response into database' });
        }
    }
    else {
        $Servlet->vars( 'ExperienceProfileIdError', 1 );
        $Logger->logOnboarding({
            'Status' => 'Unexpected web service error when getting experience profile information: '
                    . $ExperienceProfileIdService->getErrorCode() . ' / ' . $ExperienceProfileIdService->getErrorMessage()
        });
    }
}

#----------------------------------------------------------------------------------------
sub _createExperienceProfileIdService {
    my $self = shift;
    my ($PaymentMethod) = @_;
    return DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service->new($PaymentMethod);
}

#----------------------------------------------------------------------------------------
sub _BuildShopUrl {
    my $self = shift;
    my ( $Object, $hParams, $hOptions, $Servlet ) = @_;
    return BuildShopUrl( $Object, $hParams, $hOptions, $Servlet );
}

#----------------------------------------------------------------------------------------
sub _createLogContext {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::ContextLogger->new();
}
#----------------------------------------------------------------------------------------
sub _createLogger{
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}
#----------------------------------------------------------------------------------------
sub _fetchOnBoardingUrlOnServlet {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $ReturnURL = $self->_getReturnToPartnerUrl($Servlet);
    my $Service = $self->_createOnBoardingService();
    eval {
        $Service->doRequest($PaymentMethod, $ReturnURL);
    };

    if (ExistsError()) {
        my $Error = GetError();
        my $Logger = $self->_createLogger();
        $Logger->logOnboarding({
            'Status' => 'The Boarding Url could not be created.',
            'ErrorMessage' => $Error->message(),
        });
        $Servlet->vars( 'OnboardingError', 1 );
        return;
    }
    $Servlet->vars( 'OnboardingURL', $Service->getOnboardingURL() );
    return;
}

#----------------------------------------------------------------------------------------
sub _gatherMerchantStatusFromService {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $GetMerchantStatusService = $self->_createGetMerchantStatusService();
    my $WasSucessfulRequest = $GetMerchantStatusService->doRequest($PaymentMethod);
    my $Logger = $self->_createLogger();
    
    if ($WasSucessfulRequest) {
        
        $Servlet->vars( 'HasCollectedMerchantStatus', 1 );
        if ($GetMerchantStatusService->hasMerchantStatusConflicts()) {
            my $aMerchantStatusConflicts = $GetMerchantStatusService->getMerchantStatusConflicts();
            $Servlet->vars( 'HasMerchantStatusConflicts', 1 );
            $Servlet->vars( 'GetMerchantStatusConflicts', $aMerchantStatusConflicts );
        } else {
            $Servlet->vars( 'HasMerchantStatusConflicts', 0 );
        }
        if ($GetMerchantStatusService->persistResponse($PaymentMethod)) {
            $Servlet->vars( 'GetMerchantStatusPrimaryEmail', $GetMerchantStatusService->getPrimaryEmail() );

            $Logger->logOnboarding({ 'Status' => 'Add MerchantId & MerchantClientId to PaymentMethod (Path): ' . $PaymentMethod->get('Path') });
        } else {
            $Servlet->vars( 'GetMerchantStatusError', 1 );
            $Logger->logOnboarding({ 'Status' => 'Unexpected error when persisting merchant status response into database' });
        }
        
        $Servlet->vars( 'IsActivePayPal', $GetMerchantStatusService->getIsActivePayPal() );
        $Servlet->vars( 'IsActiveDirectDebit', $GetMerchantStatusService->getIsActiveDirectDebit() );
        $Servlet->vars( 'IsActiveCreditCard', $GetMerchantStatusService->getIsActiveCreditCard() );
        $Servlet->vars( 'IsActiveInvoice', $GetMerchantStatusService->getIsActiveInvoice() );

    } else {
        $Servlet->vars( 'HasCollectedMerchantStatus', 0 );
        my $StatusServiceErrorMessage = $GetMerchantStatusService->getErrorMessage();
        if ($StatusServiceErrorMessage =~ /404/) { # "Not Found" : merchant may have not been connected yet
            $Logger->logOnboarding({ 'Status' => 'Merchant not connected yet (payment path): ' . $PaymentMethod->get('Path') });
        } else {
            $Servlet->vars( 'GetMerchantStatusError', 1 );
            $Logger->logOnboarding({
                'Status' => 'Unexpected web service error when getting merchant status information: '
                        . $GetMerchantStatusService->getErrorCode() . ' / ' . $GetMerchantStatusService->getErrorMessage()
            });
        }
    }

    return $WasSucessfulRequest;
}

#----------------------------------------------------------------------------------------
sub _isMerchantConnected {
    my $self = shift;
    my ($Servlet) = @_;

    return $Servlet->params('HasCollectedMerchantStatus');
}

#----------------------------------------------------------------------------------------
sub _createOnBoardingService {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::OnBoarding::Service->new();
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService {
    my $self = shift;

    my $GetMerchantStatusService = DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service->new();

    return $GetMerchantStatusService;
}

#========================================================================================
# Wrappers
#========================================================================================
sub _LoadRootObject {
    my $self = shift;
    return LoadRootObject();
}

1;
