#========================================================================================
# package      DE_EPAGES::Permission::API::Object::User
# state        public
# base         DE_EPAGES::Permission::API::Object::Trustee
# description  object interface for User.
#========================================================================================
package DE_EPAGES::Permission::API::Object::User;
use base DE_EPAGES::Permission::API::Object::Trustee;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Table::Locale qw (GetInfoLocale);
use DE_EPAGES::Permission::API::Table::Permission qw ( InsertPermission );
use DE_EPAGES::Permission::API::Permission qw ( RoleByName );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateShortGUID isValid);

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $User->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'User' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $User->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $User = DE_EPAGES::Permission::API::Object::User->insert($hInfo);
# example      my $Class = LoadClassByAlias('User');
#               my $User = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertUser' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $User | new object | object
# hook         OBJ_InsertUser | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>UserID        - user id              - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    $hInfo->{'Alias'} = $class->generateUserAlias() unless isValid($hInfo->{'Alias'});

    my $self = $class->SUPER::insert($hInfo);

    InsertPermission({'ObjectID'=>$self->id, 'TrusteeID'=>$self->id, 'MetaActionID'=>RoleByName($self->class,'User')->id, 'Allow'=>1 });
    TriggerHook('OBJ_InsertUser', {'Object' => $self, 'UserID' => $self->id });
    delete $hInfo->{$_} foreach qw ( LocaleID CurrencyID Password IsActivated DeleteConfirmation IsLoginCookieAllowed IsHtmlEMailAllowed ReminderQuestion ChallengePhrase EMail CountLoginFailure LastLoginFailure Name IsEmailVerified ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $User->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteUser' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteUser | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>UserID        - user id              - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteUser', {'Object' => $self, 'UserID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $User->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateUser' and 'OBJ_AfterUpdateUser'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateUser | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>UserID        - user id              - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateUser | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>UserID        - user id              - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateUser', {'Object'=>$self, 'Values'=> $hValues, 'UserID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateUser',  {'Object'=>$self, 'Values'=> $hValues, 'UserID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Permission::API::Object::User->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    my $Site = $Parent->getSite();

    my $hValues = $Site->get([ qw (LocaleID CurrencyID)]);

    if (defined $hValues->{'LocaleID'}) {
        my $Locale = GetInfoLocale($hValues->{'LocaleID'});
        $hValues->{'CurrencyID'} = $Locale->{'CurrencyID'} unless defined $hValues->{'CurrencyID'};
    }
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        %$hValues,
        'IsActivated' => 1,
        'DeleteConfirmation' => 1,
        'IsLoginCookieAllowed' => 0,
        'IsHtmlEMailAllowed' => 1,
        'CountLoginFailure' => 0,
        'IsEmailVerified' => 1,
    };
}

#========================================================================================
# function     generateUserAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $userAlias = $self->generateUserAlias();
#----------------------------------------------------------------------------------------
# description  Generates a user alias.
#----------------------------------------------------------------------------------------
# return       $userAlias | generated user alias | string
#========================================================================================
sub generateUserAlias {
    my $self = shift;

    return GenerateShortGUID();
}

1;
