#========================================================================================
# package      DE_EPAGES::Permission::DAL::Permission
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>attributepermission
#                   <li>permission
#               </ul>
#========================================================================================
package DE_EPAGES::Permission::DAL::Permission;
use base qw (Exporter);

use strict;
our @EXPORT = qw (
    DAL_InsertAttributePermission
    DAL_UpdateAttributePermission
    DAL_DeleteAttributePermission
    DAL_GetAllPKeyAttributePermissions
    DAL_GetAllInfoAttributePermissions
    DAL_GetInfoAttributePermission
    DAL_GetPKeyAttributePermissionsByTrusteeID
    DAL_GetInfoAttributePermissionsByTrusteeID
    DAL_GetPKeyAttributePermissionsByAttributeID
    DAL_GetInfoAttributePermissionsByAttributeID
    DAL_GetInfoAttributePermissionsByClassID

    DAL_InsertPermission
    DAL_UpdatePermission
    DAL_DeletePermission
    DAL_GetAllPKeyPermissions
    DAL_GetAllInfoPermissions
    DAL_GetInfoPermission
    DAL_GetPKeyPermissionsByObjectID
    DAL_GetInfoPermissionsByObjectID
    DAL_GetPKeyPermissionsByTrusteeID
    DAL_GetInfoPermissionsByTrusteeID
    DAL_GetPKeyPermissionsByMetaActionID
    DAL_GetInfoPermissionsByMetaActionID
    DAL_GetUsersWithEmailInAlias
);

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::DataCache::API::Cache qw ( GetCache UpdateCache GetCacheSize );

my $DAL_PACKAGE = 'DE_EPAGES::Database::API::DAL';
my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Permission::DAL::'.$dbi->driverName.'::Permission';
    UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

sub _DoFillInfoCache ($$) {
    my ($CacheName, $CountRows) = @_;
    my $dbi = GetCurrentDBHandle();
    my $CacheSize = GetCacheSize($dbi->cacheKey, $CacheName);
    return $CountRows < $CacheSize ? 1 : 0;
}

#========================================================================================
# function     DAL_InsertAttributePermission
#----------------------------------------------------------------------------------------
# syntax       DAL_InsertAttributePermission($hParameter);
#----------------------------------------------------------------------------------------
# description  insert entry in table attributepermission
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                  <li>TrusteeID     - trustee id           (part of primary key)- int
#                  <li>AttributeID   - attribute id         (part of primary key)- int
#                  <li>Permission    - readable=2, writable=4 - tinyint
#               </ul> | reference to hash
#========================================================================================
sub DAL_InsertAttributePermission ($) {
    my ($hParameter) = @_;
    _CallDriver('InsertAttributePermission', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'AllAttributePermission');
        UpdateCache($dbi->cacheKey,'InfoAttributePermission', join('<||>', @$hParameter{qw (TrusteeID AttributeID)}),$hParameter );
        UpdateCache($dbi->cacheKey,'AttributePermissionByTrusteeID', $hParameter->{'TrusteeID'});
        UpdateCache($dbi->cacheKey,'AttributePermissionByAttributeID', $hParameter->{'AttributeID'});
    }
    return;
}

#========================================================================================
# function     DAL_UpdateAttributePermission
#----------------------------------------------------------------------------------------
# syntax       DAL_UpdateAttributePermission($TrusteeID, $AttributeID, $hParameter);
#----------------------------------------------------------------------------------------
# description  updates an entry of table attributepermission
#----------------------------------------------------------------------------------------
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to hash
#========================================================================================
sub DAL_UpdateAttributePermission ($$$) {
    my ($TrusteeID, $AttributeID, $hParameter) = @_;
    my $hInfo = DAL_GetInfoAttributePermission($TrusteeID, $AttributeID);
    my $hChangedValues = {};
    my $dbi = GetCurrentDBHandle();
    foreach my $ChangedKey (qw ( Permission )) {
        next unless exists $hParameter->{$ChangedKey};
        next if not defined $hInfo->{$ChangedKey} and not defined $hParameter->{$ChangedKey};
        if ( defined $hParameter->{$ChangedKey}
             and not defined $hInfo->{$ChangedKey}
             or
             not defined $hParameter->{$ChangedKey}
             and defined $hInfo->{$ChangedKey}
        ) {
            $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey});
            next;
        }
        next if $hInfo->{$ChangedKey} eq $hParameter->{$ChangedKey};
        $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey})
    }
    return 0 unless scalar keys %$hChangedValues;

    _CallDriver('UpdateAttributePermission', $TrusteeID, $AttributeID, $hChangedValues);
    if($DO_CACHE) {
        # Copy old values and updates ones to a new hash which is used to update the cache.
        my $hMergedCacheInfo = { %$hInfo, %$hChangedValues };
        UpdateCache($dbi->cacheKey,'InfoAttributePermission', join('<||>',$TrusteeID, $AttributeID), $hMergedCacheInfo);
        if (exists $hChangedValues->{'AttributeID'}) {
            UpdateCache($dbi->cacheKey,'AttributePermissionByAttributeID', $hInfo->{'AttributeID'});
            UpdateCache($dbi->cacheKey,'AttributePermissionByAttributeID', $hChangedValues->{'AttributeID'});
        }
    }
    return 1;
}

#========================================================================================
# function     DAL_DeleteAttributePermission
#----------------------------------------------------------------------------------------
# syntax       DAL_DeleteAttributePermission($TrusteeID, $AttributeID);
#----------------------------------------------------------------------------------------
# description  removes an entry of table attributepermission
#----------------------------------------------------------------------------------------
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
#========================================================================================
sub DAL_DeleteAttributePermission ($$) {
    my ($TrusteeID, $AttributeID) = @_;
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my $hInfo = undef;
    if ($DO_CACHE) {
        my $InfoCache = GetCache($cacheKey,'InfoAttributePermission');
        $hInfo = $InfoCache->{ join('<||>',$TrusteeID, $AttributeID)};
    }
    _CallDriver('DeleteAttributePermission', $TrusteeID, $AttributeID);
    if ($DO_CACHE) {
        UpdateCache($cacheKey,'InfoAttributePermission',  join('<||>',$TrusteeID, $AttributeID));
        UpdateCache($cacheKey,'AllAttributePermission');
        UpdateCache($cacheKey,'AttributePermissionByTrusteeID');
        if (defined $hInfo) {
            UpdateCache($cacheKey,'AttributePermissionByAttributeID', $hInfo->{'AttributeID'});
        } else {
            UpdateCache($cacheKey,'AttributePermissionByAttributeID');
        }
    }
    return;
}

#========================================================================================
# function     DAL_GetAllPKeyAttributePermissions
#----------------------------------------------------------------------------------------
# syntax       $rahAttributePermissionIDs = DAL_GetAllPKeyAttributePermissions();
#----------------------------------------------------------------------------------------
# description  returns all primary keys of table attributepermission
#----------------------------------------------------------------------------------------
# return       $rahAttributePermissionIDs | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllPKeyAttributePermissions () {
    return _CallDriver('GetAllPKeyAttributePermissions') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllAttributePermission');
    $CacheRef->{'1'} = _CallDriver('GetAllPKeyAttributePermissions') unless exists $CacheRef->{'1'};
    return [ @{$CacheRef->{'1'}} ];
}

#========================================================================================
# function     DAL_GetAllInfoAttributePermissions
#----------------------------------------------------------------------------------------
# syntax       $rahAttributePermissionIDs = DAL_GetAllInfoAttributePermissions();
#----------------------------------------------------------------------------------------
# description  returns all rows of table attributepermission
#----------------------------------------------------------------------------------------
# return       $rahAttributePermissionIDs | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllInfoAttributePermissions ();
sub DAL_GetAllInfoAttributePermissions () {
    return _CallDriver('GetAllInfoAttributePermissions') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllAttributePermission');
    if (not exists $CacheRef->{'1'}) {
        my $rResultSet = _CallDriver('GetAllInfoAttributePermissions');
        # make copy to primary key cache
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (TrusteeID AttributeID )} = @$hRow{qw (TrusteeID AttributeID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{'1'} = \@PrimaryKeySet;
        # make copy to info cache
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoAttributePermission');
        if (_DoFillInfoCache('InfoAttributePermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join('<||>', @$Row{qw (TrusteeID AttributeID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoAttributePermission', @{$CacheRef->{'1'}})) {
        my @ResultSet;
        foreach my $PrimaryKey (@{$CacheRef->{'1'}}) {
            my $Info = DAL_GetInfoAttributePermission($PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'AttributeID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{'1'};
    return DAL_GetAllInfoAttributePermissions();
}

#========================================================================================
# function     DAL_GetInfoAttributePermission
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoAttributePermission($TrusteeID, $AttributeID);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>| reference to hash
#========================================================================================
sub DAL_GetInfoAttributePermission ($$) {
    my ($TrusteeID, $AttributeID) = @_;
    return _CallDriver('GetInfoAttributePermission', $TrusteeID, $AttributeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef    = GetCache($dbi->cacheKey,'InfoAttributePermission');
    my $CacheString =  join('<||>',$TrusteeID, $AttributeID);
    $CacheRef->{$CacheString} = _CallDriver('GetInfoAttributePermission', $TrusteeID, $AttributeID) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_GetPKeyAttributePermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyAttributePermissionsByTrusteeID($TrusteeID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table attributepermission by primary key
#----------------------------------------------------------------------------------------
# input        $TrusteeID | trustee id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyAttributePermissionsByTrusteeID ($) {
    my ($TrusteeID) = @_;
    return _CallDriver('GetPKeyAttributePermissionsByTrusteeID', $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AttributePermissionByTrusteeID');
    my $CacheString = $TrusteeID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyAttributePermissionsByTrusteeID', $TrusteeID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoAttributePermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoAttributePermissionsByTrusteeID($TrusteeID);
#----------------------------------------------------------------------------------------
# description  get full row of table attributepermission by primary key
#----------------------------------------------------------------------------------------
# input        $TrusteeID | trustee id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoAttributePermissionsByTrusteeID ($);
sub DAL_GetInfoAttributePermissionsByTrusteeID ($) {
    my ($TrusteeID) = @_;
    return _CallDriver('GetInfoAttributePermissionsByTrusteeID', $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AttributePermissionByTrusteeID');
    my $CacheString = $TrusteeID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoAttributePermissionsByTrusteeID', $TrusteeID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (TrusteeID AttributeID )} = @$hRow{qw (TrusteeID AttributeID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoAttributePermission');
        if (_DoFillInfoCache('InfoAttributePermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (TrusteeID AttributeID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoAttributePermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoAttributePermission($PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'AttributeID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoAttributePermissionsByTrusteeID($TrusteeID);
}

#========================================================================================
# function     DAL_GetPKeyAttributePermissionsByAttributeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyAttributePermissionsByAttributeID($AttributeID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table attributepermission by index i_attributepermission_attributeid
#----------------------------------------------------------------------------------------
# input        $AttributeID | attribute id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyAttributePermissionsByAttributeID ($) {
    my ($AttributeID) = @_;
    return _CallDriver('GetPKeyAttributePermissionsByAttributeID', $AttributeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AttributePermissionByAttributeID');
    my $CacheString = $AttributeID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyAttributePermissionsByAttributeID', $AttributeID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoAttributePermissionsByAttributeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoAttributePermissionsByAttributeID($AttributeID);
#----------------------------------------------------------------------------------------
# description  get full row of table attributepermission by index i_attributepermission_attributeid
#----------------------------------------------------------------------------------------
# input        $AttributeID | attribute id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoAttributePermissionsByAttributeID ($);
sub DAL_GetInfoAttributePermissionsByAttributeID ($) {
    my ($AttributeID) = @_;
    return _CallDriver('GetInfoAttributePermissionsByAttributeID', $AttributeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AttributePermissionByAttributeID');
    my $CacheString = $AttributeID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoAttributePermissionsByAttributeID', $AttributeID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (TrusteeID AttributeID )} = @$hRow{qw (TrusteeID AttributeID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoAttributePermission');
        if (_DoFillInfoCache('InfoAttributePermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (TrusteeID AttributeID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoAttributePermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoAttributePermission($PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'AttributeID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoAttributePermissionsByAttributeID($AttributeID);
}

#========================================================================================
# function     DAL_GetInfoAttributePermissionsByClassID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoAttributePermissionsByClassID($ClassID, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  get {attributeid, permission} rows of table attributepermission by 
#               the given class id and trustee id. A join with table 'attribute' is
#               needed in order to establish a connection between an attribute id and
#               its class id.
#----------------------------------------------------------------------------------------
# input        $ClassID | class id | int
# input        $TrusteeID | trustee id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoAttributePermissionsByClassID ($$);
sub DAL_GetInfoAttributePermissionsByClassID ($$) {
    my ($ClassID, $TrusteeID) = @_;
    return _CallDriver('GetInfoAttributePermissionsByClassID', $ClassID, $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AttributePermissionByClassID');
    my $CacheString = join('<||>', $ClassID, $TrusteeID);
    my $CacheContent = $CacheRef->{$CacheString};

    if ((not defined $CacheContent) || (scalar @{ $CacheContent} == 0) ) {
        my $rResultSet = _CallDriver('GetInfoAttributePermissionsByClassID', $ClassID, $TrusteeID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (TrusteeID AttributeID )} = @$hRow{qw (TrusteeID AttributeID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoAttributePermission');
        if (_DoFillInfoCache('InfoAttributePermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (TrusteeID AttributeID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoAttributePermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoAttributePermission($PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'AttributeID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoAttributePermissionsByClassID($ClassID, $TrusteeID);
}

#========================================================================================
# function     DAL_InsertPermission
#----------------------------------------------------------------------------------------
# syntax       DAL_InsertPermission($hParameter);
#----------------------------------------------------------------------------------------
# description  insert entry in table permission
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ObjectID      - permission object id (part of primary key)- int
#                  <li>TrusteeID     - trustee id           (part of primary key)- int
#                  <li>MetaActionID  - action id            (part of primary key)- int
#                  <li>Allow         - allow=1, deny=2      - tinyint
#               </ul> | reference to hash
#========================================================================================
sub DAL_InsertPermission ($) {
    my ($hParameter) = @_;
    _CallDriver('InsertPermission', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'AllPermission');
        UpdateCache($dbi->cacheKey,'InfoPermission', join('<||>', @$hParameter{qw (ObjectID TrusteeID MetaActionID)}),$hParameter );
        UpdateCache($dbi->cacheKey,'PermissionByObjectID');
        UpdateCache($dbi->cacheKey,'PermissionByTrusteeID', $hParameter->{'TrusteeID'});
        UpdateCache($dbi->cacheKey,'PermissionByMetaActionID', $hParameter->{'MetaActionID'});
    }
    return;
}

#========================================================================================
# function     DAL_UpdatePermission
#----------------------------------------------------------------------------------------
# syntax       DAL_UpdatePermission($ObjectID, $TrusteeID, $MetaActionID, $hParameter);
#----------------------------------------------------------------------------------------
# description  updates an entry of table permission
#----------------------------------------------------------------------------------------
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to hash
#========================================================================================
sub DAL_UpdatePermission ($$$$) {
    my ($ObjectID, $TrusteeID, $MetaActionID, $hParameter) = @_;
    my $hInfo = DAL_GetInfoPermission($ObjectID, $TrusteeID, $MetaActionID);
    my $hChangedValues = {};
    my $dbi = GetCurrentDBHandle();
    foreach my $ChangedKey (qw ( Allow )) {
        next unless exists $hParameter->{$ChangedKey};
        next if not defined $hInfo->{$ChangedKey} and not defined $hParameter->{$ChangedKey};
        if ( defined $hParameter->{$ChangedKey}
             and not defined $hInfo->{$ChangedKey}
             or
             not defined $hParameter->{$ChangedKey}
             and defined $hInfo->{$ChangedKey}
        ) {
            $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey});
            next;
        }
        next if $hInfo->{$ChangedKey} eq $hParameter->{$ChangedKey};
        $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey})
    }
    return 0 unless scalar keys %$hChangedValues;

    _CallDriver('UpdatePermission', $ObjectID, $TrusteeID, $MetaActionID, $hChangedValues);
    if($DO_CACHE) {
        # Copy old values and updates ones to a new hash which is used to update the cache.
        my $hMergedCacheInfo = { %$hInfo, %$hChangedValues };
        UpdateCache($dbi->cacheKey,'InfoPermission', join('<||>',$ObjectID, $TrusteeID, $MetaActionID), $hMergedCacheInfo);
        if (exists $hChangedValues->{'TrusteeID'}) {
            UpdateCache($dbi->cacheKey,'PermissionByTrusteeID', $hInfo->{'TrusteeID'});
            UpdateCache($dbi->cacheKey,'PermissionByTrusteeID', $hChangedValues->{'TrusteeID'});
        }
        if (exists $hChangedValues->{'MetaActionID'}) {
            UpdateCache($dbi->cacheKey,'PermissionByMetaActionID', $hInfo->{'MetaActionID'});
            UpdateCache($dbi->cacheKey,'PermissionByMetaActionID', $hChangedValues->{'MetaActionID'});
        }
    }
    return 1;
}

#========================================================================================
# function     DAL_DeletePermission
#----------------------------------------------------------------------------------------
# syntax       DAL_DeletePermission($ObjectID, $TrusteeID, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  removes an entry of table permission
#----------------------------------------------------------------------------------------
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
#========================================================================================
sub DAL_DeletePermission ($$$) {
    my ($ObjectID, $TrusteeID, $MetaActionID) = @_;
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my $hInfo = $DO_CACHE ? DAL_GetInfoPermission($ObjectID, $TrusteeID, $MetaActionID) : undef;
    _CallDriver('DeletePermission', $ObjectID, $TrusteeID, $MetaActionID);
    if (defined $hInfo) {
        my $CacheName1 = $hInfo->{'ObjectID'}. '<||>';
        my $CacheName2 = join('<||>', @$hInfo{qw (ObjectID TrusteeID)});
        my $CacheName3 = join('<||>', @$hInfo{qw (ObjectID TrusteeID MetaActionID)});
        UpdateCache($cacheKey,'AllPermission');
        UpdateCache($cacheKey,'InfoPermission', $CacheName3);
        UpdateCache($cacheKey,'PermissionByObjectID', $CacheName1);
        UpdateCache($cacheKey,'PermissionByObjectID', $CacheName2);
        UpdateCache($cacheKey,'PermissionByTrusteeID', $hInfo->{'TrusteeID'});
        UpdateCache($cacheKey,'PermissionByMetaActionID', $hInfo->{'MetaActionID'});
    }
    return;
}

#========================================================================================
# function     DAL_GetAllPKeyPermissions
#----------------------------------------------------------------------------------------
# syntax       $rahPermissionIDs = DAL_GetAllPKeyPermissions();
#----------------------------------------------------------------------------------------
# description  returns all primary keys of table permission
#----------------------------------------------------------------------------------------
# return       $rahPermissionIDs | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllPKeyPermissions () {
    return _CallDriver('GetAllPKeyPermissions') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllPermission');
    $CacheRef->{'1'} = _CallDriver('GetAllPKeyPermissions') unless exists $CacheRef->{'1'};
    return [ @{$CacheRef->{'1'}} ];
}

#========================================================================================
# function     DAL_GetAllInfoPermissions
#----------------------------------------------------------------------------------------
# syntax       $rahPermissionIDs = DAL_GetAllInfoPermissions();
#----------------------------------------------------------------------------------------
# description  returns all rows of table permission
#----------------------------------------------------------------------------------------
# return       $rahPermissionIDs | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllInfoPermissions ();
sub DAL_GetAllInfoPermissions () {
    return _CallDriver('GetAllInfoPermissions') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllPermission');
    if (not exists $CacheRef->{'1'}) {
        my $rResultSet = _CallDriver('GetAllInfoPermissions');
        # make copy to primary key cache
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (ObjectID TrusteeID MetaActionID )} = @$hRow{qw (ObjectID TrusteeID MetaActionID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{'1'} = \@PrimaryKeySet;
        # make copy to info cache
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoPermission');
        if (_DoFillInfoCache('InfoPermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join('<||>', @$Row{qw (ObjectID TrusteeID MetaActionID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoPermission', @{$CacheRef->{'1'}})) {
        my @ResultSet;
        foreach my $PrimaryKey (@{$CacheRef->{'1'}}) {
            my $Info = DAL_GetInfoPermission($PrimaryKey->{'ObjectID'}, $PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'MetaActionID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{'1'};
    return DAL_GetAllInfoPermissions();
}

#========================================================================================
# function     DAL_GetInfoPermission
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoPermission($ObjectID, $TrusteeID, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>| reference to hash
#========================================================================================
sub DAL_GetInfoPermission ($$$) {
    my ($ObjectID, $TrusteeID, $MetaActionID) = @_;
    return _CallDriver('GetInfoPermission', $ObjectID, $TrusteeID, $MetaActionID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef    = GetCache($dbi->cacheKey,'InfoPermission');
    my $CacheString =  join('<||>',$ObjectID, $TrusteeID, $MetaActionID);
    $CacheRef->{$CacheString} = _CallDriver('GetInfoPermission', $ObjectID, $TrusteeID, $MetaActionID) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_GetPKeyPermissionsByObjectID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyPermissionsByObjectID($ObjectID, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table permission by primary key
#----------------------------------------------------------------------------------------
# input        $ObjectID | permission object id | int
# input        $TrusteeID | trustee id (optional) | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyPermissionsByObjectID ($;$) {
    my ($ObjectID, $TrusteeID) = @_;
    return _CallDriver('GetPKeyPermissionsByObjectID', $ObjectID, $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByObjectID');
    my $CacheString =  join ('<||>', (map {defined $_ ? $_ : ''} $ObjectID, $TrusteeID));
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyPermissionsByObjectID', $ObjectID, $TrusteeID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoPermissionsByObjectID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoPermissionsByObjectID($ObjectID, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  get full row of table permission by primary key
#----------------------------------------------------------------------------------------
# input        $ObjectID | permission object id | int
# input        $TrusteeID | trustee id (optional) | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoPermissionsByObjectID ($;$);
sub DAL_GetInfoPermissionsByObjectID ($;$) {
    my ($ObjectID, $TrusteeID) = @_;
    return _CallDriver('GetInfoPermissionsByObjectID', $ObjectID, $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByObjectID');
    my $CacheString =  join ('<||>', (map {defined $_ ? $_ : ''} $ObjectID, $TrusteeID));
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoPermissionsByObjectID', $ObjectID, $TrusteeID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (ObjectID TrusteeID MetaActionID )} = @$hRow{qw (ObjectID TrusteeID MetaActionID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoPermission');
        if (_DoFillInfoCache('InfoPermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (ObjectID TrusteeID MetaActionID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoPermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoPermission($PrimaryKey->{'ObjectID'}, $PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'MetaActionID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoPermissionsByObjectID($ObjectID, $TrusteeID);
}

#========================================================================================
# function     DAL_GetPKeyPermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyPermissionsByTrusteeID($TrusteeID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table permission by index i_permission_trustee
#----------------------------------------------------------------------------------------
# input        $TrusteeID | trustee id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyPermissionsByTrusteeID ($) {
    my ($TrusteeID) = @_;
    return _CallDriver('GetPKeyPermissionsByTrusteeID', $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByTrusteeID');
    my $CacheString = $TrusteeID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyPermissionsByTrusteeID', $TrusteeID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoPermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoPermissionsByTrusteeID($TrusteeID);
#----------------------------------------------------------------------------------------
# description  get full row of table permission by index i_permission_trustee
#----------------------------------------------------------------------------------------
# input        $TrusteeID | trustee id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoPermissionsByTrusteeID ($);
sub DAL_GetInfoPermissionsByTrusteeID ($) {
    my ($TrusteeID) = @_;
    return _CallDriver('GetInfoPermissionsByTrusteeID', $TrusteeID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByTrusteeID');
    my $CacheString = $TrusteeID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoPermissionsByTrusteeID', $TrusteeID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (ObjectID TrusteeID MetaActionID )} = @$hRow{qw (ObjectID TrusteeID MetaActionID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoPermission');
        if (_DoFillInfoCache('InfoPermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (ObjectID TrusteeID MetaActionID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoPermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoPermission($PrimaryKey->{'ObjectID'}, $PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'MetaActionID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoPermissionsByTrusteeID($TrusteeID);
}

#========================================================================================
# function     DAL_GetPKeyPermissionsByMetaActionID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyPermissionsByMetaActionID($MetaActionID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table permission by index i_permission_action
#----------------------------------------------------------------------------------------
# input        $MetaActionID | action id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyPermissionsByMetaActionID ($) {
    my ($MetaActionID) = @_;
    return _CallDriver('GetPKeyPermissionsByMetaActionID', $MetaActionID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByMetaActionID');
    my $CacheString = $MetaActionID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyPermissionsByMetaActionID', $MetaActionID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoPermissionsByMetaActionID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoPermissionsByMetaActionID($MetaActionID);
#----------------------------------------------------------------------------------------
# description  get full row of table permission by index i_permission_action
#----------------------------------------------------------------------------------------
# input        $MetaActionID | action id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoPermissionsByMetaActionID ($);
sub DAL_GetInfoPermissionsByMetaActionID ($) {
    my ($MetaActionID) = @_;
    return _CallDriver('GetInfoPermissionsByMetaActionID', $MetaActionID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'PermissionByMetaActionID');
    my $CacheString = $MetaActionID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoPermissionsByMetaActionID', $MetaActionID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (ObjectID TrusteeID MetaActionID )} = @$hRow{qw (ObjectID TrusteeID MetaActionID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoPermission');
        if (_DoFillInfoCache('InfoPermission', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (ObjectID TrusteeID MetaActionID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoPermission', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoPermission($PrimaryKey->{'ObjectID'}, $PrimaryKey->{'TrusteeID'}, $PrimaryKey->{'MetaActionID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoPermissionsByMetaActionID($MetaActionID);
}

#========================================================================================
# function     DAL_GetUsersWithEmailInAlias
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->DAL_GetUsersWithEmailInAlias($ShopID);
#----------------------------------------------------------------------------------------
# description  get users from a given shop where alias is equal to email
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>Alias => 'customer1@mail.com'</li>
#                   <li>UserId => '21670'</li>
#               </ul>
#========================================================================================
sub DAL_GetUsersWithEmailInAlias ($) {
    my ($ShopID) = @_;

    my $aaUsers = _CallDriver('GetUsersWithEmailInAlias', $ShopID);

    return $aaUsers;
}

1;
