#========================================================================================
# package      DE_EPAGES::Permission::DAL::SQL::Permission
# state        private
# description  Generic SQL database interface for tables:
#               <ul>
#                   <li>attributepermission
#                   <li>permission
#               </ul>
#========================================================================================
package DE_EPAGES::Permission::DAL::SQL::Permission;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);

## no critic qw(ProhibitLongLines)

my %TABLE_DEF = (
    'AttributePermission' => {
        'TrusteeID' => ['trusteeid','Integer'],
        'AttributeID' => ['attributeid','Integer'],
        'Permission' => ['permission','Integer'],
    },
    'Permission' => {
        'ObjectID' => ['objectid','Integer'],
        'TrusteeID' => ['trusteeid','Integer'],
        'MetaActionID' => ['metaactionid','Integer'],
        'Allow' => ['allow','Integer'],
    },
);

sub ErrorDefinition {
    return {
        'TableNotExists'  => 'table #Table does not exist at package DE_EPAGES::Permission::DAL::SQL::Permission',
        'ColumnNotExists' => 'column #Column does not exist at table #Table at package DE_EPAGES::Permission::DAL::SQL::Permission',
    };
}

sub TableDef {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[0];
}

sub TableType {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[1];
}

#========================================================================================
# function     InsertAttributePermission
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertAttributePermission($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>TrusteeID     - trustee id           (part of primary key)- int
#                  <li>AttributeID   - attribute id         (part of primary key)- int
#                  <li>Permission    - readable=2, writable=4 - tinyint
#               </ul> | reference to hash
#========================================================================================
sub InsertAttributePermission {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO attributepermission (trusteeid, attributeid, permission) VALUES (?, ?, ?)',
        [ @{$hParameter}{qw( TrusteeID AttributeID Permission )} ]
    );
}

#========================================================================================
# function     UpdateAttributePermission
#----------------------------------------------------------------------------------------
# syntax       UpdateAttributePermission($dbi, $TrusteeID, $AttributeID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to hash
#========================================================================================
sub UpdateAttributePermission {
    my $self = shift;
    my ($dbi, $TrusteeID, $AttributeID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE attributepermission SET ';
        $sql .= join(', ', map { $self->TableDef('AttributePermission',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE trusteeid=? AND attributeid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $TrusteeID, $AttributeID]);
    }
    return;
}

#========================================================================================
# function     DeleteAttributePermission
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteAttributePermission($dbi, $TrusteeID, $AttributeID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
#========================================================================================
sub DeleteAttributePermission {
    my $self = shift;
    my ($dbi, $TrusteeID, $AttributeID) = @_;
    $dbi->do('DELETE FROM attributepermission WHERE trusteeid=? AND attributeid=?', [$TrusteeID, $AttributeID]);
    return;
}

#========================================================================================
# function     GetAllPKeyAttributePermissions
#----------------------------------------------------------------------------------------
# syntax       $ahAttributePermissionIDs = $Package->GetAllPKeyAttributePermissions($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahAttributePermissionIDs | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyAttributePermissions {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT trusteeid, attributeid FROM attributepermission', [], ['Integer', 'Integer'], [qw(TrusteeID AttributeID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoAttributePermissions
#----------------------------------------------------------------------------------------
# syntax       $ahAttributePermissionIDs = $Package->GetAllInfoAttributePermissions($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahAttributePermissionIDs | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoAttributePermissions {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT trusteeid, attributeid, permission FROM attributepermission', [],
        ['Integer', 'Integer', 'Integer'],
        [qw(TrusteeID AttributeID Permission)]
    );
}

#========================================================================================
# function     GetInfoAttributePermission
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoAttributePermission($dbi, $TrusteeID, $AttributeID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $AttributeID   | attribute id         (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>| reference to hash
#========================================================================================
sub GetInfoAttributePermission {
    my $self = shift;
    my ($dbi, $TrusteeID, $AttributeID) = @_;
    return $dbi->execute('SELECT trusteeid, attributeid, permission FROM attributepermission WHERE trusteeid=? AND attributeid=?',
        [$TrusteeID, $AttributeID],
        ['Integer', 'Integer', 'Integer'],
        [qw(TrusteeID AttributeID Permission)]
    )->[0];
}

#========================================================================================
# function     GetPKeyAttributePermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyAttributePermissionsByTrusteeID($dbi, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table attributepermission by primary key.
#               Uses the index pk_attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID | trustee id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyAttributePermissionsByTrusteeID {
    my $self = shift;
    my ($dbi, $TrusteeID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'trusteeid = ?';
    unshift @Parameter, $TrusteeID;

    my $sql = 'SELECT trusteeid, attributeid FROM attributepermission WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(TrusteeID AttributeID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoAttributePermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoAttributePermissionsByTrusteeID($dbi, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table attributepermission by primary key.
#               Uses the index pk_attributepermission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID | trustee id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoAttributePermissionsByTrusteeID {
    my $self = shift;
    my ($dbi, $TrusteeID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'trusteeid = ?';
    unshift @Parameter, $TrusteeID;

    my $sql = 'SELECT trusteeid, attributeid, permission FROM attributepermission WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'Integer', 'Integer'],
        [qw(TrusteeID AttributeID Permission)]
    );
}

#========================================================================================
# function     GetPKeyAttributePermissionsByAttributeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyAttributePermissionsByAttributeID($dbi, $AttributeID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table attributepermission by index i_attributepermission_attributeid.
#               Uses the index i_attributepermission_attributeid.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $AttributeID | attribute id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyAttributePermissionsByAttributeID {
    my $self = shift;
    my ($dbi, $AttributeID) = @_;
    my @Parameter = ($AttributeID);
    my $sql = 'SELECT trusteeid, attributeid FROM attributepermission WHERE attributeid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(TrusteeID AttributeID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoAttributePermissionsByAttributeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoAttributePermissionsByAttributeID($dbi, $AttributeID);
#----------------------------------------------------------------------------------------
# description  get full row of table attributepermission by index i_attributepermission_attributeid
#               Uses the index i_attributepermission_attributeid.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $AttributeID | attribute id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoAttributePermissionsByAttributeID {
    my $self = shift;
    my ($dbi, $AttributeID) = @_;
    return $dbi->execute('SELECT trusteeid, attributeid, permission FROM attributepermission WHERE attributeid = ?',
        [$AttributeID],
        ['Integer', 'Integer', 'Integer'],
        [qw(TrusteeID AttributeID Permission)]
    );
}

#========================================================================================
# function     GetInfoAttributePermissionsByClassID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoAttributePermissionsByClassID($dbh, $AttributeID);
#----------------------------------------------------------------------------------------
# description  get {attributeid, permission} rows of table attributepermission by
#               class id and trustee id. Since the corresponding class id for each
#               attribute id is stored separately in the table attribute, both tables
#               needs to be joined.
#               Uses the index i_attributepermission_trusteeid_attributeid.
#               Uses the primary key of table attribute.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ClassID | class id | int
# input        $TrusteeID | trustee id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>AttributeID   - attribute id         (part of primary key)- int
#                   <li>Permission    - readable=2, writable=4 - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoAttributePermissionsByClassID {
    my $self = shift;
    my ($dbi, $ClassID, $TrusteeID) = @_;
    return $dbi->execute(<<'END_SQL', [$ClassID, $TrusteeID], [ 'Integer', 'Integer', 'Integer' ], [qw (TrusteeID AttributeID Permission)]);
SELECT DISTINCT ap.trusteeid, ap.attributeid, ap.permission
FROM attribute AS a
INNER JOIN attributepermission AS ap ON (a.attributeid = ap.attributeid)
WHERE a.attributeclassid = ? AND ap.trusteeid = ?
END_SQL
}

#========================================================================================
# function     InsertPermission
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertPermission($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ObjectID      - permission object id (part of primary key)- int
#                  <li>TrusteeID     - trustee id           (part of primary key)- int
#                  <li>MetaActionID  - action id            (part of primary key)- int
#                  <li>Allow         - allow=1, deny=2      - tinyint
#               </ul> | reference to hash
#========================================================================================
sub InsertPermission {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO permission (objectid, trusteeid, metaactionid, allow) VALUES (?, ?, ?, ?)',
        [ @{$hParameter}{qw( ObjectID TrusteeID MetaActionID Allow )} ]
    );
}

#========================================================================================
# function     UpdatePermission
#----------------------------------------------------------------------------------------
# syntax       UpdatePermission($dbi, $ObjectID, $TrusteeID, $MetaActionID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to hash
#========================================================================================
sub UpdatePermission {
    my $self = shift;
    my ($dbi, $ObjectID, $TrusteeID, $MetaActionID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE permission SET ';
        $sql .= join(', ', map { $self->TableDef('Permission',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE objectid=? AND trusteeid=? AND metaactionid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $ObjectID, $TrusteeID, $MetaActionID]);
    }
    return;
}

#========================================================================================
# function     DeletePermission
#----------------------------------------------------------------------------------------
# syntax       $Package->DeletePermission($dbi, $ObjectID, $TrusteeID, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
#========================================================================================
sub DeletePermission {
    my $self = shift;
    my ($dbi, $ObjectID, $TrusteeID, $MetaActionID) = @_;
    $dbi->do('DELETE FROM permission WHERE objectid=? AND trusteeid=? AND metaactionid=?', [$ObjectID, $TrusteeID, $MetaActionID]);
    return;
}

#========================================================================================
# function     GetAllPKeyPermissions
#----------------------------------------------------------------------------------------
# syntax       $ahPermissionIDs = $Package->GetAllPKeyPermissions($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahPermissionIDs | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyPermissions {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT objectid, trusteeid, metaactionid FROM permission', [], ['Integer', 'Integer', 'Integer'], [qw(ObjectID TrusteeID MetaActionID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoPermissions
#----------------------------------------------------------------------------------------
# syntax       $ahPermissionIDs = $Package->GetAllInfoPermissions($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahPermissionIDs | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoPermissions {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT objectid, trusteeid, metaactionid, allow FROM permission', [],
        ['Integer', 'Integer', 'Integer', 'Integer'],
        [qw(ObjectID TrusteeID MetaActionID Allow)]
    );
}

#========================================================================================
# function     GetInfoPermission
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoPermission($dbi, $ObjectID, $TrusteeID, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ObjectID      | permission object id (part of primary key) | int
# input        $TrusteeID     | trustee id           (part of primary key) | int
# input        $MetaActionID  | action id            (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>| reference to hash
#========================================================================================
sub GetInfoPermission {
    my $self = shift;
    my ($dbi, $ObjectID, $TrusteeID, $MetaActionID) = @_;
    return $dbi->execute('SELECT objectid, trusteeid, metaactionid, allow FROM permission WHERE objectid=? AND trusteeid=? AND metaactionid=?',
        [$ObjectID, $TrusteeID, $MetaActionID],
        ['Integer', 'Integer', 'Integer', 'Integer'],
        [qw(ObjectID TrusteeID MetaActionID Allow)]
    )->[0];
}

#========================================================================================
# function     GetPKeyPermissionsByObjectID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyPermissionsByObjectID($dbi, $ObjectID, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table permission by primary key.
#               Uses the index pk_permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ObjectID | permission object id | int
# input        $TrusteeID | trustee id (optional) | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyPermissionsByObjectID {
    my $self = shift;
    my ($dbi, $ObjectID, $TrusteeID) = @_;

    my (@WhereClauses, @Parameter);
    if (defined $TrusteeID) {
        unshift @WhereClauses, 'trusteeid = ?';
        unshift @Parameter, $TrusteeID;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'objectid = ?';
    unshift @Parameter, $ObjectID;

    my $sql = 'SELECT objectid, trusteeid, metaactionid FROM permission WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer', 'Integer'], [qw(ObjectID TrusteeID MetaActionID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoPermissionsByObjectID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoPermissionsByObjectID($dbi, $ObjectID, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table permission by primary key.
#               Uses the index pk_permission.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ObjectID | permission object id | int
# input        $TrusteeID | trustee id (optional) | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoPermissionsByObjectID {
    my $self = shift;
    my ($dbi, $ObjectID, $TrusteeID) = @_;

    my (@WhereClauses, @Parameter);
    if (defined $TrusteeID) {
        unshift @WhereClauses, 'trusteeid = ?';
        unshift @Parameter, $TrusteeID;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'objectid = ?';
    unshift @Parameter, $ObjectID;

    my $sql = 'SELECT objectid, trusteeid, metaactionid, allow FROM permission WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'Integer', 'Integer', 'Integer'],
        [qw(ObjectID TrusteeID MetaActionID Allow)]
    );
}

#========================================================================================
# function     GetPKeyPermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyPermissionsByTrusteeID($dbi, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table permission by index i_permission_trustee.
#               Uses the index i_permission_trustee.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID | trustee id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyPermissionsByTrusteeID {
    my $self = shift;
    my ($dbi, $TrusteeID) = @_;
    my @Parameter = ($TrusteeID);
    my $sql = 'SELECT objectid, trusteeid, metaactionid FROM permission WHERE trusteeid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer', 'Integer'], [qw(ObjectID TrusteeID MetaActionID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoPermissionsByTrusteeID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoPermissionsByTrusteeID($dbi, $TrusteeID);
#----------------------------------------------------------------------------------------
# description  get full row of table permission by index i_permission_trustee
#               Uses the index i_permission_trustee.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $TrusteeID | trustee id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoPermissionsByTrusteeID {
    my $self = shift;
    my ($dbi, $TrusteeID) = @_;
    return $dbi->execute('SELECT objectid, trusteeid, metaactionid, allow FROM permission WHERE trusteeid = ?',
        [$TrusteeID],
        ['Integer', 'Integer', 'Integer', 'Integer'],
        [qw(ObjectID TrusteeID MetaActionID Allow)]
    );
}

#========================================================================================
# function     GetPKeyPermissionsByMetaActionID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyPermissionsByMetaActionID($dbi, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table permission by index i_permission_action.
#               Uses the index i_permission_action.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $MetaActionID | action id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyPermissionsByMetaActionID {
    my $self = shift;
    my ($dbi, $MetaActionID) = @_;
    my @Parameter = ($MetaActionID);
    my $sql = 'SELECT objectid, trusteeid, metaactionid FROM permission WHERE metaactionid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer', 'Integer'], [qw(ObjectID TrusteeID MetaActionID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoPermissionsByMetaActionID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoPermissionsByMetaActionID($dbi, $MetaActionID);
#----------------------------------------------------------------------------------------
# description  get full row of table permission by index i_permission_action
#               Uses the index i_permission_action.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $MetaActionID | action id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ObjectID      - permission object id (part of primary key)- int
#                   <li>TrusteeID     - trustee id           (part of primary key)- int
#                   <li>MetaActionID  - action id            (part of primary key)- int
#                   <li>Allow         - allow=1, deny=2      - tinyint
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoPermissionsByMetaActionID {
    my $self = shift;
    my ($dbi, $MetaActionID) = @_;
    return $dbi->execute('SELECT objectid, trusteeid, metaactionid, allow FROM permission WHERE metaactionid = ?',
        [$MetaActionID],
        ['Integer', 'Integer', 'Integer', 'Integer'],
        [qw(ObjectID TrusteeID MetaActionID Allow)]
    );
}

#========================================================================================
# function     GetUsersWithEmailInAlias
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetUsersWithEmailInAlias($dbi, $ShopID);
#----------------------------------------------------------------------------------------
# description  get users from a given shop where alias is equal to email
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ShopID | shop id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>Alias => 'customer1@mail.com'</li>
#                   <li>UserId => '21670'</li>
#               </ul>
#========================================================================================
sub GetUsersWithEmailInAlias {
    my $self = shift;
    my ($dbi, $ShopID) = @_;
    return $dbi->execute('SELECT u.userid, o.alias FROM object AS o INNER JOIN users AS u ON o.objectid = u.userid where o.alias = u.email and o.siteid = ?',
        , [$ShopID], ['String'], [qw(UserId Alias)]
    );
}

1;
