#========================================================================================
# package      DE_EPAGES::Presentation::API::TLE::PermissionHandler
# state        public
#========================================================================================
package DE_EPAGES::Presentation::API::TLE::PermissionHandler;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Core::API::Url qw (UrlEncodeValue);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Presentation::API::DereferrerToken qw (GenerateDereferrerToken);
use DE_EPAGES::Permission::API::Permission qw (
    ActionByName
    ExistsActionByName
    IsActionAllowed
    RoleByName
);
use DE_EPAGES::Presentation::API::Table::Session qw (GetInfoSession);
use DE_EPAGES::Presentation::API::Session qw (
    CreateAdminSFSessionID
    GetAdminSFSessionCookie
    TestAdminSFSession
);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    my $TLEProcessor = $Params->{'Processor'};
    my $Handler = __PACKAGE__->new();
    $Handler->register($TLEProcessor);
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Presentation::API::TLE::PermissionHandler;
# example      my $PermissionHandler  = new DE_EPAGES::Presentation::API::TLE::PermissionHandler;
#----------------------------------------------------------------------------------------
# description  Creates a new handler.
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  This function registers this handle on the processor.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    $Processor->registerHandler('FunctionHandler', $self, 'ISACTIONALLOWED');
    $Processor->registerHandler('FunctionHandler', $self, 'HASROLE');
    $Processor->registerHandler('FunctionHandler', $self, 'CanAction');
    $Processor->registerHandler('FunctionHandler', $self, 'DEREFER');

    $Processor->registerHandler('VariableHandler', $self, 'MerchantSFSession');
    $Processor->registerHandler('VariableHandler', $self, 'IsMerchantSFAllowed');

    return 1;
}
#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    return _CryptAdminSession($Processor) if $TLEName eq 'MerchantSFSession';

    return _IsMerchantSFAllowed($Processor) if $TLEName eq 'IsMerchantSFAllowed';

    return;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    return 1 if ElementOf($TLEName, qw (MerchantSFSession IsMerchantSFAllowed));
    return undef;
}

#========================================================================================
# function     ISACTIONALLOWED
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ISACTIONALLOWED(#ActionName)
# syntax       #ISACTIONALLOWED(#ActionName, #Object)
# syntax       #ISACTIONALLOWED(#ActionName, #Object, #User)
# example      #IF( #ISACTIONALLOWED("View", #Shop.Object, #SESSION.User.Object) )
#                 ...
#               #ENDIF
#----------------------------------------------------------------------------------------
# description  Determines whether an action is allowed for the given object and user.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        #ActionName | name of an action | string
# input        #Object | object of the action, default is object in current scope
#                       (optional) | ref.object
# input        #User | user to check for this permission, default is the current
#                       user (optional) | ref.object
# return       #Allowed | true if action is allowed | boolean
#========================================================================================
sub ISACTIONALLOWED {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($ActionName, $Object, $User) = @$raParams;
    $Object = $Processor->tle('Object') unless defined $Object;

    unless (defined $ActionName) {
        warn('TLE function ISACTIONALLOWED: ActionName is undefined');
        return 0;
    }

    unless (defined $Object) {
        warn('TLE function ISACTIONALLOWED: Object is undefined');
        return 0;
    }

    my $Action;
    eval {
        $Action = ActionByName($Object->class, $ActionName);
    };
    if (ExistsError()) {
        my $Error = GetError();
        return 0 if $Error->code eq 'ActionNotFound';
        Error($Error);
    }
    unless (defined $Action) {
        warn('TLE function ISACTIONALLOWED: Action is undefined');
        return 0;
    }

    $User = $Processor->tle('Session.User.Object') unless defined $User;

    return IsActionAllowed( $Object, $User, $Action );
}

#========================================================================================
# function     HASROLE
# state        public
#----------------------------------------------------------------------------------------
# syntax       #HASROLE(#RoleClass, #RoleName)
# syntax       #HASROLE(#RoleClass, #RoleName, #Object)
# syntax       #HASROLE(#RoleClass, #RoleName, #Object, #User)
# example      #IF( #HASROLE("Object", "View", #Shop.Object, #SESSION.User.Object) )
#                 ..
#               #ENDIF
#----------------------------------------------------------------------------------------
# description  Determines, whether the user has a specific role for an object.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        #RoleClass | class of the role | string
# input        #RoleName | name of the role | string
# input        #Object | object to check for, default is object in current scope
#                       (optional) | ref.object
# input        #User | user to check for this role, default is the current
#                       user (optional) | ref.object
# return       #Allowed | true if role is allowed | boolean
#========================================================================================
sub HASROLE {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($RoleClassName, $RoleName, $Object, $User) = @$raParams;
    $Object = $Processor->tle('Object') unless defined $Object;
    $User = $Processor->tle('Session.User.Object') unless defined $User;

    return 0 unless defined $User;

    unless (defined $RoleClassName) {
        warn('TLE function HASROLE: RoleClassName is undefined');
        return 0;
    }

    my $RoleClass = LoadClassByAlias($RoleClassName);
    unless (defined $RoleClass) {
        warn('TLE function HASROLE: RoleClass does not exist');
        return 0;
    }

    unless (defined $RoleName) {
        warn('TLE function HASROLE: RoleName is undefined');
        return 0;
    }

    my $Role = RoleByName($RoleClass, $RoleName);
    unless (defined $RoleClass) {
        warn('TLE function HASROLE: Role does not exist');
        return 0;
    }

    unless (defined $Object) {
        warn('TLE function HASROLE: Object is undefined');
        return 0;
    }

    foreach my $permission (@{$Object->get('Permissions')}) {
        return 1 if ($permission->{'Trustee'}->id == $User->id &&
                     $permission->{'MetaAction'}->id == $Role->id &&
                     $permission->{'Allow'});
    }

    return 0;
}

# helper function for crypting the session id of the merchant
# with the shop guid as cryptkey
sub _CryptAdminSession {
    my ($Processor) = @_;

    my $Object  = $Processor->tle('Object');
    my $Site    = $Object->getSite();

    my $SessionID               = _GetSessionID($Processor);
    my $CryptedAdminSessionID   = CreateAdminSFSessionID($Site,$SessionID);

    return $CryptedAdminSessionID;
}

sub _GetSessionID {
    my ($Processor) = @_;

    my $SessionID = $Processor->tle('INPUT.SessionID');

    unless (isValid $SessionID) {
        my $hSession = GetInfoSession($Processor->tle('INPUT.IntSessionID'));
        $SessionID = $hSession->{'SessionID'};
    }

    return $SessionID;
}

# helper function for decrypting the MerchantSF cookie value and test if a
# valid session is available for the merchant
sub _IsMerchantSFAllowed {
    my ($Processor, $aParameter) = @_;

    my $Site = $Processor->tle('Object')->getSite();
    return 1 if TestAdminSFSession($Site,GetAdminSFSessionCookie()) == 1;
    return 0;
}

#========================================================================================
# function     CanAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       #CanAction(#Object, #ActionName )
# example      #IF(#CanAction( #Object, "MBO-ViewSettings")) ... #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns true if the given Action is available for that object,
#               considering the inheritance chain of the object
#----------------------------------------------------------------------------------------
# input        #Object | the object to check | object
# input        #ActionName | the name of the Action, e.g. "MBO-ViewSettings" | string
# return       #Result | true if the Action exists for this Object | boolean
#========================================================================================
sub CanAction {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $ActionName) = @$aParams;

    return ExistsActionByName( $Object->class, $ActionName);
}

#========================================================================================
# function     DEREFER
# state        private
#----------------------------------------------------------------------------------------
# syntax       #DEREFER(#URL)
# example      #DEREFER("http://www.epages.com/de/produkte/")
# example      #DEREFER(#System.MyURL)
#----------------------------------------------------------------------------------------
# description  Returns the action and parameters for a dereferred url call
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $URL | tle name | string
# return       $Value | action for dereferred call | string
#========================================================================================
sub DEREFER {
    my $self = shift;
    my ($Processor, $raParams ) = @_;
    my ($TLEURL) = @$raParams;
    return undef unless $TLEURL;

    my $URL = $Processor->tle($TLEURL) // $TLEURL;

    # create a security token
    my $Site = $Processor->tle('Object')->getSite();
    my $Token = GenerateDereferrerToken($URL);

    $URL = UrlEncodeValue($URL);

    my $AdminURLSSL = $Site->get('WebUrlAdminSSL');

    return $AdminURLSSL .
           ($AdminURLSSL =~ /\?/ ? '&' : '/?') .
           "ViewAction=DereferredRedirect&URL=".$URL."&nonce=".$Token;

}

1;
