#========================================================================================
# package      DE_EPAGES::Presentation::API::User
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides an interface for User in Sessions.
#               Note: Use $Servlet->createAnonymousUser($Site) to create a new session
#               and a User object without login. See
#               <function DE_EPAGES::Presentation::Servlet::Permission::createAnonymousUser>.
#========================================================================================
package DE_EPAGES::Presentation::API::User;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    CreateUser
    ExistsUser
    ExistsUserByEMail
    GetUser
    GetUserFolder
    GetUsersByEMail
    GetVerfiedUsersByEMail
);

use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestHash TestString);
use DE_EPAGES::Object::API::Factory     qw ( LoadObject LoadClassByAlias );
use DE_EPAGES::Presentation::DAL::Users qw ( DAL_GetShopUserIDsByEMail DAL_GetVerifiedShopUserIDsByEMail);

#========================================================================================
# function     GetUserFolder
#----------------------------------------------------------------------------------------
# syntax       $Folder = GetUserFolder($Site);
#----------------------------------------------------------------------------------------
# description  Returns the folder for registered users of the site.
#----------------------------------------------------------------------------------------
# input        $Site | Site object (e.g. shop or system) | object
# return       $Folder | a folder object | object
#========================================================================================
sub GetUserFolder ($) {
    my ($ParentObject) = @_;
    TestObject( 'ParentObject', $ParentObject );
    return $ParentObject->folder('Users', {'Position'=>120});
}

#========================================================================================
# function     CreateUser
#----------------------------------------------------------------------------------------
# syntax       $User = CreateUser($Site, $hInfo);
#----------------------------------------------------------------------------------------
# description  Generates a registered user.
#----------------------------------------------------------------------------------------
# input        $Site | Site object (e.g. shop or system) of new user | object
# input        $hInfo |  user attributes like Alias/Password | ref.hash
# return       $User | a new user | object
#========================================================================================
sub CreateUser ($$) {
    my ($ParentObject, $hInfo) = @_;
    TestHash( 'hInfo', $hInfo );

    # get folder for registered users
    my $UserFolder = GetUserFolder($ParentObject);

    # insert user
    return $UserFolder->insertChild('User', $hInfo);
}

#========================================================================================
# function     ExistsUser
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsUser($Site, $Alias);
#----------------------------------------------------------------------------------------
# description  Looks if a registered user with the given login exists in the site.
#----------------------------------------------------------------------------------------
# input        $Site | Site object (e.g. shop or system) | object
# input        $Alias |  login of user | ref.hash
# return       $Exists | exists user | boolean
#========================================================================================
sub ExistsUser ($$) {
    my ($ParentObject, $Login) = @_;
    TestString( 'Login', $Login );

    # get folder for registered users
    my $UserFolder = GetUserFolder($ParentObject);

    # insert user
    return $UserFolder->existsChild($Login);
}
#========================================================================================
# function     ExistsUserByEMail
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsUserByEMail($Site, $Alias);
#----------------------------------------------------------------------------------------
# description  Looks if a registered user with the given login or email exists in the site.
#----------------------------------------------------------------------------------------
# input        $Site | Site object (e.g. shop or system) | object
# input        $EMail |  login/email of user | ref.hash
# return       $Exists | exists user | boolean
#========================================================================================
sub ExistsUserByEMail ($$) {
    my ($ParentObject, $EMail) = @_;
    TestString( 'EMail', $EMail );

    my $Class = LoadClassByAlias('User');
    my $aUserIDs = DAL_GetShopUserIDsByEMail($EMail, $ParentObject->id, $Class->id);

    return scalar(@$aUserIDs) ? 1 : 0;
}

#========================================================================================
# function     GetVerfiedUsersByEMail
#----------------------------------------------------------------------------------------
# syntax       $aUsers = GetVerfiedUsersByEMail($EMail, $Site);
#----------------------------------------------------------------------------------------
# description  Returns all users by email which are verified.
#----------------------------------------------------------------------------------------
# input        $EMail |  email or login of user | ref.hash
# input        $Site | Site object (e.g. shop or system) of the user | object
# return       $aUsers | users | array.object
#========================================================================================
sub GetVerfiedUsersByEMail ($$) {
    my ($EMail, $Site) = @_;
    TestString( 'EMail', $EMail );

    # get folder for registered users
    my $Class = LoadClassByAlias('User');

    # get all users by email or alias
    my $aUserIDs = DAL_GetVerifiedShopUserIDsByEMail($EMail, $Site->id, $Class->id);

    # return all found users
    return [ map { LoadObject( $_ ) } @$aUserIDs ];
}

#========================================================================================
# function     GetUsersByEMail
#----------------------------------------------------------------------------------------
# syntax       $aUsers = GetUsersByEMail($EMail, $Site);
#----------------------------------------------------------------------------------------
# description  Returns all users by email.
#----------------------------------------------------------------------------------------
# input        $EMail |  email or login of user | ref.hash
# input        $Site | Site object (e.g. shop or system) of the user | object
# return       $aUsers | users | array.object
#========================================================================================
sub GetUsersByEMail ($$) {
    my ($EMail, $Site) = @_;
    TestString( 'EMail', $EMail );

    # get folder for registered users
    my $Class = LoadClassByAlias('User');

    # get all users by email or alias
    my $aUserIDs = DAL_GetShopUserIDsByEMail($EMail, $Site->id, $Class->id);

    # return all found users
    return [ map { LoadObject( $_ ) } @$aUserIDs ];
}

#========================================================================================
# function     GetUser
#----------------------------------------------------------------------------------------
# syntax       $User = GetUser($Site, $Alias);
#----------------------------------------------------------------------------------------
# description  Returns a registered user exists by login.
#----------------------------------------------------------------------------------------
# input        $Site | Site object (e.g. shop or system) of the user | object
# input        $Alias |  login of user | ref.hash
# return       $User | a registered user | object
#========================================================================================
sub GetUser ($$) {
    my ($ParentObject, $Login) = @_;
    TestString( 'Login', $Login );

    # get folder for registered users
    my $UserFolder = GetUserFolder($ParentObject);

    # return user
    return $UserFolder->child($Login);
}

1;
