#========================================================================================
# package      DE_EPAGES::Shop::Hooks::Session
# state        public
# description  Updates the LastMerchantLogin date of a shop whenever a merchant logs on.
#========================================================================================
package DE_EPAGES::Shop::Hooks::Session;

use strict;
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Presentation::API::Session qw (GetSessionAttribute);
use DE_EPAGES::Permission::API::Permission qw (HaveSameSite);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );


#========================================================================================
# function     OnAfterLoginUser
#----------------------------------------------------------------------------------------
# syntax       OnAfterLoginUser( $hParams );
#----------------------------------------------------------------------------------------
# description  Updates the LastMerchantLogin date of a shop when the logged on user is
#               a merchant
#               (Hook UI_AfterLoginUser)
#========================================================================================
sub OnAfterLoginUser {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    my $Shop = $Servlet->object->getSite();
    my $User = $Servlet->user;

    # set the shopattribute LastMerchantLogin if the user is a merchant
    eval {
        $Shop->set({'LastMerchantLogin' => GetCurrentDateTime() }) if $User->get('IsMerchant');
    };

    # error is caused by long running process which blocks shop table (EPC-234)
    if (ExistsError()) {
        my $Error = GetError();
        my $ErrorCode = $Error->code;
        my $ErrorMsg = $Error->shortMessage;

        if ($ErrorCode eq 'DBIExecute' && $ErrorMsg =~ /Lock wait timeout exceeded/) {

            Warning($Error);
        } else {

            Error($Error);
        }
    }
}

#========================================================================================
# function     OnTestAdminSFSession
#----------------------------------------------------------------------------------------
# syntax       OnTestAdminSFSession( $hParams );
#----------------------------------------------------------------------------------------
# description  Test if the shop is closed by merchant and if the session user is
#               a merchant, if the internal session id exists.
#               (Hook API_TestAdminSFSession)
#========================================================================================
sub OnTestAdminSFSession {
    my ($hParams) = @_;
    my $IntSessionID = $hParams->{'IntSessionID'};
    my $Shop = $hParams->{'Site'};

    $hParams->{'ClosedByMerchant'} = $Shop->get('ClosedByMerchant');

    if ($IntSessionID){
        my $User = GetSessionAttribute($IntSessionID,'User');
        return unless defined $User;
        # check Site
        unless (HaveSameSite($User, $Shop)) {
            die('Try to use a foreign site (' . $Shop->id . ') by (' . $User->id . ')');
        }
        $hParams->{'IsMerchant'} = $User->get('IsMerchant');
    }

    return;
}

1;
