use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient;
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Shop::API::Constants qw( UNITY_STOREFRONT_APP_NAME UNITY_STOREFRONT_APP_TYPE );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );

#========================================================================================
# function         RenewUnityStorefrontApp
# state            public
#----------------------------------------------------------------------------------------
# syntax           RenewUnityStorefrontApp();
#----------------------------------------------------------------------------------------
# description      Renews the Unity storefront app.
#========================================================================================
sub RenewUnityStorefrontApp {
    my ($ShopAlias, $Force) = @_;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            if (!$Force && $Shop->get('IsUnityStorefrontAppRenewed')) {
                print 'Unity Storefront App for Shop ' . $ShopAlias . " got already renewed.\n"
                 . "Rerun with -force flag, to renew nevertheless. \n";
            } else {
                _renewUnityStorefrontApp($Shop);
            }
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                # skip shops with already renews app
                next if (!$Force && $Shop->get('IsUnityStorefrontAppRenewed'));

                _renewUnityStorefrontApp($Shop);
            };
            warn('Unity Storefront App renewal failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    return;
}

sub _renewUnityStorefrontApp {
    my ($Shop) = @_;

    if (EpagesConfig->get('DE_EPAGES::Object', 'UnityUseAuth', 1)) {
        # Renews the Unity storefront app
        my $Client = DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient->create;
        my $Credentials = $Client->createAndSaveCredentials($Shop->get('GUID'), {
            'name' => UNITY_STOREFRONT_APP_NAME,
            'type' => UNITY_STOREFRONT_APP_TYPE,
        });

        my $hContent->{'jrest'}->{'oauth'} = {
            'version'     => '2.0',
            'credentials' => {
                'clientId'           => $Credentials->clientId,
                'clientSecret'       => $Credentials->clientSecret,
                'accessToken'        => $Credentials->accessToken
            }
        };

        # send credentials to the Unity database
        $Shop->sendUnityRequest('PATCH', '/shop', undef, $hContent);
        # set flag on shop to avoid multiple renewals
        $Shop->set({'IsUnityStorefrontAppRenewed' => 1});
    }
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Force;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'force'       => \$Force,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { RenewUnityStorefrontApp($ShopAlias, $Force); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Renews the Unity storefront app.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -force      renew app regardless if it got renewed already
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );