#==============================================================================
# package      DE_EPAGES::Shop::UI::User
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#==============================================================================
package DE_EPAGES::Shop::UI::User;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Password qw ( CryptPassword CheckPassword );
use DE_EPAGES::Presentation::API::User qw ( GetUsersByEMail );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#==============================================================================
# Event handlers
#==============================================================================

sub SaveUserDetails {
    my $self = shift;
    my ($Servlet) = @_;
    my $User   = $Servlet->object;
    my $CurrentUser = $Servlet->user;

    if(     $User->id != $CurrentUser->id
        and $User->get('IsMerchant')
        and not $CurrentUser->get('IsSuperMerchant') ){
        $Servlet->form->executeFormError({
            'Reason'=> 'ACTION_ILLIGAL',
            'Form'  => 'Save',
            'Name'  => 'Action'
        });
    }

    my $hValues = $Servlet->form->form($User,'SaveSettings');
    $hValues->{'IsActivated'} = $Servlet->params('IsActivated') ? 1:0;

    # merchant tries to change the users email address
    my $OldEmail = $User->get('EMail') // '';
    if ($hValues->{'EMail'} ne $OldEmail) {
        # test for existing aliases or email addresses with the given mail
        my $aUsers = GetUsersByEMail($hValues->{'EMail'},$User->getSite());
        if( scalar(@$aUsers)) {
            $Servlet->form->executeFormError({
                'Reason'=> 'EMAIL_EXISTS',
                'Name'  => 'EMail',
                'Form'  => 'Form',
            });
        }
    }

    my $NewPassword ;
    if( defined $hValues->{'NewPassword'} && $hValues->{'NewPassword'} =~ /\S/ ) {
        $NewPassword = $hValues->{'NewPassword'} ;

        my $OldUserPassword = $User->get('Password');
        # check old password, if ...
        # - current user is not defined
        # - current user is not a merchant
        # - current user is a merchant and edits his own profile
        if ((not defined($CurrentUser) or not $CurrentUser->get('IsMerchant') or $CurrentUser->id == $User->id) and
            not (CheckPassword($hValues->{'OldPassword'}, $OldUserPassword))) {
            $Servlet->form->addFormError({
                'Reason'=> 'WRONG_OLD_PWD',
                'Form'  => 'Save',
                'Name'  => 'OldPassword',
            }) ;
        }
        if (not defined $hValues->{'PasswordConf'} or not ($hValues->{'PasswordConf'} eq $NewPassword) ){
            $Servlet->form->addFormError({
                'Reason'=> 'PWD_NOT_SAME',
                'Form'  => 'Save',
                'Name'  => 'NewPassword',
            }) ;
            $Servlet->form->addFormError({
                'Reason'=> 'PWD_NOT_SAME',
                'Form'  => 'Save',
                'Name'  => 'PasswordConf',
            }) ;
        }
        TriggerHook(
            'UI_BeforeSaveShopUserPassword',
            {
                'Servlet'           => $Servlet,
                'User'              => $User,
                'Password'          => $NewPassword,
                'FormName'          => 'Save',
                'FormField'         => 'NewPassword',
            }
        );
        $Servlet->form->executeFormError;

        #crypt password if new password submitted
        $hValues->{'Password'} = CryptPassword($NewPassword);

        #show message that pwd was changed
        $Servlet->vars('PWD_CHANGED', 1);
    }

    # set locale for user
    my $Shop = $User->getSite();
    my $Locale = $Shop->siteLocale( $hValues->{'LocaleID'} );

    $hValues->{'LocaleID'}   = $Locale->get('Locale')->{'LocaleID'};
    $hValues->{'CurrencyID'} = $Locale->get('Locale')->{'CurrencyID'};

    $hValues->{'IsHtmlEMailAllowed'} = $Servlet->params('IsHtmlEMailAllowed') if defined $Servlet->params('IsHtmlEMailAllowed');

    delete $hValues->{$_} foreach qw(NewPassword PasswordConf OldPassword SiteLocaleID);
    $User->set($hValues);
    if ( defined $NewPassword ) {
        TriggerHook(
            'UI_AfterSaveShopUserPassword',
            {
                'Servlet'           => $Servlet,
                'User'              => $User,
                'Password'          => $NewPassword
            }
        );
    }

    return;
}

#========================================================================================
# function     Delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Delete( $Servlet );
#----------------------------------------------------------------------------------------
# description  Deletes the current user object (merchant).
#               <br/>
#               Throws an error if user is supermerchant or if one merchant tries to
#               delete another one.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Parent | parent object of $Servlet->object | object
#========================================================================================
sub Delete {
    my $self = shift;
    my ($Servlet) = @_;
    my $User   = $Servlet->object;
    Error('CANNOT_DELETE_SUPERMERCHANT', {
        'SuperMerchant'  => $User->id,
        'ShopID' => $User->getSite()->id,
    }) if $User->get('IsSuperMerchant');

    my $CurrentUser = $Servlet->user;
    if($User->get('IsMerchant') and not $CurrentUser->get('IsSuperMerchant') ){
        Error('MERCHANT_CANNOT_DELETE_MERCHANT', {
            'Merchant'  => $User->id,
            'ShopID' => $User->getSite()->id,
        });
    }

    $self->SUPER::Delete($Servlet);
    return;
}

#========================================================================================
# function     SetStartPageTab
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetStartPageTab($Servlet);
#----------------------------------------------------------------------------------------
# description  Changes the value of the shop attribute StartPageTab.
#               Values: 0 - 'Overview' | 1 - 'Dashboard' | 2 - 'NextSteps'
#----------------------------------------------------------------------------------------
# input        $StartPageTab | StartPageTab flag | Integer
#========================================================================================
sub SetStartPageTab {
    my $self = shift;
    my ($Servlet) = @_;
    my $User = $Servlet->object;
    $User->set( { 'StartPageTab' => $Servlet->form->value('StartPageTab') } );
    return;
}

1;
