#==============================================================================
# package      DE_EPAGES::ShopConfiguration::API::Attributes::Store_Ext
# state        private
#==============================================================================
package DE_EPAGES::ShopConfiguration::API::Attributes::Store_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use Sort::Versions qw(versioncmp);
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Language qw(GetCodeByLanguageID);
use DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem;
use DE_EPAGES::ShopConfiguration::API::StoreCartridge qw(GetStoreCartridgeVersion);


#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Store, $aNames );
# example      # never call getAttributes directly!
#               $Value = $Store->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Returns attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Store | Store object | object
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Store, $aNames, $LanguageID ) = @_;

    my $hInfo =  $Store->get('Connection');
    return {'IsWebServiceAvailable' => 0 } unless $hInfo->{'active'};

    my $hValues  = {};
    eval {
        my $RemoteSystem = DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem->new( Store => $Store );
        my @Keys = qw (
            LoginFailureCount
            LoginFailureTimeLimit
            SessionTimeout
            DomainName
            WebServerPort
            HasSSLCertificate
            HasSSLCertificateUnity
            SSLWebServerPort
            WebServerScriptNamePart
            GUID
            );
        push @Keys, 'SessionIdleTimeout' if versioncmp(GetStoreCartridgeVersion($Store, 'DE_EPAGES::Permission'), '6.17.5') >= 0;

        my $StoreObjectCartridgeVersion = GetStoreCartridgeVersion($Store, 'DE_EPAGES::Object');

        #Flex compatibility
        if (versioncmp($StoreObjectCartridgeVersion, '6.17.44') >= 0 ){
            push @Keys, qw( UnityUrlScheme
                    UnityApiUrlScheme
                    UnityUseSubdomainRouting
                    UnitySSLWebServerPort
                    UnityWebServerPort
                    );
        }

        if (versioncmp(GetStoreCartridgeVersion($Store, 'DE_EPAGES::Shop'), '6.17.48') >= 0 ){
            push @Keys, qw( ShopSurveyShopMinAge
                    ShopSurveyShopMaxAge
                    ShopSurveyURL
                    ShopSurveyIntroText
                    ShopSurveyIsActive
                    ShopSurveyShowButtonBar
                    );
        }

        if (versioncmp($StoreObjectCartridgeVersion, '7.12.0') >= 0 ) {
            push @Keys, qw( HasSSLCertificateUnityAPI );
        }

        if (versioncmp($StoreObjectCartridgeVersion, '7.27.0') >= 0 ) {
            push @Keys, qw( UnityApiRoute );
        }

        $self->_filterKeys(\@Keys,$aNames);

        my $LanguageCode = $LanguageID ? GetCodeByLanguageID($LanguageID) : undef;

        $hValues = $RemoteSystem->get(\@Keys,$LanguageCode);

        # enforce hash
        $hValues = ref $hValues eq  'HASH' ? $hValues : {};

        $hValues->{'IsWebServiceAvailable'} = 1;
        $hValues->{'WebServerName'} = $hValues->{'DomainName'};
        $hValues->{'SSLWebServerName'} = $hValues->{'DomainName'};
    };
    if( ExistsError() ) {
        LogError( GetError() );
        $hValues->{'IsWebServiceAvailable'} = 0;
    }

    return $hValues;
}

#========================================================================================
# function     _filterKeys
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $self->_filterKeys(\@Keys,$aNames);
# example      $hValue =  $self->_filterKeys($aKeys,['DomainName']);
#----------------------------------------------------------------------------------------
# description  filters the attributes dependent from the attributes we want to get
#----------------------------------------------------------------------------------------
# input        $aKeys | list of RemoteShop Attributes                      | ref.array
# input        $aNames| list of attribute names                            | ref.array
# return       $aKeys | list of RemoteShop Attributes which should be get  | ref.array
#========================================================================================
sub _filterKeys {
    my $self = shift;
    my ($aKeys, $aNames ) = @_;

    foreach my $index (0 .. scalar @$aKeys) {
        delete $aKeys->[$index]
            unless $aKeys->[$index] ~~ $aNames;
    }

    # remove all undef values
    @$aKeys = grep {defined} @$aKeys;

    # we  need domain name for WebServerName and SSLWebServerName
    push(@$aKeys,'DomainName') if 'WebServerName'   ~~ $aNames;
    push(@$aKeys,'DomainName') if 'SSLWebServerName'~~ $aNames;

    return $aKeys;
}

sub setAttributes {
    shift;
    my ($Store, $hValues, $LanguageID) = @_;

    my $LanguageCode = $LanguageID ? GetCodeByLanguageID($LanguageID) : undef;

    delete $hValues->{'SessionIdleTimeout'} if versioncmp(GetStoreCartridgeVersion($Store, 'DE_EPAGES::Permission'), '6.17.5') < 0;

    #Flex compatibility
    my $StoreObjectCartridgeVersion = GetStoreCartridgeVersion($Store, 'DE_EPAGES::Object');

    if(versioncmp($StoreObjectCartridgeVersion, '6.17.44') < 0) {
        delete $hValues->{$_} foreach qw( UnityUrlScheme UnityApiUrlScheme UnityUseSubdomainRouting);
    };

    if(versioncmp($StoreObjectCartridgeVersion, '7.27.0') < 0) {
        delete $hValues->{$_} foreach qw( UnityApiRoute );
    };

    if(versioncmp(GetStoreCartridgeVersion($Store, 'DE_EPAGES::Shop'), '6.17.48') < 0) {
        delete $hValues->{$_} foreach ('ShopSurveyShopMinAge', 'ShopSurveyShopMaxAge', 'ShopSurveyURL', 'ShopSurveyIntroText', 'ShopSurveyIsActive', 'ShopSurveyShowButtonBar');
    };

    return unless scalar(keys %$hValues);

    my $RemoteSystem = DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem->new( Store => $Store );
    $RemoteSystem->set( $hValues, $LanguageCode);

    return;
}

1;
