#==============================================================================
# package      DE_EPAGES::ShopMailType::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::ShopMailType::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::TestParameter qw( TestEMailAddress );
use DE_EPAGES::MailType::API::NewPassword qw( SendNewPasswordToken );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Presentation::API::User qw( GetUserFolder );
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

#==============================================================================
# Event handlers
#==============================================================================

# Returns the ShopMailType object with the given Alias.
# Creates a new ShopMailType if it does not exist yet.
sub NewShopMailType {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    return GetShopMailType( $Shop, $Servlet->params('Alias') );
}

sub SaveEMailSettings {
    my $self = shift;
    my ($Servlet) = @_;

    # Check if email address is valid.
    my $Shop = $Servlet->object;
    $Servlet->form->form($Shop, 'EMailSettings');

    $self->SUPER::Save($Servlet);
    return;
}

sub SaveMailTypesStatus {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my %ActiveAliases = map { $_ => 1 } $Servlet->form->value('Alias');
    my $aShopMailTypes = LoadRootObject()->folder('ShopMailTypeTemplates')->children;

    $Shop->folder('MailTypes')->children;
    my %FormErrorAlias; # list of aliases with errors
    foreach my $MailTypeTemplate ( @$aShopMailTypes ) {
        my $Alias = $MailTypeTemplate->alias;
        if( $ActiveAliases{$Alias} ) {
            my $ShopMT = GetShopMailType( $Shop, $Alias );
            $ShopMT->set({'IsActive'=>1});
        }
        else {
            my $ShopMT = GetShopMailType( $Shop, $Alias, 1 );
            $ShopMT->set({'IsActive'=>0}) if defined $ShopMT;
        }
    }

    $Servlet->vars( 'FormErrorAlias', \%FormErrorAlias );
    $Servlet->form->executeFormError;

    return;
}

#========================================================================================
# function     _sendUserPassword
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_sendUserPassword($Servlet, $User, $FormEMail, $Shop);
# example      $self->_sendUserPassword($Servlet, $CurrentUser, $FormEMail, $Shop);
#----------------------------------------------------------------------------------------
# description  Used by the *SendUserNewPassword methods. First determines the correct
#               MailType. Then calls to send a new password to the supplied User using
#               the fetched MailType.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet | object
# input        $User | user to set the new password | object
# input        $FormEMail | email address to use as From field | string
# input        $Shop | shop object | object
#========================================================================================
sub _sendUserPassword {
    my $self = shift;
    my ($Servlet, $User, $FormEMail, $Shop) = @_;

    # Get mailtype.
    my $MailType = undef;
    my $From = undef;
    if ($Servlet->form->formValue('UseSystemMailType')) {
        $MailType = LoadObjectByPath('/MailTypes/NewPasswordMerchant');
        $From = $FormEMail if defined $MailType && $MailType->get('IsActive');
    }
    else {
        if ($Shop->get('IsUnityRegistered')) {
            $MailType = GetShopMailType( $Shop, 'UnityNewPassword', 1 );
        } else {
            $MailType = GetShopMailType( $Shop, 'NewPassword', 1 );
        }
        $From = $MailType->from if defined $MailType && $MailType->get('IsActive');
    }
    # No valid mailtype found - maybe the merchant has not set up his mailaddress yet?
    $Servlet->form->executeFormError({
        'Reason'=> 'SEND_MAIL_FAILED',
        'Form'  => 'NewPassword',
        'Name'  => 'EMail',
    }) unless defined $MailType && $MailType->get('IsActive');

    # Check if FROM is valid.
    unless( defined $From && $From =~ /\S/ ) {
        $Servlet->params( 'SEND_MAIL_FAILED', 1 );
        return;
    }

    if( SendNewPasswordToken( $Shop, $User, $MailType, {'FROM'=>$From} ) ) {
        $Servlet->params( 'PASSWORD_SEND_OK', 1 );
    }
    else {
        $Servlet->params( 'SEND_MAIL_FAILED', 1 );
    }
    return;
}

#========================================================================================
# function     SFSendUserNewPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SFSendUserNewPassword( $Servlet );
#----------------------------------------------------------------------------------------
# description  Password recovery for the STOREFRONT!
#               Searches for all non-anynymous users that use the given email address.
#               Then checks if there're any active users among them. After determining
#               the Shop's mail type, a new passwort is sent to EACH active user that
#               uses then given email address.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains email input value) | object
#========================================================================================
sub SFSendUserNewPassword {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hFormValues = $Servlet->form->form($Shop, 'NewMerchantPassword');
    my $FormEMail = $hFormValues->{'EMail'};

    my $DisableReCaptcha;
    $DisableReCaptcha = 1 if $Shop->get('IsUnityRegistered');

    $self->GoogleCaptcha($Servlet, $DisableReCaptcha);

    # Spam protection
    # Checks if honeypot form field 'SenderName' is set
    if($Servlet->form->value('SenderName')) {
        $Servlet->vars('ViewAction', 'View');
        return;
    }

    # No value is given.
    $Servlet->form->executeFormError({
        'Reason'=> 'USER_NO_EMAIL',
        'Form'  => 'NewPassword',
        'Name'  => 'EMail',
    }) if not defined $FormEMail or $FormEMail eq '';

    # Check if from has valid mail address format.
    eval {
        TestEMailAddress('FormMail', $FormEMail);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if( $Error->code eq 'FORMAT_NOT_EMAILADDRESS' ) {
            $Servlet->form->executeFormError({
                'Reason'=> 'FORMAT_NOT_EMAILADDRESS',
                'Form'  => 'NewPassword',
                'Name'  => 'EMail',
            });
        } else {
            Error($Error);
        }
    }

    # get all activated users and send mail only to the last created user
    my @AllActivatedUsers = sort { $b->get('CreationDate') <=> $a->get('CreationDate') } @{_getActivatedSFUsers($Servlet, $Shop, $FormEMail)};

    # Now send link to change password to each user.
    $self->_sendUserPassword($Servlet, $AllActivatedUsers[0], $FormEMail, $Shop);
    return;
}

sub _getActivatedSFUsers {
    my ($Servlet, $Shop, $FormEMail) = @_;
    # All non-anonymous users, that are no MBO users.
    my $UserClass = LoadClassByAlias('User');
    my @AllUsers = grep {
         $_->getSite()->id == $Shop->id  &&
        !$_->get('IsAnonymous') &&
        !$_->get('IsMerchant')
    } @{$UserClass->getObjects('ByEMail', [lc($FormEMail)])};

    # No user with that email.
    $Servlet->form->executeFormError({
        'Reason'=> 'EMAIL_MISMATCH',
        'Form'  => 'NewPassword',
        'Name'  => 'EMail',
    }) if scalar(@AllUsers) == 0;

    # Is there any activated user?
    my @AllActivatedUsers = grep { $_->get('IsActivated') } @AllUsers;

    if(scalar(@AllActivatedUsers) == 0) {
        # No activated user. Exit with error.
        $Servlet->form->executeFormError({
            'Reason'=> 'LOGIN_INACTIVE',
            'Form'  => 'NewPassword',
            'Name'  => 'EMail',
            'Value' => $FormEMail,
        });
        return;
    }
    return \@AllActivatedUsers;
}

#========================================================================================
# function     MBOSendUserNewPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->MBOSendUserNewPassword( $Servlet );
#----------------------------------------------------------------------------------------
# description  Password recovery for the MBO!
#               Searches for all non-anynymous users that use the given email address.
#               Then checks if there're any active users among them. After determining
#               the Shop's mail type, a new passwort is sent to EACH active user that
#               uses then given email address.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains email input value) | object
#========================================================================================
sub MBOSendUserNewPassword {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    my $UserFolder = GetUserFolder($Shop);
    unless ($UserFolder->existsChild($Form->value('Login'))) {
        $Servlet->form->addFormError({
            'Reason'=> 'LOGIN_NOT_FOUND',
            'Form'  => 'NewPassword',
            'Name'  => 'Login',
            'Value' => $Form->value('Login'),
        });
        $Servlet->form->addFormError({
            'Reason'=> 'LOGIN_NOT_FOUND',
            'Form'  => 'NewPassword',
            'Name'  => 'EMail',
            'Value' => $Form->value('Login'),
        });
        $Servlet->form->executeFormError;
    }

    my ($User, $UserEMail) = _checkUser($Servlet, $UserFolder, $Form->value('Login'), $Form->value('EMail'));
    $self->_sendUserPassword($Servlet, $User, $UserEMail, $Shop);

    return;
}

sub _checkUser {
    my ($Servlet, $UserFolder, $Login, $EMail) = @_;

    my $User = $UserFolder->child($Login);
    # Check if user is activated.
    $Servlet->form->executeFormError({
        'Reason'=> 'LOGIN_INACTIVE',
        'Form'  => 'NewPassword',
        'Name'  => 'Login',
        'Value' => $Login,
    }) unless $User->get('IsActivated');

    # Check if user has email.
    my $UserEMail = $User->get('EMail');
    $Servlet->form->executeFormError({
        'Reason'=> 'USER_NO_EMAIL',
        'Form'  => 'NewPassword',
        'Name'  => 'EMail',
    }) unless defined $UserEMail;

    unless ($EMail =~ /^$UserEMail$/i) {
        $Servlet->form->addFormError({
            'Reason'=> 'EMAIL_MISMATCH',
            'Form'  => 'NewPassword',
            'Name'  => 'Login',
        });
        $Servlet->form->addFormError({
            'Reason'=> 'EMAIL_MISMATCH',
            'Form'  => 'NewPassword',
            'Name'  => 'EMail',
        });
        $Servlet->form->executeFormError;
    }
    return ($User, $UserEMail);
}

sub ViewLostPassword {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->user;
    my $Shop = $Servlet->object->getSite();
    if ($Shop->get('IsUnityRegistered') && defined $User) {
        my $Basket = $User->get('Baskets')->[0];
        $Servlet->vars('BasketID', $User->get('Baskets')->[0]->id) if defined $Basket;
    } else {
        if (defined $User && !$User->get('IsAnonymous')) {
            # If user logged in , don't show password lost only for non Unity shops.
            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'ViewObjectID' => $User->id,
                },
                'ViewAction' => 'ViewMyAccount',
            });
        }
    }

    $Servlet->redirectToSecureConnection($Shop->get('WebServerScriptUrlSSL'));

    $self->View($Servlet);
    return;
}

1;
