#========================================================================================
# package      DE_EPAGES::WebService::API::Client
# state        public
#----------------------------------------------------------------------------------------
# description  Loads the SOAP::Lite module and installs a universal error handler. Based
#               on SOAP::Lite.
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $soap = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/HelloService/2004/04')
#                   ->proxy('http://login:invalid@servername/epages/Site.soap');
#               my $result = eval { $soap->protected->result; };
#               print GetError()->message if ExistsError();
#========================================================================================
package DE_EPAGES::WebService::API::Client;
use base qw ( SOAP::Lite );

use strict;
use URI;
use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(GetCommunicationLog);
use DE_EPAGES::Core::API::TestParameter qw( TestInteger TestString );
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::XML::API::Tidy qw(tidy);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'WebServiceFault' => 'WebService error: #FaultString (#FaultCode)',
        'WebServiceTransport' => 'WebService transport error: #StatusLine',
        'InvalidSOAPResponse' => 'Invalid SOAP response: #Content',
        'WrongLWPVersion' => 'A patched version of LWP::Protocol::http is required. '.
            'Current version: #Protocol_http_Version, '.
            'Required: #Protocol_http_Version_Required, '.
            'Perl version: #PerlVersion '.
            '(see also: http://rt.cpan.org/Public/Bug/Display.html?id=11333)',
        'WebServiceCallFailed' => 'Web service call failed (#Try. try): #Error',
    };
}

#ERRORDEFINITION END

sub on_fault {
    return \&SoapError;
};

sub SoapError {
    my $self = shift;
    my ($res) = @_;

    # remove user name and password from the URL for logging purposes
    my $EndpointURI = URI->new( $self->proxy->endpoint );
    $EndpointURI->userinfo(undef);

    if (ref $res) {
        if( $res->fault ) {
            # convert empty list () to undef because of error: odd number of elements in hash
            my $FaultCode = $res->faultcode;
            my $FaultString = $res->faultstring;
            my $FaultDetail = $res->faultdetail;
            my $FaultActor = $res->faultactor;
            Error('WebServiceFault', {
                'FaultCode'   => $FaultCode,
                'FaultString' => $FaultString,
                'FaultDetail' => $FaultDetail,
                'FaultActor'  => $FaultActor,
                'EndpointURI' => $EndpointURI,
            });
        }
        else {
            my $SoapSerializer = SOAP::Serializer->new;
            my $Content = $SoapSerializer->serialize( $res->dataof );

            Error('InvalidSOAPResponse', {
                'Content' => $Content,
                'SoapURI' => $self->uri,
                'EndpointURI' => $EndpointURI,
            });
        }
    }
    else {
        Error('WebServiceTransport', {
            'Code' => $self->transport->code,
            'Message' => $self->transport->message,
            'StatusLine' => $self->transport->status,
            'EndpointURI' => $EndpointURI,
            'Response' => $self->transport->http_response->as_string,
        });
    }

    return $res;
}

#========================================================================================
# function     userinfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = $Service->userinfo( "$Login:$Password" );
# syntax       $UserInfo = $Service->userinfo;
# example      $Service->userinfo( "wurzel:geheim" );
# example      my $Service = DE_EPAGES::WebService::API::Client->new
#                   ->proxy( 'http://localhost/epages/MyStore.soap' )
#                   ->userinfo( '/Users/wsadmin:wsadmin' );
#----------------------------------------------------------------------------------------
# description  Sets the login and password for Basic Authentication.
#----------------------------------------------------------------------------------------
# input        $Login | login | string
# input        $Password | password | string
# return       $Service | returns the web service object | object
# return       $UserInfo | login:password | string
#========================================================================================
sub userinfo {
    my $self = shift;
    my ($UserInfo) = @_;

    if( $UserInfo ) {
        # set the user info
        TestString( 'UserInfo', $UserInfo );

        my $URI = URI->new( $self->proxy->endpoint );
        $URI->userinfo( $UserInfo );
        $self->proxy->endpoint( $URI->as_string );
        return $self;
    }
    else {
        # get the user info
        return URI->new( $self->proxy->endpoint )->userinfo;
    }
}

sub port {
    my $self = shift;
    my ($Port) = @_;

    if( $Port ) {
        # set the port number
        TestInteger( 'Port', $Port );

        my $URI = URI->new( $self->proxy->endpoint );
        $URI->port( $Port );
        $self->proxy->endpoint( $URI->as_string );
    }
    else {
        # get the port number
        return URI->new( $self->proxy->endpoint )->port;
    }
}

#========================================================================================
# function     logCommunication
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service->logCommunication( $Category, $Code, $FilterSub );
# example      $Service->logCommunication( 'Ebay::RemoteEbay', sub {
#                   $Service->call( $method, $header, @params );
#               });
#----------------------------------------------------------------------------------------
# description  Logs all SOAP communication in the code block.
#----------------------------------------------------------------------------------------
# input        $Category | log category, undef for no logging | string
# input        $Sub | code to be executed with debug information | code ref
# input        $FilterSub | code to be executed to Filter data to be logged
#               (could be used to remove passwords from logging)
# return       $Code | executed code with debug information | code ref
#========================================================================================
sub logCommunication {
    shift;
    my ($Category, $Sub, $FilterSub) = @_;

    my $Logger = GetCommunicationLog($Category);
    $FilterSub = sub { $_[0] } unless defined $FilterSub;

    no warnings 'redefine'; ##no critic (ProhibitNoWarnings)
    local *SOAP::Trace::transport = defined $Category && $Logger->is_debug
        ? sub {
            my $Message = shift;
            if( $Message->isa('HTTP::Request') ) {
                #$Logger->debug( "Request headers:\n" . tidy($Message->headers_as_string) );
                $Logger->debug( $Message->method .' '. $Message->uri ."\n". tidy($FilterSub->($Message->content)) );
            }
            else {
                #$Logger->debug( "Response headers:\n" . tidy($Message->headers_as_string) );
                $Logger->debug( $Message->status_line . "\n" . tidy($FilterSub->($Message->content)) );
            }
        }
        : sub {};

    return &$Sub;
}

#========================================================================================
# function     call
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ret = $Service->call( $MethodName, @Params );
# example      $Service->logCommunication( 'Ebay::RemoteEbay', sub {
#                   $Service->call( $method, $header, @params );
#               });
#----------------------------------------------------------------------------------------
# description  Wraps the call() method of SOAP::Lite to re-try the request if certain
#               errors occur (see EPG-21811).
#----------------------------------------------------------------------------------------
# input        $MethodName | method name | string
# input        @Params | parameters | array
# return       $ret | return value, see SOAP::Lite | scalar
#========================================================================================
sub call { ## no critic qw( RequireCommentForExportedFunctions RequireArgUnpacking )
    my $self = shift;

    my $MaxTries = 3;
    my $Tries = 0;
    do {
        my $ret = eval { $self->SUPER::call(@_); };
        return $ret unless ExistsError();

        my $Error = GetError();
        Error( $Error ) unless $Error->code eq 'NativeDie' && $Error->shortMessage =~ /
            ^
            (?:
               connect:\ Illegal\ seek
             | read\ failed:\ Illegal\ seek
             | select\ failed:\ Permission\ denied
            )
            /x;

        $Tries++;
        if( $Tries >= $MaxTries ) {
            $Error->addVars({ Try => $Tries });
            Error($Error);
        }

        Warning('WebServiceCallFailed', {
            Error => $Error->shortMessage,
            Try => $Tries,
        });
    }
    while ( 1 );
}

#todo: implement proxy() within:
    # no hostname check in SSL cert unless VerifyHostname to prevent error:
    # 500 Can't connect to <hostname>:443 (certificate verify failed)
#    my $VerifyHostname = EpagesConfig()->get('DE_EPAGES::Store', 'WebServiceVerifyHostname', 1);
#    $self->transport->ssl_opts( 'verify_hostname' => 0 )
#        if !$VerifyHostname && 'SOAP::Transport::HTTP::Client'->can('ssl_opts');


1;
